@@ -18,22 +18,31 @@
 #ifndef DIALOGLAUNCHER_H
 #define DIALOGLAUNCHER_H
 #include <qstring.h>
+#include "kst_export.h"
+
+class QWidget;
 
 namespace KstDialogs {
-  void showHistogramDialog(const QString& name);
+  KST_EXPORT void showHistogramDialog(const QString& name);
 
-  void showPluginDialog(const QString& name);
+  KST_EXPORT void showPluginDialog(const QString& name);
 
-  void showEquationDialog(const QString& name);
+  KST_EXPORT void showEquationDialog(const QString& name);
+  
+  KST_EXPORT void showCSDDialog(const QString& name);
 
-  void showPSDDialog(const QString& name);
+  KST_EXPORT void showPSDDialog(const QString& name);
 
-  void showMatrixDialog(const QString& name);
+  KST_EXPORT void newMatrixDialog(QWidget *parent, const char *createdSlot = 0L, const char *selectedSlot = 0L, const char *updateSlot = 0L);
+  KST_EXPORT void showMatrixDialog(const QString& name);
 
-  void showImageDialog(const QString& name);
+  KST_EXPORT void showImageDialog(const QString& name);
 
-  void showCurveDialog(const QString& name);
-};
+  KST_EXPORT void showCurveDialog(const QString& name);
+  
+  KST_EXPORT void newVectorDialog(QWidget *parent, const char *createdSlot = 0L, const char *selectedSlot = 0L, const char *updateSlot = 0L);
+  KST_EXPORT void showVectorDialog(const QString& name);
+}
 #endif
 
 // vim: ts=2 sw=2 et
