@@ -13,8 +13,8 @@
  *
  *  You should have received a copy of the GNU Library General Public License
  *  along with this library; see the file COPYING.LIB.  If not, write to
- *  the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- *  Boston, MA 02111-1307, USA.
+ *  the Free Software Foundation, Inc., 51 Franklin Steet, Fifth Floor,
+ *  Boston, MA 02110-1301, USA.
  */
 
 #include <qobject.h>
@@ -128,7 +128,7 @@ JSObjectProxyImp::JSObjectProxyImp( KJS::ExecState *exec, int mid, JSObjectProxy
 {
 }
 
-JSObjectProxyImp::JSObjectProxyImp( KJS::ExecState *exec, 
+JSObjectProxyImp::JSObjectProxyImp( KJS::ExecState *exec,
 				    int mid, const QCString &name, JSObjectProxy *parent )
     : JSProxyImp(exec), id(mid), slotname(name), proxy(parent), obj(parent->obj)
 {
@@ -141,7 +141,7 @@ JSObjectProxyImp::JSObjectProxyImp( KJS::ExecState *exec,
 }
 
 JSObjectProxyImp::JSObjectProxyImp( KJS::ExecState *exec,
-				    int mid, const char *ret, int sid, const QCString &name, 
+				    int mid, const char *ret, int sid, const QCString &name,
 				    JSObjectProxy *parent )
     : JSProxyImp(exec), id(mid), rettype(ret), sigid(sid), slotname(name), proxy(parent), obj(parent->obj)
 {
@@ -343,7 +343,7 @@ KJS::Boolean JSObjectProxyImp::connect( KJS::ExecState *exec,
     JSObjectProxy *sendproxy = JSProxy::toObjectProxy( args[0].imp() );
     QObject *sender = sendproxy ? sendproxy->object() : 0;
     QString sig = args[1].toString(exec).qstring();
-    kdDebug( 80001 ) << "connecting " << sig << endl;
+    kdDebug( 80001 ) << "connecting C++ signal" << sig << endl;
     // Receiver and slot/signal
     KJS::Object recvObj;
     QString dest;
@@ -444,7 +444,7 @@ KJS::Boolean JSObjectProxyImp::connect( QObject *sender, const char *sig,
 	case JSSlotUtils::SignatureIntIntIntInt:
 	    ok = JSSlotUtils::connect( sender, sig, slotp, "slot_intintint(int, int, int)" );
 	    break;
-	
+
 	case JSSlotUtils::SignatureDate:
 	    ok = JSSlotUtils::connect( sender, sig, slotp, "slot_date(const QDate&)" );
 	    break;
@@ -474,7 +474,7 @@ KJS::Boolean JSObjectProxyImp::connect( QObject *sender, const char *sig,
 			.remove('&')
 			.remove('*');
 		mangledSig = mangledSig.lower();
-		
+
 		kdDebug(80001) << "Custom slot signature: " << mangledSig << endl;
 		break;
 	}
@@ -591,12 +591,19 @@ KJS::Value JSObjectProxyImp::hasAttribute( KJS::ExecState *exec, KJS::Object &,
 KJS::Value JSObjectProxyImp::getAttribute( KJS::ExecState *exec, KJS::Object &, const KJS::List &args )
 {
     if ( !args.size() )
-	return KJS::Null();
+    {
+        QString msg = i18n( "No property was defined." );
+        return throwError(exec, msg);
+    }
 
     QMetaObject *meta = obj->metaObject();
     QString s = args[0].toString(exec).qstring();
+    kdDebug(80001) << "Get property " << s << " from " << obj->name() << endl;
     if ( meta->findProperty( s.ascii(), true ) == -1 )
-	return KJS::Null();
+    {
+        QString msg = i18n( "Property '%1' could not be found." ).arg( s );
+        return throwError(exec, msg,KJS::ReferenceError);
+    }
 
     QVariant val = obj->property( s.ascii() );
     return convertToValue( exec, val );
@@ -611,8 +618,11 @@ KJS::Value JSObjectProxyImp::setAttribute( KJS::ExecState *exec, KJS::Object &,
     QString s = args[0].toString(exec).qstring();
 
     if ( meta->findProperty( s.ascii(), true ) == -1 )
-	return KJS::Boolean(false);
-
+    {
+        QString msg = i18n( "Property '%1' could not be found." ).arg( s );
+        return throwError(exec, msg,KJS::GeneralError);
+    }
+    kdDebug(80001) << "Set property " << s << " from " << obj->name() << endl;
     QVariant val = convertToVariant( exec, args[1] );
     bool ok = obj->setProperty( s.ascii(), val );
 
