@@ -21,13 +21,15 @@
 #include <qptrstack.h>
 
 // include files for KDE
-#include <kdebug.h>
+#include "ksdebug.h"
 #include <klistview.h>
 #include <kmessagebox.h>
+#include <kstandarddirs.h>
 
 // application specific includes
 #include "kst2dplot.h"
 #include "kstcurvedialog_i.h"
+#include "kstcsddialog_i.h"
 #include "kstdatacollection.h"
 #include "kstdatamanager_i.h"
 #include "kstdoc.h"
@@ -45,34 +47,52 @@
 
 
 static QMap<int,Kst2DPlotPtr> PlotMap;
-static QMap<int,QString> FilterMap;
 
 #define RTTI_OBJ_VECTOR          4201
 #define RTTI_OBJ_OBJECT          4202
 #define RTTI_OBJ_DATA_VECTOR     4203
-
+#define RTTI_OBJ_STATIC_VECTOR   4204
+#define RTTI_OBJ_MATRIX          4205
+#define RTTI_OBJ_DATA_MATRIX     4206
+#define RTTI_OBJ_STATIC_MATRIX   4207
 
 KstObjectItem::KstObjectItem(QListView *parent, KstRVectorPtr x, KstDataManagerI *dm, int localUseCount)
 : QObject(), QListViewItem(parent), _rtti(RTTI_OBJ_DATA_VECTOR), _name(x->tagName()), _dm(dm) {
   assert(x);
+  _inUse = false;
   setText(0, x->tagName());
   setText(1, i18n("Data Vector"));
   x = 0L; // keep the counts in sync
   update(true, localUseCount);
 }
 
+
+KstObjectItem::KstObjectItem(QListView *parent, KstSVectorPtr x, KstDataManagerI *dm, int localUseCount)
+: QObject(), QListViewItem(parent), _rtti(RTTI_OBJ_STATIC_VECTOR), _name(x->tagName()), _dm(dm) {
+  assert(x);
+  _inUse = false;
+  setText(0, x->tagName());
+  setText(1, i18n("Static Vector"));
+  x = 0L; // keep the counts in sync
+  update(true, localUseCount);
+}
+
+
 KstObjectItem::KstObjectItem(QListViewItem *parent, KstVectorPtr x, KstDataManagerI *dm, int localUseCount)
 : QObject(), QListViewItem(parent), _rtti(RTTI_OBJ_VECTOR), _name(x->tagName()), _dm(dm) {
   assert(x);
+  _inUse = false;
   setText(0, x->tagName());
   setText(1, i18n("Slave Vector"));
   x = 0L; // keep the counts in sync
   update(true, localUseCount);
 }
 
+
 KstObjectItem::KstObjectItem(QListView *parent, KstDataObjectPtr x, KstDataManagerI *dm, int localUseCount)
 : QObject(), QListViewItem(parent), _rtti(RTTI_OBJ_OBJECT), _name(x->tagName()), _dm(dm) {
   assert(x);
+  _inUse = false;
   setText(0, x->tagName());
   for (KstVectorMap::Iterator i = x->outputVectors().begin();
       i != x->outputVectors().end();
@@ -80,159 +100,356 @@ KstObjectItem::KstObjectItem(QListView *parent, KstDataObjectPtr x, KstDataManag
     KstObjectItem *item = new KstObjectItem(this, i.data(), _dm);
     connect(item, SIGNAL(updated()), this, SIGNAL(updated()));
   }
+  for (KstMatrixMap::Iterator i = x->outputMatrices().begin();
+       i != x->outputMatrices().end();
+       ++i) {
+    KstObjectItem *item = new KstObjectItem(this, i.data(), _dm);
+    connect(item, SIGNAL(updated()), this, SIGNAL(updated()));       
+  }
   x = 0L; // keep the counts in sync
   update(false, localUseCount);
 }
 
+
+KstObjectItem::KstObjectItem(QListView *parent, KstRMatrixPtr x, KstDataManagerI *dm, int localUseCount) 
+: QObject(), QListViewItem(parent), _rtti(RTTI_OBJ_DATA_MATRIX), _name(x->tagName()), _dm(dm) {
+  assert(x);
+  _inUse = false;
+  setText(0, x->tagName());
+  setText(1, i18n("Data Matrix"));
+  x = 0L; // keep the counts in sync
+  update(true, localUseCount);
+}
+
+
+KstObjectItem::KstObjectItem(QListView *parent, KstSMatrixPtr x, KstDataManagerI *dm, int localUseCount) 
+: QObject(), QListViewItem(parent), _rtti(RTTI_OBJ_STATIC_MATRIX), _name(x->tagName()), _dm(dm) {
+  assert(x);
+  _inUse = false;
+  setText(0, x->tagName());
+  setText(1, i18n("Static Matrix"));
+  x = 0L; // keep the counts in sync
+  update(true, localUseCount);
+}
+
+
+KstObjectItem::KstObjectItem(QListViewItem *parent, KstMatrixPtr x, KstDataManagerI *dm, int localUseCount) 
+: QObject(), QListViewItem(parent), _rtti(RTTI_OBJ_MATRIX), _name(x->tagName()), _dm(dm) {
+  assert(x);
+  _inUse = false;
+  setText(0, x->tagName());
+  setText(1, i18n("Slave Matrix"));
+  x = 0L; // keep the counts in sync
+  update(true, localUseCount);
+}
+
+
 KstObjectItem::~KstObjectItem() {
 }
 
+
 void KstObjectItem::update(bool recursive, int localUseCount) {
   switch (_rtti) {
     case RTTI_OBJ_DATA_VECTOR:
-      {
-        KstRVectorPtr x = kst_cast<KstRVector>(*KST::vectorList.findTag(_name));
-        if (x) {
-          x->readLock();
-          // getUsage: subtract 1 for KstRVectorPtr x
-          setText(2, QString::number(x->getUsage() - 1 - localUseCount));
-          setText(3, QString::number(x->sampleCount()));
-          setText(4, i18n("%3: %4 [%1..%2]").arg(x->reqStartFrame())
-              .arg(x->reqStartFrame() + x->reqNumFrames())
-              .arg(x->filename())
-              .arg(x->field()));
-          _removable = x->getUsage() == 2;
-          x->readUnlock();
+    {
+      KstRVectorPtr x = kst_cast<KstRVector>(*KST::vectorList.findTag(_name));
+      if (x) {
+        x->readLock();
+        // getUsage: subtract 1 for KstRVectorPtr x
+        bool inUse = (x->getUsage() - 1 - localUseCount) > 0;
+        if (inUse != _inUse) {
+          _inUse = inUse;
+          setPixmap(2, inUse ? _dm->yesPixmap() : QPixmap());
         }
-        // Hmmm what happens if this if() fails??  We become inconsistent?
-        break;
+        QString field = QString::number(x->length());
+        if (text(3) != field) {
+          setText(3, field);
+        }
+        field = i18n("%3: %4 [%1..%2]").arg(x->reqStartFrame())
+            .arg(x->reqStartFrame() + x->reqNumFrames())
+            .arg(x->filename())
+            .arg(x->field());
+        if (text(4) != field) {
+          setText(4, field);
+        }
+        _removable = x->getUsage() == 2;
+        x->readUnlock();
       }
+      // Hmmm what happens if this if() fails??  We become inconsistent?
+      break;
+    }
+    case RTTI_OBJ_STATIC_VECTOR:
+    {
+      KstSVectorPtr x = kst_cast<KstSVector>(*KST::vectorList.findTag(_name));
+      if (x) {
+        x->readLock();
+        // getUsage: subtract 1 for KstRVectorPtr x
+        bool inUse = (x->getUsage() - 1 - localUseCount) > 0;
+        if (inUse != _inUse) {
+          _inUse = inUse;
+          setPixmap(2, inUse ? _dm->yesPixmap() : QPixmap());
+        }
+        QString field = QString::number(x->length());
+        if (text(3) != field) {
+          setText(3, field);
+        }
+        field = i18n("%1 to %2").arg(x->min()).arg(x->max());
+        if (text(4) != field) {
+          setText(4, field);
+        }
+        _removable = x->getUsage() == 2;
+        x->readUnlock();
+      }
+      // Hmmm what happens if this if() fails??  We become inconsistent?
+      break;
+    }
     case RTTI_OBJ_VECTOR:
-      {
-        KstVectorPtr x = *KST::vectorList.findTag(_name);
-        if (x) {
-          x->readLock();
-          //
-          // getUsage:
-          //  subtract 1 for KstVectorPtr x
-          //
-          setText(2, QString::number(x->getUsage() - 1 - localUseCount));
-          setText(3, QString::number(x->sampleCount()));
-          setText(4, i18n("[%1..%2]").arg(x->min()).arg(x->max()));
-          x->readUnlock();
-          _removable = false;
+    {
+      KstVectorPtr x = *KST::vectorList.findTag(_name);
+      if (x) {
+        x->readLock();
+        // getUsage:
+        //  subtract 1 for KstVectorPtr x
+        bool inUse = (x->getUsage() - 1 - localUseCount) > 0;
+        if (inUse != _inUse) {
+          _inUse = inUse;
+          setPixmap(2, inUse ? _dm->yesPixmap() : QPixmap());
         }
-        break;
+        QString field = QString::number(x->length());
+        if (text(3) != field) {
+          setText(3, field);
+        }
+        field = i18n("[%1..%2]").arg(x->min()).arg(x->max());
+        if (text(4) != field) {
+          setText(4, field);
+        }
+        x->readUnlock();
+        _removable = false;
       }
+      break;
+    }
     case RTTI_OBJ_OBJECT:
-      {
-        KstDataObjectPtr x = *KST::dataObjectList.findTag(_name);
-        if (x) {
-          x->readLock();
-          //
-          // getUsage:
-          //  subtract 1 for KstDataObjectPtr x
-          //
-          setText(1, x->typeString());
-          setText(2, QString::number(x->getUsage() - 1 - localUseCount));
-          setText(3, QString::number(x->sampleCount()));
-          setText(4, x->propertyString());
-          if (recursive) {
-            QPtrStack<QListViewItem> trash;
+    {
+      KstDataObjectPtr x = *KST::dataObjectList.findTag(_name);
+      if (x) {
+        x->readLock();
+        QString field = x->typeString();
+        if (text(1) != field) {
+          setText(1, field);
+        }
+        // getUsage:
+        //  subtract 1 for KstDataObjectPtr x
+        bool inUse = (x->getUsage() - 1 - localUseCount) > 0;
+        if (inUse != _inUse) {
+          _inUse = inUse;
+          setPixmap(2, inUse ? _dm->yesPixmap() : QPixmap());
+        }
+        field = QString::number(x->sampleCount());
+        if (text(3) != field) {
+          setText(3, field);
+        }
+        field = x->propertyString();
+        if (text(4) != field) {
+          setText(4, field);
+        }
+        if (recursive) {
+          QPtrStack<QListViewItem> trash;
+          KstVectorMap vl = x->outputVectors();
+          KstVectorMap::Iterator vlEnd = vl.end();
 
+          for (QListViewItem *i = firstChild(); i; i = i->nextSibling()) {
+            KstObjectItem *oi = static_cast<KstObjectItem*>(i);
+            if (vl.findTag(oi->tagName()) == vlEnd) {
+              trash.push(i);
+            }
+          }
+          trash.setAutoDelete(true);
+          trash.clear();
+
+          // get the output vectors
+          for (KstVectorMap::Iterator p = vl.begin(); p != vlEnd; ++p) {
+            bool found = false;
+            QString tn = p.data()->tagName();
             for (QListViewItem *i = firstChild(); i; i = i->nextSibling()) {
               KstObjectItem *oi = static_cast<KstObjectItem*>(i);
-              if (x->outputVectors().findTag(oi->tagName()) == x->outputVectors().end()) {
-                trash.push(i);
+              if (oi->tagName() == tn) {
+                oi->update();
+                found = true;
+                break;
               }
             }
-            trash.setAutoDelete(true);
-            trash.clear();
-
-            for (KstVectorMap::Iterator p = x->outputVectors().begin();
-                p != x->outputVectors().end(); ++p) {
-              bool found = false;
-              for (QListViewItem *i = firstChild(); i; i = i->nextSibling()) {
-                KstObjectItem *oi = static_cast<KstObjectItem*>(i);
-                if (oi->tagName() == p.data()->tagName()) {
-                  oi->update();
-                  found = true;
-                  break;
-                }
+            if (!found) {
+              KstObjectItem *item = new KstObjectItem(this, p.data(), _dm);
+              connect(item, SIGNAL(updated()), this, SIGNAL(updated()));
+            }
+          }
+          
+          KstMatrixMap ml = x->outputMatrices();
+          KstMatrixMap::Iterator mlEnd = ml.end();
+          // also get the output matrices
+          for (KstMatrixMap::Iterator p = ml.begin(); p != mlEnd; ++p) {
+            bool found = false;
+            QString tn = p.data()->tagName();
+            for (QListViewItem *i = firstChild(); i; i = i->nextSibling()) {
+              KstObjectItem *oi = static_cast<KstObjectItem*>(i);
+              if (oi->tagName() == tn) {
+                oi->update();
+                found = true;
+                break;
               }
-              if (!found) {
-                KstObjectItem *item = new KstObjectItem(this, p.data(), _dm);
-                connect(item, SIGNAL(updated()), this, SIGNAL(updated()));
-              }
+            }
+            if (!found) {
+              KstObjectItem *item = new KstObjectItem(this, p.data(), _dm);
+              connect(item, SIGNAL(updated()), this, SIGNAL(updated()));
             }
           }
-          _removable = x->getUsage() == 1;
-          x->readUnlock();
         }
-        break;
+        _removable = x->getUsage() == 1;
+        x->readUnlock();
       }
+      break;
+    }
+    case RTTI_OBJ_DATA_MATRIX:
+    {
+      KstRMatrixPtr x = kst_cast<KstRMatrix>(*KST::matrixList.findTag(_name));
+      if (x) {
+        x->readLock();
+          // getUsage: subtract 1 for KstRMatrixPtr x
+        bool inUse = (x->getUsage() - 1 - localUseCount) > 0;
+        if (inUse != _inUse) {
+          _inUse = inUse;
+          setPixmap(2, inUse ? _dm->yesPixmap() : QPixmap());
+        }
+        QString field = QString::number(x->sampleCount());
+        if (text(3) != field) {
+          setText(3, field);
+        }
+        field = i18n("%1: %2 (%3 by %4)").arg(x->filename()).arg(x->field())
+                .arg(x->xNumSteps())
+                .arg(x->yNumSteps()); 
+        if (text(4) != field) {
+          setText(4, field);
+        }
+        _removable = x->getUsage() == 2;
+        x->readUnlock();
+      } 
+      break;
+    }
+    case RTTI_OBJ_STATIC_MATRIX:
+    {
+      KstSMatrixPtr x = kst_cast<KstSMatrix>(*KST::matrixList.findTag(_name));
+      if (x) {
+        x->readLock();
+          // getUsage: subtract 1 for KstRMatrixPtr x
+        bool inUse = (x->getUsage() - 1 - localUseCount) > 0;
+        if (inUse != _inUse) {
+          _inUse = inUse;
+          setPixmap(2, inUse ? _dm->yesPixmap() : QPixmap());
+        }
+        QString field = QString::number(x->sampleCount());
+        if (text(3) != field) {
+          setText(3, field);
+        }
+        field = i18n("%1 to %2").arg(x->gradZMin()).arg(x->gradZMax());
+        if (text(4) != field) {
+          setText(4, field);
+        }
+        _removable = x->getUsage() == 2;
+        x->readUnlock();
+      }
+      break;
+    }
+    case RTTI_OBJ_MATRIX:
+    {
+      KstMatrixPtr x = *KST::matrixList.findTag(_name);
+      if (x) {
+        x->readLock();
+          // getUsage:
+          //  subtract 1 for KstVectorPtr x
+        bool inUse = (x->getUsage() - 1 - localUseCount) > 0;
+        if (inUse != _inUse) {
+          _inUse = inUse;
+          setPixmap(2, inUse ? _dm->yesPixmap() : QPixmap());
+        }
+        QString field = QString::number(x->sampleCount());
+        if (text(3) != field) {
+          setText(3, field);
+        }
+        field = i18n("[%1..%2]").arg(x->minValue()).arg(x->maxValue());
+        if (text(4) != field) {
+          setText(4, field);
+        }
+        x->readUnlock();
+        _removable = false;
+      }
+      break;
+    }
     default:
       assert(0);
   }
 }
 
+
 void KstObjectItem::updateButtons() {
-  _dm->Edit->setEnabled(RTTI_OBJ_VECTOR != _rtti);
-  _dm->Delete->setEnabled(RTTI_OBJ_VECTOR != _rtti);
+  _dm->Edit->setEnabled(RTTI_OBJ_VECTOR != _rtti && RTTI_OBJ_MATRIX != _rtti);
+  _dm->Delete->setEnabled(RTTI_OBJ_VECTOR != _rtti && RTTI_OBJ_MATRIX != _rtti);
 }
 
+
 void KstObjectItem::reload() {
-  KstReadLocker ml(&KST::vectorList.lock());
-  KstVectorList::Iterator v = KST::vectorList.findTag(_name);
-  if (v != KST::vectorList.end()) {
-    KstRVectorPtr r = kst_cast<KstRVector>(*v);
-    if (r) {
-      r->writeLock();
-      r->reload();
-      r->writeUnlock();
+  if (_rtti == RTTI_OBJ_DATA_VECTOR) {
+    KstReadLocker ml(&KST::vectorList.lock());
+    KstVectorList::Iterator v = KST::vectorList.findTag(_name);
+    if (v != KST::vectorList.end()) {
+      KstRVectorPtr r = kst_cast<KstRVector>(*v);
+      if (r) {
+        r->writeLock();
+        r->reload();
+        r->writeUnlock();
+      }
+    }
+  } else if (_rtti == RTTI_OBJ_DATA_MATRIX) {
+    KstReadLocker ml(&KST::matrixList.lock());
+    KstMatrixList::Iterator m = KST::matrixList.findTag(_name);
+    if (m != KST::matrixList.end()) {
+      KstRMatrixPtr r = kst_cast<KstRMatrix>(*m);
+      if (r) {
+        r->writeLock();
+        r->reload();
+        r->writeUnlock();
+      }
     }
   }
 }
 
+
 void KstObjectItem::makeCurve() {
-  KstCurveDialogI::globalInstance()->show_New();
-  KstCurveDialogI::globalInstance()->_yVector->setSelection(_name);
+  KstCurveDialogI::globalInstance()->show();
+  KstCurveDialogI::globalInstance()->setVector(_name);
+}
+
+void KstObjectItem::makeCSD() {
+  KstCsdDialogI::globalInstance()->show();
+  KstCsdDialogI::globalInstance()->setVector(_name);
 }
 
 void KstObjectItem::makePSD() {
-  KstPsdDialogI::globalInstance()->show_New();
-  KstPsdDialogI::globalInstance()->_vector->setSelection(_name);
+  KstPsdDialogI::globalInstance()->show();
+  KstPsdDialogI::globalInstance()->setVector(_name);
 }
 
+
 void KstObjectItem::makeHistogram() {
-  KstHsDialogI::globalInstance()->show_New();
-  KstHsDialogI::globalInstance()->_vector->setSelection(_name);
+  KstHsDialogI::globalInstance()->show();
+  KstHsDialogI::globalInstance()->setVector(_name);
 }
 
-void KstObjectItem::makeMatrix() {
-  KstMatrixDialogI::globalInstance()->show_New();
-  KstMatrixDialogI::globalInstance()->_zVector->setSelection(_name);
-}
 
 void KstObjectItem::makeImage() {
-  KstImageDialogI::globalInstance()->show_New();
-  KstImageDialogI::globalInstance()->_matrix->setSelection(_name);
+  KstImageDialogI::globalInstance()->show();
+  KstImageDialogI::globalInstance()->setMatrix(_name);
 }
 
-void KstObjectItem::applyFilter(int id) {
-/* Replace with new filtering logic
-  KST::filterSetList.lock().readLock();
-  KstFilterSetPtr p = KST::filterSetList.find(FilterMap[id]);
-  KST::filterSetList.lock().readUnlock();
-  KST::vectorList.lock().readLock();
-  KstVectorPtr v = *KST::vectorList.findTag(_name);
-  KST::vectorList.lock().readUnlock();
-  if (p && v) {
-    KstFilteredVectorPtr fv = new KstFilteredVector(v, p);
-    emit updated();
-  }
-*/
-}
 
 void KstObjectItem::activateHint(int id) {
   KstDataObjectPtr d = dataObject();
@@ -240,21 +457,23 @@ void KstObjectItem::activateHint(int id) {
   int cnt = 0;
   for (KstCurveHintList::ConstIterator i = hints->begin(); i != hints->end(); ++i) {
     if (cnt == id) {
-      KstBaseCurvePtr c = (*i)->makeCurve(d->tagName() + "-curve" /* FIXME */, KstColorSequence::next());
+      KstBaseCurvePtr c = (*i)->makeCurve(KST::suggestCurveName(d->tagName(), false),
+          KstColorSequence::next());
       if (c) {
         KST::dataObjectList.lock().writeLock();
         KST::dataObjectList.append(c.data());
         KST::dataObjectList.lock().writeUnlock();
         emit updated();
       } else {
-        KMessageBox::sorry(0L, i18n("Unable to create quick curve."));
+        KMessageBox::sorry(KstApp::inst(), i18n("Unable to create quick curve."));
       }
       break;
     }
-    cnt++;
+    ++cnt;
   }
 }
 
+
 void KstObjectItem::addToPlot(int id) {
   Kst2DPlotPtr p = PlotMap[id];
   KstBaseCurvePtr c = kst_cast<KstBaseCurve>(dataObject());
@@ -266,6 +485,7 @@ void KstObjectItem::addToPlot(int id) {
   }
 }
 
+
 void KstObjectItem::removeFromPlot(int id) {
   Kst2DPlotPtr p = PlotMap[id];
   KstBaseCurvePtr c = kst_cast<KstBaseCurve>(dataObject());
@@ -277,34 +497,13 @@ void KstObjectItem::removeFromPlot(int id) {
   }
 }
 
-void KstObjectItem::addImageToPlot(int id) {
-  Kst2DPlotPtr p = PlotMap[id];
-  KstImagePtr img = kst_cast<KstImage>(dataObject());
-  if (p && img) {
-    p->addImage(img);
-    p->setDirty();
-    paintPlot(p);
-    emit updated();
-  }
-}
-
-void KstObjectItem::removeImageFromPlot(int id) {
-  Kst2DPlotPtr p = PlotMap[id];
-  KstImagePtr img = kst_cast<KstImage>(dataObject());
-  if (p && img) {
-    p->removeImage(img);
-    p->setDirty();
-    paintPlot(p);
-    emit updated();
-  }
-}
 
 void KstObjectItem::paintPlot(Kst2DPlotPtr p) {
   KstApp *app = KstApp::inst();
   KMdiIterator<KMdiChildView*> *it = app->createIterator();
   while (it->currentItem()) {
     KstViewWindow *v = dynamic_cast<KstViewWindow*>(it->currentItem());
-    if (v && v->view()->contains(static_cast<KstViewObject*>(p))) {
+    if (v && v->view()->contains(kst_cast<KstViewObject>(p))) {
       v->view()->paint(P_PLOT);
       break;
     }
@@ -313,47 +512,45 @@ void KstObjectItem::paintPlot(Kst2DPlotPtr p) {
   app->deleteIterator(it);
 }
 
+
+const QPixmap& KstDataManagerI::yesPixmap() const {
+  return _yesPixmap;
+}
+
+
 KstDataManagerI::KstDataManagerI(KstDoc *in_doc, QWidget* parent, const char* name, bool modal, WFlags fl)
 : KstDataManager(parent, name, modal, fl) {
-    doc = in_doc;
+  doc = in_doc;
 
-    connect(Edit, SIGNAL(clicked()), this, SLOT(edit_I()));
-    connect(Delete, SIGNAL(clicked()), this, SLOT(delete_I()));
-    connect(Purge, SIGNAL(clicked()), doc, SLOT(purge()));
-    connect(DataView, SIGNAL(doubleClicked(QListViewItem *)),
-            this, SLOT(edit_I()));
-    connect(DataView, SIGNAL(currentChanged(QListViewItem *)),
-            this, SLOT(currentChanged(QListViewItem *)));
-    connect(DataView, SIGNAL(selectionChanged(QListViewItem *)),
-            this, SLOT(currentChanged(QListViewItem *)));
+  _yesPixmap = QPixmap(locate("data", "kst/pics/yes.png"));
 
-    connect(NewCurve, SIGNAL(clicked()),
-            KstCurveDialogI::globalInstance(), SLOT(show_New()));
-    connect(NewHs, SIGNAL(clicked()),
-            KstHsDialogI::globalInstance(), SLOT(show_New()));
-    connect(NewEq, SIGNAL(clicked()),
-            KstEqDialogI::globalInstance(), SLOT(show_New()));
-    connect(NewPlugin, SIGNAL(clicked()),
-            KstPluginDialogI::globalInstance(), SLOT(show_New()));
-    connect(NewEvent, SIGNAL(clicked()),
-            KstEventMonitorI::globalInstance(), SLOT(show_New()));
-    connect(NewPSD, SIGNAL(clicked()),
-            KstPsdDialogI::globalInstance(), SLOT(show_New()));
-    connect(NewVector, SIGNAL(clicked()),
-            KstVectorDialogI::globalInstance(), SLOT(show_New()));
-    connect(OpenPlotDialog, SIGNAL(clicked()),
-            KstApp::inst(), SLOT(showPlotDialog()));
+  connect(Edit, SIGNAL(clicked()), this, SLOT(edit_I()));
+  connect(Delete, SIGNAL(clicked()), this, SLOT(delete_I()));
+  connect(Purge, SIGNAL(clicked()), doc, SLOT(purge()));
+  connect(DataView, SIGNAL(doubleClicked(QListViewItem *)),
+      this, SLOT(edit_I()));
+  connect(DataView, SIGNAL(currentChanged(QListViewItem *)),
+      this, SLOT(currentChanged(QListViewItem *)));
+  connect(DataView, SIGNAL(selectionChanged(QListViewItem *)),
+      this, SLOT(currentChanged(QListViewItem *)));
 
-    connect(NewImage, SIGNAL(clicked()),
-            KstImageDialogI::globalInstance(), SLOT(show_New()));
+  connect(NewCurve, SIGNAL(clicked()), KstCurveDialogI::globalInstance(), SLOT(show()));
+  connect(NewHs, SIGNAL(clicked()), KstHsDialogI::globalInstance(), SLOT(show()));
+  connect(NewEq, SIGNAL(clicked()), KstEqDialogI::globalInstance(), SLOT(show()));
+  connect(NewPlugin, SIGNAL(clicked()), KstPluginDialogI::globalInstance(), SLOT(show()));
+  connect(NewEvent, SIGNAL(clicked()), KstEventMonitorI::globalInstance(), SLOT(show()));
+  connect(NewPSD, SIGNAL(clicked()), KstPsdDialogI::globalInstance(), SLOT(show()));
+  connect(NewVector, SIGNAL(clicked()), KstVectorDialogI::globalInstance(), SLOT(show()));
+  connect(OpenPlotDialog, SIGNAL(clicked()),
+      KstApp::inst(), SLOT(showPlotDialog()));
 
-    connect(NewMatrix, SIGNAL(clicked()),
-            KstMatrixDialogI::globalInstance(), SLOT(show_New()));
+  connect(NewImage, SIGNAL(clicked()), KstImageDialogI::globalInstance(), SLOT(show()));
 
-// FIXME: Filters disabled for now...
-//  connect(Filter, SIGNAL(clicked()), this, SLOT(filter_I()));
+  connect(NewMatrix, SIGNAL(clicked()), KstMatrixDialogI::globalInstance(), SLOT(show()));
+  
+  connect(NewCSD, SIGNAL(clicked()), KstCsdDialogI::globalInstance(), SLOT(show()));
 
-    connect(DataView, SIGNAL(contextMenuRequested(QListViewItem*, const QPoint&, int)), this, SLOT(contextMenu(QListViewItem*, const QPoint&, int)));
+  connect(DataView, SIGNAL(contextMenuRequested(QListViewItem*, const QPoint&, int)), this, SLOT(contextMenu(QListViewItem*, const QPoint&, int)));
 }
 
 
@@ -390,16 +587,21 @@ void KstDataManagerI::update() {
 
   KST::dataObjectList.lock().writeLock();
   KST::vectorList.lock().writeLock();
+  KST::matrixList.lock().writeLock();
 
-  //
   // garbage collect first
-  //
   for (QListViewItem *i = DataView->firstChild(); i; i = i->nextSibling()) {
     KstObjectItem *oi = static_cast<KstObjectItem*>(i);
     if (i->rtti() == RTTI_OBJ_OBJECT) {
       if (KST::dataObjectList.findTag(oi->tagName()) == KST::dataObjectList.end()) {
         trash.push(i);
       }
+    } else if (i->rtti() == RTTI_OBJ_DATA_MATRIX || 
+               i->rtti() == RTTI_OBJ_MATRIX ||
+               i->rtti() == RTTI_OBJ_STATIC_MATRIX) {
+      if (KST::matrixList.findTag(oi->tagName()) == KST::matrixList.end()) {
+        trash.push(i);  
+      }
     } else {
       if (KST::vectorList.findTag(oi->tagName()) == KST::vectorList.end()) {
         trash.push(i);
@@ -412,9 +614,7 @@ void KstDataManagerI::update() {
   trash.clear();
   DataView->blockSignals(false);
 
-  //
   // update the data objects
-  //
   for (KstDataObjectList::iterator it = KST::dataObjectList.begin();
                                     it != KST::dataObjectList.end();
                                                                ++it) {
@@ -433,12 +633,11 @@ void KstDataManagerI::update() {
     }
   }
 
+  KST::matrixList.lock().writeUnlock();
   KST::vectorList.lock().writeUnlock();
   KST::dataObjectList.lock().writeUnlock();
 
-  //
-  // update the vectors
-  //
+  // update the data vectors
   KstRVectorList rvl = kstObjectSubList<KstVector,KstRVector>(KST::vectorList);
   for (KstRVectorList::iterator it = rvl.begin(); it != rvl.end(); ++it) {
     bool found = false;
@@ -456,6 +655,60 @@ void KstDataManagerI::update() {
     }
   }
 
+  // update the static vectors
+  KstSVectorList svl = kstObjectSubList<KstVector,KstSVector>(KST::vectorList);
+  for (KstSVectorList::iterator it = svl.begin(); it != svl.end(); ++it) {
+    bool found = false;
+    for (QListViewItem *i = DataView->firstChild(); i; i = i->nextSibling()) {
+      KstObjectItem *oi = static_cast<KstObjectItem*>(i);
+      if (oi->rtti() == RTTI_OBJ_STATIC_VECTOR && oi->tagName() == (*it)->tagName()) {
+        oi->update(true, 1);
+        found = true;
+        break;
+      }
+    }
+    if (!found) {
+      KstObjectItem *i = new KstObjectItem(DataView, *it, this, 1);
+      connect(i, SIGNAL(updated()), this, SLOT(doUpdates()));
+    }
+  }
+
+  // update the data matrices 
+  KstRMatrixList rml = kstObjectSubList<KstMatrix,KstRMatrix>(KST::matrixList);
+  for (KstRMatrixList::iterator it = rml.begin(); it != rml.end(); ++it) {
+    bool found = false;
+    for (QListViewItem *i = DataView->firstChild(); i; i = i->nextSibling()) {
+      KstObjectItem *oi = static_cast<KstObjectItem*>(i);
+      if (oi->rtti() == RTTI_OBJ_DATA_MATRIX && oi->tagName() == (*it)->tagName()) {
+        oi->update(true, 1);
+        found = true;
+        break;
+      }
+    }
+    if (!found) {
+      KstObjectItem *i = new KstObjectItem(DataView, *it, this, 1);
+      connect(i, SIGNAL(updated()), this, SLOT(doUpdates()));
+    }
+  }
+
+  // update the static matrices
+  KstSMatrixList sml = kstObjectSubList<KstMatrix,KstSMatrix>(KST::matrixList);
+  for (KstSMatrixList::iterator it = sml.begin(); it != sml.end(); ++it) {
+    bool found = false;
+    for (QListViewItem *i = DataView->firstChild(); i; i = i->nextSibling()) {
+      KstObjectItem *oi = static_cast<KstObjectItem*>(i);
+      if (oi->rtti() == RTTI_OBJ_STATIC_MATRIX && oi->tagName() == (*it)->tagName()) {
+        oi->update(true, 1);
+        found = true;
+        break;
+      }
+    }
+    if (!found) {
+      KstObjectItem *i = new KstObjectItem(DataView, *it, this, 1);
+      connect(i, SIGNAL(updated()), this, SLOT(doUpdates()));
+    }
+  }
+
   // is this really necessary?  I would think not...
   for (QListViewItem *i = DataView->firstChild(); i; i = i->nextSibling()) {
     if (i == currentItem) {
@@ -474,11 +727,6 @@ void KstDataManagerI::update() {
 }
 
 
-void KstDataManagerI::filter_I() {
-//  static_cast<KstApp*>(doc->parent())->showFilterListEditor();
-}
-
-
 void KstDataManagerI::edit_I() {
   QListViewItem *qi;
 
@@ -493,9 +741,21 @@ void KstDataManagerI::edit_I() {
     emit editDataVector(qi->text(0));
   }
 
+  if (qi->rtti() == RTTI_OBJ_STATIC_VECTOR) {
+    emit editStaticVector(qi->text(0));
+  }
+
   if (qi->rtti() == RTTI_OBJ_OBJECT) {
     static_cast<KstObjectItem*>(qi)->dataObject()->showDialog();
   }
+  
+  if (qi->rtti() == RTTI_OBJ_DATA_MATRIX) {
+    emit editDataMatrix(qi->text(0));  
+  }
+  
+  if (qi->rtti() == RTTI_OBJ_STATIC_MATRIX) {
+    emit editStaticMatrix(qi->text(0));  
+  }
 }
 
 
@@ -511,7 +771,21 @@ void KstDataManagerI::delete_I() {
       KST::vectorList.removeTag(koi->tagName());
       KST::vectorList.lock().writeUnlock();
       doUpdates();
+    } else if (qi->rtti() == RTTI_OBJ_STATIC_VECTOR) {
+      KST::vectorList.lock().writeLock();
+      KST::vectorList.removeTag(koi->tagName());
+      KST::vectorList.lock().writeUnlock();
+      doUpdates();
+    } else if (qi->rtti() == RTTI_OBJ_DATA_MATRIX) {
+      KST::matrixList.lock().writeLock();
+      KST::matrixList.removeTag(koi->tagName());
+      KST::matrixList.lock().writeUnlock();  
+    } else if (qi->rtti() == RTTI_OBJ_STATIC_MATRIX) {
+      KST::matrixList.lock().writeLock();
+      KST::matrixList.removeTag(koi->tagName());
+      KST::matrixList.lock().writeUnlock();  
     }
+    update();
   } else {
     // Don't prompt for base curves
     KstBaseCurvePtr bc = kst_cast<KstBaseCurve>(koi->dataObject());
@@ -532,8 +806,45 @@ void KstDataManagerI::delete_I() {
         } else {
           KMessageBox::sorry(this, i18n("Unknown error deleting data vector."));
         }
+      } else if (qi->rtti() == RTTI_OBJ_STATIC_VECTOR) {
+        KstSVectorPtr x = kst_cast<KstSVector>(*KST::vectorList.findTag(koi->tagName()));
+        if (x) {
+          x->deleteDependents();
+          x = 0L;
+          KST::vectorList.lock().writeLock();
+          KST::vectorList.removeTag(koi->tagName());
+          KST::vectorList.lock().writeUnlock();
+          doUpdates();
+        } else {
+          KMessageBox::sorry(this, i18n("Unknown error deleting static vector."));
+        }
+      } else if (qi->rtti() == RTTI_OBJ_DATA_MATRIX) {
+        KstRMatrixPtr x = kst_cast<KstRMatrix>(*KST::matrixList.findTag(koi->tagName()));
+        if (x) {
+          x->deleteDependents();
+          x = 0L;
+          KST::matrixList.lock().writeLock();
+          KST::matrixList.removeTag(koi->tagName());
+          KST::matrixList.lock().writeUnlock();
+          doUpdates();
+        } else {
+          KMessageBox::sorry(this, i18n("Unknown error deleting data matrix."));
+        }
+      } else if (qi->rtti() == RTTI_OBJ_STATIC_MATRIX) {
+        KstSMatrixPtr x = kst_cast<KstSMatrix>(*KST::matrixList.findTag(koi->tagName()));
+        if (x) {
+          x->deleteDependents();
+          x = 0L;
+          KST::matrixList.lock().writeLock();
+          KST::matrixList.removeTag(koi->tagName());
+          KST::matrixList.lock().writeUnlock();
+          doUpdates();
+        } else {
+          KMessageBox::sorry(this, i18n("Unknown error deleting static matrix."));
+        }  
       }
       KstApp::inst()->paintAll(P_PLOT);
+      update();
     } else {
       KMessageBox::sorry(this, i18n("Cannot delete objects with dependencies."));
     }
@@ -543,7 +854,7 @@ void KstDataManagerI::delete_I() {
 
 // Menu IDs:
 // 100->499 reserved for plots
-// 500->999 reserver for filters
+// 500->999 reserved for filters
 
 void KstDataManagerI::contextMenu(QListViewItem *i, const QPoint& p, int col) {
   Q_UNUSED(col)
@@ -551,38 +862,20 @@ void KstDataManagerI::contextMenu(QListViewItem *i, const QPoint& p, int col) {
   if (!i) {
     return;
   }
-  
+
   KstObjectItem *koi = static_cast<KstObjectItem*>(i);
   KstBaseCurvePtr c;
   KstImagePtr img;
 
   KPopupMenu *m = new KPopupMenu(this);
 
-  m->setTitle(koi->text(0));
+  m->insertTitle(koi->text(0));
 
   int id;
-  if (koi->rtti() != RTTI_OBJ_VECTOR) {
+  if (koi->rtti() != RTTI_OBJ_VECTOR && koi->rtti() != RTTI_OBJ_MATRIX) {
     id = m->insertItem(i18n("&Edit..."), this, SLOT(edit_I()));
   }
 
-  if (koi->rtti() == RTTI_OBJ_DATA_VECTOR ||
-      koi->rtti() == RTTI_OBJ_VECTOR) {
-    KPopupMenu *fMenu = new KPopupMenu(this);
-    id = 500;
-    FilterMap.clear();
-#if 0 
-    KST::filterSetList.lock().readLock();
-    for (KstFilterSetList::Iterator f = KST::filterSetList.begin(); f != KST::filterSetList.end(); ++f) {
-      fMenu->insertItem((*f)->name(), koi, SLOT(applyFilter(int)), 0, id);
-      FilterMap[id] = (*f)->name();
-      ++id;
-    }
-    KST::filterSetList.lock().readUnlock();
-#endif
-    id = m->insertItem(i18n("Apply &Filter"), fMenu);
-    m->setItemEnabled(id, !FilterMap.isEmpty());
-  }
-
   if (koi->dataObject()) {
     const KstCurveHintList* hints = koi->dataObject()->curveHints();
     if (!hints->isEmpty()) {
@@ -599,15 +892,18 @@ void KstDataManagerI::contextMenu(QListViewItem *i, const QPoint& p, int col) {
   if (koi->rtti() == RTTI_OBJ_DATA_VECTOR) {
     id = m->insertItem(i18n("&Make Curve..."), koi, SLOT(makeCurve()));
     id = m->insertItem(i18n("Make &Power Spectrum..."), koi, SLOT(makePSD()));
+    id = m->insertItem(i18n("Make Cumulative &Spectral Decay..."), koi, SLOT(makeCSD()));
     id = m->insertItem(i18n("Make &Histogram..."), koi, SLOT(makeHistogram()));
-    id = m->insertItem(i18n("Make M&atrix..."), koi, SLOT(makeMatrix()));
     id = m->insertItem(i18n("&Reload"), koi, SLOT(reload()));
   } else if (koi->rtti() == RTTI_OBJ_VECTOR) {
     id = m->insertItem(i18n("&Make Curve..."), koi, SLOT(makeCurve()));
     id = m->insertItem(i18n("Make &Power Spectrum..."), koi, SLOT(makePSD()));
+    id = m->insertItem(i18n("Make Cumulative &Spectral Decay..."), koi, SLOT(makeCSD()));
     id = m->insertItem(i18n("Make &Histogram..."), koi, SLOT(makeHistogram()));
-    id = m->insertItem(i18n("Make M&atrix..."), koi, SLOT(makeMatrix()));
-  } else if (kst_cast<KstMatrix>(koi->dataObject())) {
+  } else if (koi->rtti() == RTTI_OBJ_DATA_MATRIX) {
+    id = m->insertItem(i18n("Make &Image..."), koi, SLOT(makeImage()));  
+    id = m->insertItem(i18n("&Reload"), koi, SLOT(reload()));
+  } else if (koi->rtti() == RTTI_OBJ_MATRIX || koi->rtti() == RTTI_OBJ_STATIC_MATRIX) {
     id = m->insertItem(i18n("Make &Image..."), koi, SLOT(makeImage()));
   } else if ((c = kst_cast<KstBaseCurve>(koi->dataObject()))) {
     KPopupMenu *addMenu = new KPopupMenu(this);
@@ -643,46 +939,11 @@ void KstDataManagerI::contextMenu(QListViewItem *i, const QPoint& p, int col) {
     m->setItemEnabled(id, haveAdd);
     id = m->insertItem(i18n("&Remove From Plot"), removeMenu);
     m->setItemEnabled(id, haveRemove);
-  } else if ((img = kst_cast<KstImage>(koi->dataObject()))) {
-    KPopupMenu *addMenu = new KPopupMenu(this);
-    KPopupMenu *removeMenu = new KPopupMenu(this);
-    PlotMap.clear();
-    id = 100;
-    bool haveAdd = false, haveRemove = false;
+  } 
 
-    KstApp *app = KstApp::inst();
-    KMdiIterator<KMdiChildView*> *it = app->createIterator();
-    while (it->currentItem()) {
-      KstViewWindow *v = dynamic_cast<KstViewWindow*>(it->currentItem());
-      if (v) {
-        Kst2DPlotList plots = v->view()->findChildrenType<Kst2DPlot>();
-        for (Kst2DPlotList::Iterator i = plots.begin(); i != plots.end(); ++i) {
-          Kst2DPlotPtr plot = *i;
-          if (!plot->hasImage(img)) {
-            addMenu->insertItem(i18n("%1 - %2").arg(v->caption()).arg(plot->tagName()), koi, SLOT(addImageToPlot(int)), 0, id);
-            haveAdd = true;
-          } else {
-            removeMenu->insertItem(i18n("%1 - %2").arg(v->caption()).arg(plot->tagName()), koi, SLOT(removeImageFromPlot(int)), 0, id);
-            haveRemove = true;
-          }
-          PlotMap[id++] = plot;
-        }
-      }
-      it->next();
-    }
-
-    app->deleteIterator(it);
-
-    id = m->insertItem(i18n("&Add to Plot"), addMenu);
-    m->setItemEnabled(id, haveAdd);
-    id = m->insertItem(i18n("&Remove From Plot"), removeMenu);
-    m->setItemEnabled(id, haveRemove);
-  }
-
-  if (koi->rtti() != RTTI_OBJ_VECTOR) {
-    // no slave vectors get this
+  if (koi->rtti() != RTTI_OBJ_VECTOR && koi->rtti() != RTTI_OBJ_MATRIX) {
+    // no slave vectors or matrices get this
     id = m->insertItem(i18n("&Delete"), this, SLOT(delete_I()));
-    m->setItemEnabled(id, koi->removable());
   }
 
   m->popup(p);
