@@ -24,6 +24,7 @@
 #include <assert.h>
 #include <fcntl.h>
 #include <unistd.h>
+#include <math.h>
 
 // include files for Qt
 #include <qdeepcopy.h>
@@ -32,23 +33,36 @@
 
 // include files for KDE
 #include <dcopclient.h>
-#include <kdebug.h>
+#include "ksdebug.h"
+#include <kdeversion.h>
 #include <kfiledialog.h>
 #include <kmessagebox.h>
+#include <kmdimainfrm.h>
 #include <kprogress.h>
 #include <ksavefile.h>
 
 // application specific includes
+#include "kst2dplot.h"
+#include "kstcsd.h"
 #include "kstdoc.h"
 #include "kstgraphfiledialog_i.h"
 #include "kstequation.h"
+#include "ksteventmonitorentry.h"
 #include "ksthistogram.h"
+#include "kstimage.h"
+#include "kstmatrixdefaults.h"
 #include "kstplugin.h"
 #include "kstpsd.h"
 #include "kstrvector.h"
 #include "kstvcurve.h"
 #include "kstvectordefaults.h"
+#include "kstsvector.h"
+#include "kstavector.h"
+#include "kstrmatrix.h"
+#include "kstamatrix.h"
+#include "kstsmatrix.h"
 #include "kstviewwindow.h"
+#include "logevents.h"
 #include "threadevents.h"
 
 static bool backupFile(const QString& qFilename, const QString& backupDir = QString::null, const QString& backupExtension = "~");
@@ -58,7 +72,6 @@ KstDoc::KstDoc(QWidget *parent, const char *name)
   _lock = 0;
   _updating = false;
   _stopping = false;
-  _updateDelay = 5;
   _nextEventPaint = false;
 }
 
@@ -78,6 +91,14 @@ const QString& KstDoc::absFilePath() const {
   return _absFilePath;
 }
 
+void KstDoc::setLastFilePath(const QString& filename) {
+  _lastFilePath = filename;
+}
+
+const QString& KstDoc::lastFilePath() const {
+  return _lastFilePath;
+}
+
 void KstDoc::setTitle(const QString& t) {
   _title = t;
 }
@@ -90,39 +111,41 @@ bool KstDoc::saveModified(bool doDelete) {
   bool completed = true;
 
   if (_modified) {
-    KstApp *win = static_cast<KstApp*>(parent());
-    int want_save = KMessageBox::warningYesNoCancel( win, i18n("The current plot definition has been modified. Do you want to save it?"), i18n("Question"));
-    switch (want_save) {
-      case KMessageBox::Yes:
-        if (_title == i18n("Untitled")) {
-          if (!win->slotFileSaveAs()) {
-            return false;
+    KstApp *win = KstApp::inst();
+    if (win->activeWindow()) {
+      int want_save = KMessageBox::warningYesNoCancel( win, i18n("The current plot definition has been modified. Do you want to save it?"), i18n("Question"));
+      switch (want_save) {
+        case KMessageBox::Yes:
+          if (_title == i18n("Untitled")) {
+            if (!win->slotFileSaveAs()) {
+              return false;
+            }
+          } else {
+            saveDocument(absFilePath());
           }
-        } else {
-          saveDocument(absFilePath());
-        }
 
-        if (doDelete) {
-          deleteContents();
-        }
-        completed = true;
-        break;
+          if (doDelete) {
+            deleteContents();
+          }
+          completed = true;
+          break;
 
-      case KMessageBox::No:
-        if (doDelete) {
-          setModified(false);
-          deleteContents();
-        }
-        completed = true;
-        break;
+        case KMessageBox::No:
+          if (doDelete) {
+            setModified(false);
+            deleteContents();
+          }
+          completed = true;
+          break;
 
-      case KMessageBox::Cancel:
-        completed = false;
-        break;
+        case KMessageBox::Cancel:
+          completed = false;
+          break;
 
-      default:
-        completed = false;
-        break;
+        default:
+          completed = false;
+          break;
+      }
     }
   }
 
@@ -155,38 +178,47 @@ bool KstDoc::openDocument(const KURL& url, const QString& o_file,
     return false;
   }
 
+  QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
   opening = true;
 
   deleteContents();
-  _updating = true; // block update thread from sending events 'till we're done
+  KstApp::inst()->setPaused(true);
+  _updating = true; // block update thread from sending events until we're done
 
   QFile f(url.path());
   if (!f.exists()) {
-    KMessageBox::sorry(0L, i18n("%1: There is no file with that name to open.").arg(url.path()));
+    KMessageBox::sorry(KstApp::inst(), i18n("%1: There is no file with that name to open.").arg(url.path()));
     opening = false;
     _updating = false;
+    KstApp::inst()->setPaused(false);
+    QApplication::restoreOverrideCursor();
     return false;
   }
 
   _title = url.fileName(false);
   _absFilePath = url.path();
+  _lastFilePath = url.path();
   if (_title.isEmpty()) {
     _title = _absFilePath;
   }
   QDomDocument doc(_title);
 
   if (!f.open(IO_ReadOnly)) {
-    KMessageBox::sorry(0L, i18n("%1: File exists, but kst could not open it.").arg(url.path()));
+    KMessageBox::sorry(KstApp::inst(), i18n("%1: File exists, but kst could not open it.").arg(url.path()));
     opening = false;
     _updating = false;
+    KstApp::inst()->setPaused(false);
+    QApplication::restoreOverrideCursor();
     return false;
   }
 
   if (!doc.setContent(&f)) {
-    KMessageBox::sorry(0L, i18n("%1: Not a valid kst plot specification file.").arg(url.path()));
+    KMessageBox::sorry(KstApp::inst(), i18n("%1: Not a valid kst plot specification file.").arg(url.path()));
     f.close();
     opening = false;
     _updating = false;
+    KstApp::inst()->setPaused(false);
+    QApplication::restoreOverrideCursor();
     return false;
   }
 
@@ -197,10 +229,33 @@ bool KstDoc::openDocument(const KURL& url, const QString& o_file,
   int count;
   int handled = 0;
   bool warnOldKstFile = false;
+  QString activeWindow;
   KstRVectorPtr vector;
   KstApp *app = KstApp::inst();
   QString readingDocument = i18n("Reading Kst file");
 
+  if (docElem.tagName() != "kstdoc") {
+    QString err = i18n("Error opening file %1.  Does not appear to be a Kst file.").arg(url.path());
+    KstDebug::self()->log(err, KstDebug::Error);
+    KMessageBox::sorry(KstApp::inst(), err);
+    opening = false;
+    _updating = false;
+    KstApp::inst()->setPaused(false);
+    QApplication::restoreOverrideCursor();
+    return false;
+  }
+
+  if (docElem.attribute("version") != "1.2" && !docElem.attribute("version").isEmpty()) {
+    QString err = i18n("Error opening file %2.  Version %1 is too new.  Update Kst or fix the Kst file.").arg(docElem.attribute("version")).arg(url.path());
+    KstDebug::self()->log(err, KstDebug::Error);
+    KMessageBox::sorry(KstApp::inst(), err);
+    opening = false;
+    _updating = false;
+    KstApp::inst()->setPaused(false);
+    QApplication::restoreOverrideCursor();
+    return false;
+  }
+
   QDomNode n = docElem.firstChild();
   count = docElem.childNodes().length();
   app->slotUpdateProgress(count, handled, readingDocument);
@@ -211,19 +266,21 @@ bool KstDoc::openDocument(const KURL& url, const QString& o_file,
     if (!e.isNull()) { // the node was really an element.
       if (e.tagName() == "windowsize") {
         QDomNode nn = e.firstChild();
-        QSize size = static_cast<KstApp*>(parent())->size();
+        QSize size = app->size();
         while (!nn.isNull()) {
           QDomElement ee = nn.toElement(); // convert child to element
           if (ee.tagName() == "width") {
             size.setWidth(ee.text().toInt());
           } else if (ee.tagName() == "height") {
             size.setHeight(ee.text().toInt());
+          } else if (ee.tagName() == "active") {
+            activeWindow = ee.attribute("name");
           }
           nn = nn.nextSibling();
         }
-        static_cast<KstApp*>(parent())->resize(size);
+        app->resize(size);
       } else if (e.tagName() == "graphicsautosave") {
-        KstGraphFileDialogI *gdlg = KstApp::inst()->graphFileDlg();
+        KstGraphFileDialogI *gdlg = app->graphFileDlg();
         int time = e.attribute("time").toInt();
         bool en = e.attribute("enabled") == "true";
         QString fmt = e.attribute("format");
@@ -255,9 +312,6 @@ bool KstDoc::openDocument(const KURL& url, const QString& o_file,
         gdlg->applyAutosave();
       } else if (e.tagName() == "plotcols") {
         columns = e.text().toInt();
-      } else if (e.tagName() == "updateDelay") {
-// FIXME XXXX
-        setDelay(e.text().toInt());
       } else if (e.tagName() == "kstfile") {
         KstWriteLocker dswl(&KST::dataSourceList.lock());
         KstDataSourcePtr file;
@@ -280,6 +334,13 @@ bool KstDoc::openDocument(const KURL& url, const QString& o_file,
         vector = new KstRVector(e, o_file, o_n, o_f, o_s, o_ave);
         KST::addVectorToList(KstVectorPtr(vector));
         // Vectors are automatically added to the global list.
+      } else if (e.tagName() == "svector") {
+        KstSVectorPtr svector = new KstSVector(e);
+        KST::addVectorToList(KstVectorPtr(svector));
+      } else if (e.tagName() == "avector") {
+        KstAVectorPtr avector = new KstAVector(e);
+        avector = new KstAVector(e);
+        KST::addVectorToList(KstVectorPtr(avector));
       } else if (e.tagName() == "plugin") {
         KstDataObjectPtr p = new KstPlugin(e);
         KstWriteLocker dowl(&KST::dataObjectList.lock());
@@ -306,6 +367,10 @@ bool KstDoc::openDocument(const KURL& url, const QString& o_file,
         KstDataObjectPtr p = new KstPSD(e);
         KstWriteLocker dowl(&KST::dataObjectList.lock());
         KST::dataObjectList.append(p);
+      } else if (e.tagName() == "csdobject") {
+        KstDataObjectPtr p = new KstCSD(e);
+        KstWriteLocker dowl(&KST::dataObjectList.lock());
+        KST::dataObjectList.append(p);
       } else if (e.tagName() == "histogram") {
         KstDataObjectPtr p = new KstHistogram(e);
         KstWriteLocker dowl(&KST::dataObjectList.lock());
@@ -317,23 +382,28 @@ bool KstDoc::openDocument(const KURL& url, const QString& o_file,
       } else if (e.tagName() == "plot") {
         KstBaseCurveList l = kstObjectSubList<KstDataObject,KstBaseCurve>(KST::dataObjectList);
         Kst2DPlotPtr plot = new Kst2DPlot(e);
-        KstApp::inst()->plotHolderWhileOpeningDocument().insert(plot->tagName(), plot);
-      } else if (e.tagName() == "matrix") {
-        KstDataObjectPtr p = new KstMatrix(e);
-        KstWriteLocker dowl(&KST::dataObjectList.lock());
-        KST::dataObjectList.append(p);
+        app->plotHolderWhileOpeningDocument()->insert(plot->tagName(), plot);
+      } else if (e.tagName() == "amatrix") {
+        KstAMatrixPtr p = new KstAMatrix(e);
+        KST::addMatrixToList(KstMatrixPtr(p));
+      } else if (e.tagName() == "smatrix") {
+        KstSMatrixPtr p = new KstSMatrix(e);
+        KST::addMatrixToList(KstMatrixPtr(p));
+      } else if (e.tagName() == "rmatrix") {
+        KstRMatrixPtr p = new KstRMatrix(e);
+        KST::addMatrixToList(KstMatrixPtr(p));
       } else if (e.tagName() == "image") {
         KstDataObjectPtr p = new KstImage(e);
         KstWriteLocker dowl(&KST::dataObjectList.lock());
         KST::dataObjectList.append(p);
       } else if (e.tagName() == "window") {
-        KstViewWindow *p = dynamic_cast<KstViewWindow*>(KstApp::inst()->activeWindow());
+        KstViewWindow *p = dynamic_cast<KstViewWindow*>(app->activeWindow());
         KstViewWindow *toDelete = 0L;
 
         if (!p) {
-          KMdiIterator<KMdiChildView*> *it = KstApp::inst()->createIterator();
+          KMdiIterator<KMdiChildView*> *it = app->createIterator();
           p = dynamic_cast<KstViewWindow*>(it->currentItem());
-          KstApp::inst()->deleteIterator(it);
+          app->deleteIterator(it);
         }
 
         if (p && p->view()->children().count() == 0) {
@@ -341,10 +411,10 @@ bool KstDoc::openDocument(const KURL& url, const QString& o_file,
         }
 
         p = new KstViewWindow(e);
-        KstApp::inst()->addWindow(p);
+        app->addWindow(p, KMdi::StandardAdd | KMdi::UseKMdiSizeHint);
         p->activate();
         if (toDelete) {
-          KstApp::inst()->closeWindow(toDelete);
+          app->closeWindow(toDelete);
         }
       } else {
         KstDebug::self()->log(i18n("Unsupported element '%1' in file %2.").arg(e.tagName()).arg(o_file), KstDebug::Warning);
@@ -357,16 +427,14 @@ bool KstDoc::openDocument(const KURL& url, const QString& o_file,
   }
 
   if (warnOldKstFile) {
-    KMessageBox::sorry(0L, i18n("You tried to load an old Kst file.  Curves created by equations or PSDs will not be loaded."));
+    KMessageBox::sorry(KstApp::inst(), i18n("You tried to load an old Kst file.  Curves created by equations or PSDs will not be loaded."));
   }
 
   app->slotUpdateProgress(0, 0, i18n("Creating plots"));
-  //
   // if we have anything left in plotHolderWhileOpeningDocument then
   //  we are most likely reading an old style document, so we create
   //  a default view and fill it with whatever is left...
-  //
-  if (!app->plotHolderWhileOpeningDocument().isEmpty() && !app->activeWindow()) {
+  if (!app->plotHolderWhileOpeningDocument()->isEmpty() && !app->activeWindow()) {
 
     QString winName = app->newWindow(QString::null);
     KstViewWindow *win = dynamic_cast<KstViewWindow*>(app->findWindow(winName));
@@ -377,8 +445,8 @@ bool KstDoc::openDocument(const KURL& url, const QString& o_file,
         win->view()->setColumns(columns);
       }
 
-      for (Kst2DPlotMap::Iterator it = app->plotHolderWhileOpeningDocument().begin();
-          it != app->plotHolderWhileOpeningDocument().end(); ++it ) {
+      for (Kst2DPlotMap::Iterator it = app->plotHolderWhileOpeningDocument()->begin();
+          it != app->plotHolderWhileOpeningDocument()->end(); ++it ) {
         Kst2DPlotPtr plot = *it;
 
         if (plot->_width > 0.0 && plot->_height > 0.0 ) {
@@ -397,12 +465,11 @@ bool KstDoc::openDocument(const KURL& url, const QString& o_file,
     }
   }
 
-  KstApp::inst()->plotHolderWhileOpeningDocument().clear();
+  app->plotHolderWhileOpeningDocument()->clear();
 
   app->slotUpdateProgress(0, 0, i18n("Loading data"));
-  //
+
   // lazy load data
-  //
   KstDataObjectList bitBucket;
   {
     KST::dataObjectList.lock().readLock();
@@ -410,14 +477,12 @@ bool KstDoc::openDocument(const KURL& url, const QString& o_file,
     KST::dataObjectList.lock().readUnlock();
     for (KstDataObjectList::Iterator i = dol.begin(); i != dol.end(); ++i) {
       assert(*i);
-      //kdDebug() << "Load inputs for " << (*i)->tagName() << " " << (void*)*i << endl;
+      //kstdDebug() << "Load inputs for " << (*i)->tagName() << " " << (void*)*i << endl;
       (*i)->KstRWLock::writeLock();
       bool rc = (*i)->loadInputs();
       (*i)->KstRWLock::writeUnlock();
       if (!rc) {
-        //
         // schedule for removal
-        //
         bitBucket.append(*i);
       }
       kapp->eventLoop()->processEvents(QEventLoop::ExcludeSocketNotifiers, 10);
@@ -431,38 +496,34 @@ bool KstDoc::openDocument(const KURL& url, const QString& o_file,
   KST::dataObjectList.lock().writeUnlock();
 
   if (!bitBucket.isEmpty()) {
-    KMessageBox::sorry(0L, i18n("The Kst file could not be loaded in its entirety due to missing objects or data."));
+    QStringList names;
+    for (KstDataObjectList::Iterator i = bitBucket.begin(); i != bitBucket.end(); ++i) {
+      names += (*i)->tagName();
+    }
+
+#if KDE_VERSION < KDE_MAKE_VERSION(3,3,90)
+    KMessageBox::informationList(KstApp::inst(), i18n("The Kst file could not be loaded in its entirety due to missing objects or data."), names);
+#else
+    KMessageBox::errorList(KstApp::inst(), i18n("The Kst file could not be loaded in its entirety due to missing objects or data."), names);
+#endif
   }
 
   createScalars();
 
   app->slotUpdateProgress(0, 0, i18n("Synchronizing data"));
-  //
-  // update plots now that lazy loading is done
-  //
-  // the updating happens anyway, so it appears that this is significantly
-  // redundant
-  //
-//   KMdiIterator<KMdiChildView*> *it = KstApp::inst()->createIterator();
-//   while (it->currentItem()) {
-//     KstViewWindow *c = dynamic_cast<KstViewWindow*>(it->currentItem());
-//     if (c) {
-//       Kst2DPlotList allplots = c->view()->findChildrenType<Kst2DPlot>(true);
-//       for (Kst2DPlotList::Iterator i = allplots.begin(); i != allplots.end(); ++i) {
-//         (*i)->draw();
-//         kapp->eventLoop()->processEvents(QEventLoop::ExcludeSocketNotifiers, 10);
-//       }
-//     }
-//     it->next();
-//   }
-
-  //KstApp::inst()->deleteIterator(it);
   kapp->dcopClient()->setAcceptCalls(true);
 
   _nextEventPaint = true;
 
   emit updateDialogs();
 
+  if (!activeWindow.isEmpty()) {
+    KMdiChildView *c = app->findWindow(activeWindow);
+    if (c) {
+      c->activate();
+    }
+  }
+
   app->slotUpdateProgress(0, 0, QString::null);
 
   _modified = false;
@@ -471,57 +532,70 @@ bool KstDoc::openDocument(const KURL& url, const QString& o_file,
 
   opening = false;
 
+  KstApp::inst()->setPaused(false);
+    QApplication::restoreOverrideCursor();
   return true;
 }
 
+
 void KstDoc::saveDocument(QTextStream& ts, bool saveAbsoluteVectorPositions) {
   KstApp *app = KstApp::inst();
   ts << "<?xml version=\"1.0\" encoding=\"UTF-8\"?>" << endl;
-  ts << "<kstdoc>" << endl;
+  ts << "<kstdoc version=\"1.2\">" << endl;
   // save window geometry for this kst file
   ts << "  <windowsize>" << endl;
-  ts << "    <width>" << ((KstApp *)parent())->width() << "</width>" << endl;
-  ts << "    <height>"<< ((KstApp *)parent())->height()<< "</height>" << endl;
+  ts << "    <width>" << app->width() << "</width>" << endl;
+  ts << "    <height>"<< app->height()<< "</height>" << endl;
+  KMdiChildView *c = app->activeWindow();
+  if (c) {
+    ts << "    <active name=\"" << QStyleSheet::escape(c->caption()) << "\"/>" << endl;
+  }
   ts << "  </windowsize>" << endl;
 
   // save graphics autosave settings
   ts << "  <graphicsautosave time=\""
-    << KstApp::inst()->graphFileDlg()->autoSaveTimer()
+    << app->graphFileDlg()->autoSaveTimer()
     << "\" enabled=\""
-    << (KstApp::inst()->graphFileDlg()->autoSaving() ? "true" : "false")
+    << (app->graphFileDlg()->autoSaving() ? "true" : "false")
     << "\" format=\""
-    << QStyleSheet::escape(KstApp::inst()->graphFileDlg()->format())
+    << QStyleSheet::escape(app->graphFileDlg()->format())
     << "\" xsize=\""
-    << KstApp::inst()->graphFileDlg()->imageXSize()
+    << app->graphFileDlg()->imageXSize()
     << "\" ysize=\""
-    << KstApp::inst()->graphFileDlg()->imageYSize()
+    << app->graphFileDlg()->imageYSize()
     << "\" display=\""
-    << KstApp::inst()->graphFileDlg()->display()
+    << app->graphFileDlg()->display()
     // #### Kst2 Remove me - compatibility
     << "\" square=\""
-    << (KstApp::inst()->graphFileDlg()->display() == 1 ? "true" : "false");
+    << (app->graphFileDlg()->display() == 1 ? "true" : "false");
 
-  if (KstApp::inst()->graphFileDlg()->autoSaving()) {
-    ts << "\" url=\"" << QStyleSheet::escape(KstApp::inst()->graphFileDlg()->url().url());
+  if (app->graphFileDlg()->autoSaving()) {
+    ts << "\" url=\"" << QStyleSheet::escape(app->graphFileDlg()->url().url());
   }
   ts << "\" />" << endl;
 
-  // save update delay
-//FIXME XXXX
-  ts << " <updateDelay>" << delay() << "</updateDelay>" << endl;
-
   KstRVectorList rvl = kstObjectSubList<KstVector,KstRVector>(KST::vectorList);
+  KstRMatrixList rml = kstObjectSubList<KstMatrix,KstRMatrix>(KST::matrixList);
 
   // save files
   KST::dataSourceList.lock().readLock();
   for (uint i = 0; i < KST::dataSourceList.count(); i++) {
     KstDataSourcePtr dsp = KST::dataSourceList[i];
-    for (KstRVectorList::Iterator it = rvl.begin(); it != rvl.end(); ++it) {
+    bool saved = false;
+    for (KstRVectorList::Iterator it = rvl.begin(); it != rvl.end() && !saved; ++it) {
       if ((*it)->dataSource() == dsp) {
         ts << "  <kstfile>" << endl;
         dsp->save(ts, "    ");
         ts << "  </kstfile>" << endl;
-        break;
+        saved = true;
+      } 
+    }
+    for (KstRMatrixList::Iterator it = rml.begin(); it != rml.end() && !saved; ++it) {
+      if ((*it)->dataSource() == dsp) {
+        ts << "  <kstfile>" << endl;
+        dsp->save(ts, "    ");
+        ts << "  </kstfile>" << endl;
+        saved = true;
       }
     }
   }
@@ -546,10 +620,24 @@ void KstDoc::saveDocument(QTextStream& ts, bool saveAbsoluteVectorPositions) {
   }
 
   // save vectors
-  for (KstRVectorList::Iterator it = rvl.begin(); it != rvl.end(); ++it) {
-    ts << "  <vector>" << endl;
-    (*it)->save(ts, "    ", saveAbsoluteVectorPositions);
-    ts << "  </vector>" << endl;
+//   for (KstRVectorList::Iterator it = rvl.begin(); it != rvl.end(); ++it) {
+//     ts << "  <vector>" << endl;
+//     (*it)->save(ts, "    ", saveAbsoluteVectorPositions);
+//     ts << "  </vector>" << endl;
+//   }
+  for (KstVectorList::Iterator it = KST::vectorList.begin(); it != KST::vectorList.end(); ++it) {
+    if ((*it)->saveable()) {
+      //      ts << "  <vector>" << endl;
+      (*it)->save(ts, "  ", saveAbsoluteVectorPositions);
+      //ts << "  </vector>" << endl;
+    }
+  }
+  
+  // save matrices
+  for (KstMatrixList::Iterator it = KST::matrixList.begin(); it != KST::matrixList.end(); ++it) {
+    if ((*it)->saveable()) {
+      (*it)->save(ts, "  ");  
+    }  
   }
 
   // save data objects
@@ -570,7 +658,6 @@ void KstDoc::saveDocument(QTextStream& ts, bool saveAbsoluteVectorPositions) {
           ts << "  <plot>" << endl;
           plot->save(ts, "    ");
           ts << "  </plot>" << endl;
-        } else {
         }
       }
     }
@@ -601,7 +688,7 @@ bool KstDoc::saveDocument(const QString &filename, bool saveAbsoluteVectorPositi
   QFile f(filename);
 
   if (f.exists() && filename != absFilePath()) {
-    if (KMessageBox::warningYesNo(0L, i18n("%1: A file of this name already exists.").arg(filename),
+    if (KMessageBox::warningYesNo(KstApp::inst(), i18n("%1: A file of this name already exists.").arg(filename),
                                   i18n("Warning"),i18n("&Overwrite"),
                                   i18n("&Cancel")) == KMessageBox::No) {
       return false;
@@ -609,12 +696,15 @@ bool KstDoc::saveDocument(const QString &filename, bool saveAbsoluteVectorPositi
   }
 
   if (!f.open(IO_WriteOnly|IO_Truncate)) {
-    KMessageBox::sorry(0L, i18n("%1: Could not open file for saving. The plot description has not been saved. Try a different filename or directory.").arg(filename));
+    KMessageBox::sorry(KstApp::inst(), i18n("%1: Could not open file for saving. The plot description has not been saved. Try a different filename or directory.").arg(filename));
     return false;
   }
 
   QTextStream ts(&f);
+  ts.setEncoding(QTextStream::UnicodeUTF8);
 
+  _lastFilePath = KURL(filename).path();
+  
   saveDocument(ts, saveAbsoluteVectorPositions);
 
   f.close();
@@ -623,9 +713,10 @@ bool KstDoc::saveDocument(const QString &filename, bool saveAbsoluteVectorPositi
   return true;
 }
 
-void KstDoc::deleteContents() {
 
+void KstDoc::deleteContents() {
   KST::vectorDefaults.sync();
+  KST::matrixDefaults.sync();
 
   KstApp *app = KstApp::inst();
   if (app) { // Can be null on application exit
@@ -643,6 +734,10 @@ void KstDoc::deleteContents() {
   KST::vectorList.lock().writeLock();
   KST::vectorList.clear();
   KST::vectorList.lock().writeUnlock();
+  
+  KST::matrixList.lock().writeLock();
+  KST::matrixList.clear();
+  KST::matrixList.lock().writeUnlock();
 
   KST::scalarList.lock().writeLock();
   KST::scalarList.clear();
@@ -664,72 +759,81 @@ void KstDoc::deleteContents() {
 }
 
 void KstDoc::samplesUp() {
-  KstRVectorPtr V;
-  int f0, n, skip;
-  bool doSkip, doAve;
-  int fileN;
-
+  bool changed = false;
   KstRVectorList rvl = kstObjectSubList<KstVector,KstRVector>(KST::vectorList);
   for (int i = 0; i < (int)rvl.count(); i++) {
-    V = rvl[i];
+    KstRVectorPtr V = rvl[i];
     V->writeLock();
-    f0 = V->reqStartFrame();
-    n = V->reqNumFrames();
-    skip = V->skip();
-    doSkip = V->doSkip();
-    doAve =  V->doAve();
-    fileN = V->fileLength();
+    int f0 = V->reqStartFrame();
+    int n = V->reqNumFrames();
+    int skip = V->skip();
+    bool doSkip = V->doSkip();
+    bool doAve = V->doAve();
+    int fileN = V->fileLength();
 
-    if (f0 + n > fileN) {
+    bool didChange = false;
+    if (f0 + 2 * n > fileN) {
+      didChange = f0 != fileN - n;
+      changed = changed || didChange;
       f0 = fileN - n;
     } else {
+      didChange = true;
       f0 += n;
     }
-    V->changeFrames(f0, n, skip, doSkip, doAve);
+
+    if (didChange) {
+      V->changeFrames(f0, n, skip, doSkip, doAve);
+    }
     V->writeUnlock();
   }
 
-  setModified();
-  forceUpdate();
-
-  emit dataChanged();
+  if (changed) {
+    setModified();
+    forceUpdate();
+    emit dataChanged();
+  }
 }
 
-void KstDoc::samplesDown() {
-  KstRVectorPtr V;
-  int f0, n, skip;
-  bool doSkip, doAve;
-  int fileN;
 
+void KstDoc::samplesDown() {
+  bool changed = false;
   KstRVectorList rvl = kstObjectSubList<KstVector,KstRVector>(KST::vectorList);
   for (int i = 0; i < (int)rvl.count(); i++) {
-    V = rvl[i];
+    KstRVectorPtr V = rvl[i];
     V->writeLock();
-    f0 = V->reqStartFrame();
-    if (f0 == -1)
+    int f0 = V->reqStartFrame();
+    if (f0 == -1) {
       f0 = V->startFrame();
-    n = V->reqNumFrames();
-    skip = V->skip();
-    doSkip = V->doSkip();
-    doAve =  V->doAve();
-    fileN = V->fileLength();
+    }
+    int n = V->reqNumFrames();
+    int skip = V->skip();
+    bool doSkip = V->doSkip();
+    bool doAve =  V->doAve();
 
+    bool didChange = false;
     if (f0 - n < 0) {
+      didChange = f0 != 0;
+      changed = changed || didChange;
       f0 = 0;
     } else {
+      didChange = true;
       f0 -= n;
     }
 
-    V->changeFrames(f0, n, skip, doSkip, doAve);
+    if (didChange) {
+      V->changeFrames(f0, n, skip, doSkip, doAve);
+    }
     V->writeUnlock();
   }
 
-  setModified();
-  forceUpdate();
-
-  emit dataChanged();
+  if (changed) {
+    setModified();
+    forceUpdate();
+    emit dataChanged();
+  }
 }
 
+
 void KstDoc::samplesEnd() {
   KstRVectorPtr V;
 
@@ -747,12 +851,14 @@ void KstDoc::samplesEnd() {
   emit dataChanged();
 }
 
+
 void KstDoc::wasModified() {
   _modified = true;
   forceUpdate();
   QTimer::singleShot(0, this, SIGNAL(updateDialogs()));
 }
 
+
 RemoveStatus KstDoc::removeDataObject(const QString& tag) {
   KST::dataObjectList.lock().writeLock();
   KST::dataObjectList.removeTag(tag);
@@ -766,56 +872,103 @@ RemoveStatus KstDoc::removeDataObject(const QString& tag) {
 void KstDoc::purge() {
   QString purging = i18n("Purging unused objects");
   bool modified = false;
-  KstRVectorList rvl = kstObjectSubList<KstVector,KstRVector>(KST::vectorList);
+  bool again = true;
   KstApp *app = KstApp::inst();
 
-  KST::dataObjectList.lock().writeLock();
-  int cnt = rvl.count() + KST::dataObjectList.count();
-  int prg = 0;
-
-  app->slotUpdateProgress(cnt, prg, purging);
-
-  //
-  // ASSUMPTION: this only gets called from the data manager!
-  //
-  for (KstDataObjectList::Iterator it = KST::dataObjectList.begin(); it != KST::dataObjectList.end(); ++it) {
-    //kdDebug() << "OBJECT: " << (*it)->tagName() << " USAGE: " << (*it)->getUsage() << endl;
-    if ((*it)->getUsage() == 0) {
-      //kdDebug() << "    -> REMOVED" << endl;
-      KstDataObjectList::Iterator byebye = it;
-      --it;
-      KST::dataObjectList.remove(byebye);
-      modified = true;
-    }
-    prg++;
+  while (again) {
+    KST::dataObjectList.lock().readLock();
+    KST::matrixList.lock().readLock();
+    KST::vectorList.lock().readLock();
+    int cnt = KST::matrixList.count() + KST::vectorList.count() + KST::dataObjectList.count();
+    KST::vectorList.lock().readUnlock();
+    KST::matrixList.lock().readUnlock();
+    KST::dataObjectList.lock().readUnlock();
+    
+    int prg = 0;
     app->slotUpdateProgress(cnt, prg, purging);
-  }
-  KST::dataObjectList.lock().writeUnlock();
+    again = false;
+  
+    // ASSUMPTION: this only gets called from the data manager!
+    KST::dataObjectList.lock().writeLock();
+    for (KstDataObjectList::Iterator it = KST::dataObjectList.begin(); it != KST::dataObjectList.end(); ++it) {
+      //kstdDebug() << "OBJECT: " << (*it)->tagName() << " USAGE: " << (*it)->getUsage() << endl;
+      if ((*it)->getUsage() == 0) {
+        //kstdDebug() << "    -> REMOVED" << endl;
+        KstDataObjectList::Iterator byebye = it;
+        --it;
+        KST::dataObjectList.remove(byebye);
+        again = true;
+        modified = true;
+      }
+      ++prg;
+      app->slotUpdateProgress(cnt, prg, purging);
+    }
+    KST::dataObjectList.lock().writeUnlock();
+  
+    KST::vectorList.lock().readLock();
+    KstVectorList vectorList = QDeepCopy<KstVectorList>(KST::vectorList);
+    KST::vectorList.lock().readUnlock();
+    
+    // clear unused vectors that are editable 
+    for (KstVectorList::ConstIterator it = vectorList.begin(); it != vectorList.end(); ++it) {
+      //kstdDebug() << "VECTOR: " << (*it)->tagName() << " USAGE: " << (*it)->getUsage() << endl;
+      if ((*it)->getUsage() == 1) {
+        //kstdDebug() << "    -> REMOVED" << endl;
+        KST::vectorList.lock().writeLock();
+        KST::vectorList.remove(const_cast<KstVector*>((*it).data()));
+        KST::vectorList.lock().writeUnlock();
+        again = true;
+        modified = true;
+      }
+      ++prg;
+      app->slotUpdateProgress(cnt, prg, purging);
+    }
+    
+    KST::matrixList.lock().readLock();
+    KstMatrixList matrixList = QDeepCopy<KstMatrixList>(KST::matrixList);
+    KST::matrixList.lock().readUnlock();
 
-  for (KstRVectorList::Iterator it = rvl.begin(); it != rvl.end(); ++it) {
-    //kdDebug() << "VECTOR: " << (*it)->tagName() << " USAGE: " << (*it)->getUsage() << endl;
-    if ((*it)->getUsage() == 1) {
-      //kdDebug() << "    -> REMOVED" << endl;
-      KST::vectorList.lock().writeLock();
-      KST::vectorList.remove((*it).data());
-      KST::vectorList.lock().writeUnlock();
-      modified = true;
+    // clear unused matrices that are editable
+    for (KstMatrixList::ConstIterator it = matrixList.begin(); it != matrixList.end(); ++it) {
+      if ((*it)->getUsage() == 1) {
+        //kstdDebug() << "    -> REMOVED" << endl;
+        KST::matrixList.lock().writeLock();
+        KST::matrixList.remove(const_cast<KstMatrix*>((*it).data()));
+        KST::matrixList.lock().writeUnlock();
+        again = true;
+        modified = true;
+      }
+      ++prg;
+      app->slotUpdateProgress(cnt, prg, purging);
     }
-    prg++;
-    app->slotUpdateProgress(cnt, prg, purging);
   }
 
-  app->slotUpdateProgress(0, 0, QString::null);
-
-  rvl.clear();
+  KstDataSourceList dataList;
+  KST::dataSourceList.lock().readLock();
+  for (KstDataSourceList::ConstIterator it = KST::dataSourceList.begin(); it != KST::dataSourceList.end(); ++it) {
+      if ((*it)->getUsage() == 1) {
+        //kstdDebug() << "    -> REMOVED" << endl;
+        dataList.append(const_cast<KstDataSource*>((*it).data()));
+        modified = true;
+      }
+  }
+  KST::dataSourceList.lock().readUnlock();
+  KST::dataSourceList.lock().writeLock();
+  for (KstDataSourceList::ConstIterator it = dataList.begin(); it != dataList.end(); ++it) {
+    KST::dataSourceList.remove(const_cast<KstDataSource*>((*it).data()));
+  }
+  KST::dataSourceList.lock().writeUnlock();
+ 
   setModified(modified);
   emit updateDialogs();
+  
+  app->slotUpdateProgress(0, 0, QString::null);
 }
 
 
 void KstDoc::forceUpdate() {
   _nextEventPaint = false;
-  static_cast<KstApp*>(parent())->forceUpdate();
+  KstApp::inst()->forceUpdate();
 }
 
 
@@ -831,16 +984,47 @@ bool KstDoc::event(QEvent *e) {
     ThreadEvent *te = static_cast<ThreadEvent*>(e);
     switch (te->_eventType) {
       case ThreadEvent::UpdateDataDialogs:
-        //kdDebug() << "Update data dialogs" << endl;
-        emit dataChanged();
-        KstApp::inst()->paintAll();
+        {
+          emit dataChanged();
+          KstApp::inst()->updateDataNotifier();
+          // HACK: remove me later
+          KMdiIterator<KMdiChildView*> *it = KstApp::inst()->createIterator();
+          if (it) {
+            while (it->currentItem()) {
+              KstViewWindow *view = dynamic_cast<KstViewWindow*>(it->currentItem());
+              if (!view) {
+                it->next();
+                continue;
+              }
+
+              Kst2DPlotList pl = view->view()->findChildrenType<Kst2DPlot>(true);
+              for (Kst2DPlotList::Iterator i = pl.begin(); i != pl.end(); ++i) {
+                for (QValueList<KstBaseCurve*>::ConstIterator j = te->_curves.begin(); j != te->_curves.end(); ++j) {
+                  // race: if ((*i)->Curves.contains(*j)) {
+                  const KstBaseCurveList& cl = (*i)->Curves;
+                  bool doBreak = false;
+                  for (KstBaseCurveList::ConstIterator k = cl.begin(); k != cl.end(); ++k) {
+                    if (*j == *k) {
+                      (*i)->setDirty();
+                      doBreak = true;
+                      break;
+                    }
+                  }
+                  if (doBreak) {
+                    break;
+                  }
+                }
+              }
+              it->next();
+            }
+            KstApp::inst()->deleteIterator(it);
+          }
+          KstApp::inst()->paintAll(P_UPDATE);
+        }
         break;
       case ThreadEvent::UpdateAllDialogs:
-        //kdDebug() << "Update ALL dialogs" << endl;
-
-        if (KstApp::inst()) { // If KstApp::inst() is null, we have big
-                              // problems since this is a QObject child of
-                              // KstApp
+        //kstdDebug() << "Update ALL dialogs" << endl;
+        {
           KMdiIterator<KMdiChildView*>* it = KstApp::inst()->createIterator();
           if (it) {
             while (it->currentItem()) {
@@ -852,9 +1036,9 @@ bool KstDoc::event(QEvent *e) {
             }
             KstApp::inst()->deleteIterator(it);
           }
-        }
 
-        emit updateDialogs();
+          emit updateDialogs();
+        }
         break;
       case ThreadEvent::Done:
         break;
@@ -873,6 +1057,24 @@ bool KstDoc::event(QEvent *e) {
     _nextEventPaint = false;
     QTimer::singleShot(0, this, SLOT(enableUpdates()));
 
+    return true;
+  } else if (e->type() == KstEventTypeLog) {
+    LogEvent *le = dynamic_cast<LogEvent*>(e);
+    if (le) {
+      switch (le->_eventType) {
+        case LogEvent::LogAdded:
+          emit logAdded(le->_msg);
+          if (le->_msg.level == KstDebug::Error) {
+            QTimer::singleShot(0, KstApp::inst(), SLOT(createDebugNotifier()));
+          }
+          break;
+        case LogEvent::LogCleared:
+          emit logCleared();
+          break;
+        default:
+          break;
+      }
+    }
     return true;
   }
 
@@ -900,7 +1102,10 @@ void KstDoc::createScalars() const {
   new KstScalar("CONST_MKSA_ELECTRON_CHARGE", 1.602176462e-19, false);
   new KstScalar("CONST_MKSA_MOLAR_GAS", 8.314472e0, false);
   new KstScalar("CONST_MKSA_STANDARD_GAS_VOLUME", 2.2710981e-2, false);
-  new KstScalar("CONST_PI", 3.141592654e0, false);
+  new KstScalar("CONST_PI", M_PI, false);
+  new KstScalar("C_PI", M_PI, false);
+  new KstScalar("C_R2D", 180.0/M_PI, false); // radians to degrees
+  new KstScalar("C_D2R", M_PI/180.0, false); // degrees to radians
 }
 
 
