@@ -25,18 +25,26 @@
 
 // application specific includes
 #include "kst.h"
+#include "kst2dplot.h"
 #include "kstcolorsequence.h"
 #include "kstdoc.h"
 #include "kstequation.h"
+#include "ksteventmonitorentry.h"
 #include "ksthistogram.h"
 #include "kstiface_impl.h"
+#include "kstimage.h"
 #include "kstplugin.h"
 #include "kstpsd.h"
+#include "kstrmatrix.h"
 #include "kstrvector.h"
+#include "kstsmatrix.h"
+#include "kstsvector.h"
 #include "kstvcurve.h"
 #include "kstviewwindow.h"
 #include "plugincollection.h"
 
+#include "kmessagebox.h"
+
 KstIfaceImpl::KstIfaceImpl(KstDoc *doc, KstApp *app)
 : DCOPObject("KstIface"), _doc(doc), _app(app) {
   assert(doc);
@@ -183,7 +191,7 @@ bool KstIfaceImpl::plotEquation(double start, double end, int numSamples, const
     return false;
   }
 
-  etag = "Eq-" + QString(equation).replace(QRegExp("[\\[\\]\\s]"), "_");
+  etag = KST::suggestEQName(QString(equation).replace(QRegExp("[\\[\\]\\s]"), "_"));
   ptag = "P-" + plotName;
 
   if (!plotName.isEmpty()) {
@@ -227,17 +235,15 @@ bool KstIfaceImpl::plotEquation(double start, double end, int numSamples, const
     }
   }
 
-  while (KST::dataTagNameNotUnique(etag, false)) {
-    etag += '\'';
-  }
-
   KstEquationPtr eq = new KstEquation(etag, equation, start, end, numSamples);
 
   if (!eq->isValid()) {
     return false;
   }
+  
+  KstVCurveList vcurves = kstObjectSubList<KstBaseCurve,KstVCurve>(plot->Curves);
 
-  KstVCurvePtr vc = new KstVCurve(etag + "-curve", eq->vX(), eq->vY(), 0L, 0L, color.isValid() ? color : KstColorSequence::next(plot->Curves,plot->backgroundColor()));
+  KstVCurvePtr vc = new KstVCurve(KST::suggestCurveName(etag, true), eq->vX(), eq->vY(), 0L, 0L, 0L, 0L, color.isValid() ? color : KstColorSequence::next(vcurves,plot->backgroundColor()));
   KST::dataObjectList.lock().writeLock();
   KST::dataObjectList.append(KstDataObjectPtr(eq));
   KST::dataObjectList.append(KstDataObjectPtr(vc));
@@ -277,7 +283,7 @@ bool KstIfaceImpl::plotEquation(const QString& xvector, const QString& equation,
 
   v = *it;
 
-  etag = "Eq-" + QString(equation).replace(QRegExp("[\\[\\]\\s]"), "_");
+  etag = KST::suggestEQName(QString(equation).replace(QRegExp("[\\[\\]\\s]"), "_"));
   ptag = "P-" + plotName;
 
   if (!plotName.isEmpty()) {
@@ -322,9 +328,6 @@ bool KstIfaceImpl::plotEquation(const QString& xvector, const QString& equation,
     }
   }
 
-  while (KST::dataTagNameNotUnique(etag, false)) {
-    etag += '\'';
-  }
 
   KstEquationPtr eq = new KstEquation(etag, equation, v, true);
 
@@ -332,14 +335,16 @@ bool KstIfaceImpl::plotEquation(const QString& xvector, const QString& equation,
     return false;
   }
 
-  KstVCurvePtr vc = new KstVCurve(etag + "-curve", eq->vX(), eq->vY(), 0L, 0L, color.isValid() ? color : KstColorSequence::next(plot->Curves,plot->backgroundColor()));
+  KstVCurveList vcurves = kstObjectSubList<KstBaseCurve,KstVCurve>(plot->Curves);
+  
+  KstVCurvePtr vc = new KstVCurve(KST::suggestCurveName(etag, true), eq->vX(), eq->vY(), 0L, 0L, 0L, 0L, color.isValid() ? color : KstColorSequence::next(vcurves,plot->backgroundColor()));
   KST::dataObjectList.lock().writeLock();
   KST::dataObjectList.append(KstDataObjectPtr(eq));
   KST::dataObjectList.append(KstDataObjectPtr(vc));
   KST::dataObjectList.lock().writeUnlock();
 
   plot->Curves.append(KstBaseCurvePtr(vc));
-  
+
   _doc->forceUpdate();
   _doc->setModified();
 
@@ -350,12 +355,14 @@ const QString& KstIfaceImpl::generateScalar(const QString& name, double value) {
   KstScalarPtr s = new KstScalar(name, value);
   KstReadLocker rl(s);
   s->setOrphan(true);
+  s->setEditable(true);
   return s->tagName();
 }
 
 
 const QString& KstIfaceImpl::generateVector(const QString& name, double from, double to, int points) {
-  KstVectorPtr v = KstVector::generateVector(from, to, points, name);
+  KstVectorPtr v = new KstSVector(from, to, points, name);
+  KST::addVectorToList(v);
   KstReadLocker rl(v);
   return v->tagName();
 }
@@ -456,7 +463,7 @@ bool KstIfaceImpl::setScalar(const QString& name, double value) {
   KstReadLocker ml(&KST::scalarList.lock());
   KstScalarList::Iterator it = KST::scalarList.findTag(name);
 
-  if (it == KST::scalarList.end()) {
+  if (it == KST::scalarList.end() || !(*it)->editable()) {
     return false;
   }
 
@@ -600,7 +607,7 @@ QString KstIfaceImpl::createPlot(const QString& window, const QString& name) {
   KstApp*           pApp = KstApp::inst();
   Kst2DPlotPtr      p;
   QString           plotName;
-  
+
   pWindow = pApp->findWindow(window);
   if (pWindow) {
     pView = dynamic_cast<KstViewWindow*>(pWindow);
@@ -758,7 +765,7 @@ bool KstIfaceImpl::removeCurveFromPlot(const QString& window, const QString& plo
 bool KstIfaceImpl::removeCurveFromPlot(const QString& plot, const QString& curve) {
   KstApp *app = KstApp::inst();
   KMdiChildView *activewin = app->activeWindow();
-  
+
   return removeCurveFromPlot(activewin, plot, curve);
 }
 
@@ -782,7 +789,7 @@ const QString& KstIfaceImpl::createCurve(const QString& name, const QString& xVe
     n += "'";
   }
 
-  KstVCurvePtr c = new KstVCurve(n, vx, vy, ex, ey, color);
+  KstVCurvePtr c = new KstVCurve(n, vx, vy, ex, ey, ex, ey, color);
   KST::dataObjectList.append(KstDataObjectPtr(c));
   KST::dataObjectList.lock().writeUnlock();
   _doc->forceUpdate();
@@ -841,7 +848,7 @@ bool KstIfaceImpl::printPostScript(const QString& url) {
     app->immediatePrintActiveWindowToFile(url);
     bRetVal = true;
   }
-  
+
   return bRetVal;
 }
 
@@ -869,7 +876,7 @@ const QString& KstIfaceImpl::loadVector(const QString& file, const QString& fiel
   KstDataSourcePtr src;
   /* generate or find the kstfile */
   KST::dataSourceList.lock().writeLock();
-  KstDataSourceList::Iterator it = KST::dataSourceList.findFileName(file);
+  KstDataSourceList::Iterator it = KST::dataSourceList.findReusableFileName(file);
 
   if (it == KST::dataSourceList.end()) {
     src = KstDataSource::loadSource(file);
@@ -1015,7 +1022,7 @@ bool KstIfaceImpl::setTileMode(int mode) {
 bool KstIfaceImpl::closeWindow(const QString& windowname) {
   KstApp *app = KstApp::inst();
   KMdiChildView *win = app->findWindow(windowname);
-  
+
   if (win) {
     app->closeWindow(win, true);
     return true;
@@ -1040,7 +1047,7 @@ bool KstIfaceImpl::activateWindow(const QString& windowname) {
 bool KstIfaceImpl::positionWindow(int topleftX, int topleftY, int width, int height) {
   KstApp *app = KstApp::inst();
   KMdiChildView *win = app->activeWindow();
-  
+
   if (win) {
     if (topleftX >= 0 && topleftY >= 0 && width >= 0 && height >= 0){
       QRect rectangle(topleftX, topleftY, width, height);
@@ -1063,7 +1070,7 @@ bool KstIfaceImpl::maximizePlot(const QString& plotName) {
       Kst2DPlotList::Iterator plot_iter = plotlist.findTag(plotName);
       if (plot_iter != plotlist.end()) {
         app->deleteIterator(iter);
-        (*plot_iter)->zoom(true);
+        (*plot_iter)->setMaximized(true);
         return true;
       }
     }
@@ -1084,7 +1091,7 @@ bool KstIfaceImpl::unmaximizePlot(const QString& plotName) {
       Kst2DPlotList::Iterator plot_iter = plotlist.findTag(plotName);
       if (plot_iter != plotlist.end()) {
         app->deleteIterator(iter);
-        (*plot_iter)->zoom(false);
+        (*plot_iter)->setMaximized(false);
         return true;
       }
     }
@@ -1131,8 +1138,15 @@ bool KstIfaceImpl::setPlotAxes(const QString& plotName,
       Kst2DPlotList::Iterator plot_iter=plotlist.findTag(plotName);
       if (plot_iter != plotlist.end()) {
         app->deleteIterator(iter);
+      
+        (*plot_iter)->setXScaleMode(FIXED);
+        (*plot_iter)->setYScaleMode(FIXED);
         (*plot_iter)->setScale(XLower, YLower,
-                               XUpper, YUpper);  //set the scale
+                              XUpper, YUpper);  //set the scale
+        
+        // repaint the plot 
+        (*plot_iter)->setDirty();
+        viewwindow->view()->paint(P_PLOT);
         return true;
       }
     }
@@ -1150,13 +1164,55 @@ QString KstIfaceImpl::createHistogram(const QString& name,
     int normalizationType,
     const QColor& color) {
 
+  QStringList objList = createHistogram(name, vector, min, max, numBins, normalizationType);
+
+  if (objList.isEmpty())
+  {
+    return QString::null;
+  }
+  
+  // also create the curve for the histogram
+  QString n = objList[0] + "-C";
+  KST::vectorList.lock().readLock();
+  KstVectorPtr vx = *KST::vectorList.findTag(objList[1]);
+  KstVectorPtr vy = *KST::vectorList.findTag(objList[2]);
+  KST::vectorList.lock().readUnlock();
+
+  KST::dataObjectList.lock().readLock();
+  while (KST::dataObjectList.findTag(n) != KST::dataObjectList.end()) {
+    n += "'";
+  }
+  KST::dataObjectList.lock().readUnlock();
+
+  KstVCurvePtr c = new KstVCurve(n, vx, vy, 0L, 0L, 0L, 0L, color);
+  c->setHasPoints(false);
+  c->setHasLines(false);
+  c->setHasBars(true);
+  c->setBarStyle(1);
+  
+  KST::dataObjectList.lock().writeLock();
+  KST::dataObjectList.append(KstDataObjectPtr(c));
+  KST::dataObjectList.lock().writeUnlock();
+
+  _doc->forceUpdate();
+  _doc->setModified();
+  return c->tagName(); //return the curve name so user can plot it
+}
+
+QStringList KstIfaceImpl::createHistogram(const QString& name,
+                                          const QString& vector,
+                                          double min,
+                                          double max,
+                                          int numBins,
+                                          int normalizationType)
+{
   //get the vector
   KST::vectorList.lock().readLock();
   KstVectorList::Iterator iter = KST::vectorList.findTag(vector);
   KST::vectorList.lock().readUnlock();
 
   if (iter == KST::vectorList.end()) {
-    return QString();
+    return QStringList();
   }
 
   //set the normalization type
@@ -1165,18 +1221,18 @@ QString KstIfaceImpl::createHistogram(const QString& name,
   switch (normalizationType) {
     case 1:
       normtype = KST_HS_NUMBER;
-    break;
+      break;
     case 2:
       normtype = KST_HS_PERCENT;
-    break;
+      break;
     case 3:
       normtype = KST_HS_FRACTION;
-    break;
+      break;
     case 4:
       normtype = KST_HS_MAX_ONE;
     default:
       normtype = KST_HS_NUMBER;
-    break;
+      break;
   }
 
   //suggest a name if not supplied
@@ -1192,24 +1248,29 @@ QString KstIfaceImpl::createHistogram(const QString& name,
   QString stringnum;
   stringnum = stringnum.setNum(i);
 
-  QString htag = "C" + stringnum + "-HS_" + htag_end;
+  QString htag = stringnum + "-" + htag_end;
 
   while (KST::dataTagNameNotUnique(htag, false)) {
     stringnum.setNum(++i);
-    htag = "C" + stringnum + "-HS_" + htag_end;
+    htag = stringnum + "-" + htag_end;
   }
 
   KstHistogramPtr histogram = new KstHistogram(htag, *iter, min, max, numBins,
-                               normtype);
+                                               normtype);
 
   KST::dataObjectList.lock().writeLock();
   KST::dataObjectList.append(KstDataObjectPtr(histogram));
   KST::dataObjectList.lock().writeUnlock();
 
+  QStringList returnList;
+  returnList.push_back(histogram->tagName());
+  returnList.push_back(histogram->xVTag());
+  returnList.push_back(histogram->yVTag());
+  
   _doc->forceUpdate();
   _doc->setModified();
-
-  return htag;
+  
+  return returnList;
 }
 
 QString KstIfaceImpl::createPowerSpectrum(const QString & name,
@@ -1221,13 +1282,58 @@ QString KstIfaceImpl::createPowerSpectrum(const QString & name,
     double sampleRate,
     const QString& vectorUnits,
     const QColor& color) {
+
+  QStringList objList = createPowerSpectrum(name, vector, appodize, removeMean,
+                                            fftLength, rateUnits, sampleRate, 
+                                            vectorUnits);
+  if (objList.isEmpty())
+  {
+    return QString::null;
+  }
+  
+  KST::vectorList.lock().readLock();
+  KstVectorPtr vx = *KST::vectorList.findTag(objList[1]);
+  KstVectorPtr vy = *KST::vectorList.findTag(objList[2]);
+  KST::vectorList.lock().readUnlock();
+  
+  QString n = objList[0] + "-C";
+  KST::dataObjectList.lock().readLock();
+  while (KST::dataObjectList.findTag(n) != KST::dataObjectList.end()) {
+    n += "'";
+  }
+  KST::dataObjectList.lock().readUnlock();
+  
+  // create curve as well (but don't plot the curve)
+  KstVCurvePtr vc = new KstVCurve(n, vx, vy, 
+                                  0L, 0L, 0L, 0L, 
+                                  color.isValid() ? color : QColor("darkBlue"));
+
+  KST::dataObjectList.lock().writeLock();
+  KST::dataObjectList.append(KstDataObjectPtr(vc));
+  KST::dataObjectList.lock().writeUnlock();
+
+  _doc->forceUpdate();
+  _doc->setModified();
+
+  return vc->tagName(); //return the curve name so user can plot it
+}
+                                       
+QStringList KstIfaceImpl::createPowerSpectrum(const QString& name,
+                                              const QString& vector,
+                                              bool appodize,
+                                              bool removeMean,
+                                              int fftLength,
+                                              const QString& rateUnits,
+                                              double sampleRate,
+                                              const QString& vectorUnits)
+{
   //get the vector
   KST::vectorList.lock().readLock();
   KstVectorList::Iterator iter = KST::vectorList.findTag(vector);
   KST::vectorList.lock().readUnlock();
 
   if (iter == KST::vectorList.end()) {
-    return QString::null;
+    return QStringList();
   }
 
   //suggest a name if not supplied
@@ -1242,29 +1348,29 @@ QString KstIfaceImpl::createPowerSpectrum(const QString & name,
   int i = pslist.count() + 1;
   QString stringnum = QString::number(i);
 
-  QString pstag = "C" + stringnum + "-PSD_" + pstag_end;
+  QString pstag = stringnum + "-" + pstag_end;
 
   while (KST::dataTagNameNotUnique(pstag, false)) {
     stringnum.setNum(++i);
-    pstag = "C" + stringnum + "-PSD_" + pstag_end;
+    pstag = stringnum + "-" + pstag_end;
   }
 
   KstPSDPtr powerspectrum = new KstPSD(pstag, *iter, sampleRate, true, fftLength,
-                               appodize, removeMean, vectorUnits, rateUnits);
+                                       appodize, removeMean, vectorUnits, rateUnits, 0);
   
-  // FIXME: should we be plotting this?  Otherwise, remove color
-  //KstVCurvePtr vc = new KstVCurve(pstag + "-curve", *iter, powerspectrum->vY(), 0L, 0L, color.isValid() ? color : QColor("darkBlue"));
-
   KST::dataObjectList.lock().writeLock();
   KST::dataObjectList.append(KstDataObjectPtr(powerspectrum));
-  //KST::dataObjectList.append(KstDataObjectPtr(vc));
   KST::dataObjectList.lock().writeUnlock();
-
+  
+  QStringList returnList;
+  returnList.push_back(powerspectrum->tagName());
+  returnList.push_back(powerspectrum->xVTag());
+  returnList.push_back(powerspectrum->yVTag());
+  
   _doc->forceUpdate();
   _doc->setModified();
-
-  return pstag;
-
+  
+  return returnList;
 }
 
 QString KstIfaceImpl::createEvent(const QString& name,
@@ -1318,7 +1424,7 @@ QString KstIfaceImpl::createEvent(const QString& name,
   else {
     event->setLogKstDebug(false);
   }
-  
+
   if (email.isEmpty()) {
     event->setLogEMail(false);
   } else {
@@ -1386,7 +1492,7 @@ QStringList KstIfaceImpl::createPlugin(const QString& pluginName,
       if (scalarParamIter != scalarInputs.end()) {
         KST::scalarList.lock().readLock();
         KstScalarList::Iterator iter = KST::scalarList.findTag(*scalarParamIter);
-        KST::vectorList.lock().readUnlock();
+        KST::scalarList.lock().readUnlock();
         if (iter != KST::scalarList.end()) {
           kstplug_ptr->writeLock();
           kstplug_ptr->inputScalars().insert((*IOIter)._name, *iter);
@@ -1477,65 +1583,104 @@ QStringList KstIfaceImpl::createPlugin(const QString& pluginName,
   return outputList;
 }
 
-QString KstIfaceImpl::createMatrix(const QString& name, const QString& vector, double xMin, double yMin,
-                                                   double xStepSize, double yStepSize,
-                                                   uint xNumSteps, uint yNumSteps,
-                                                   bool useMaxX) {
-  //get the vector
-  KST::vectorList.lock().readLock();
-  KstVectorList::Iterator iter = KST::vectorList.findTag(vector);
-  KST::vectorList.lock().readUnlock();
 
-  if (iter == KST::vectorList.end()) {
-    return QString();
-  }
+QString KstIfaceImpl::loadMatrix(const QString& name, const QString& file, const QString& field,
+                                int xStart, int yStart, int xNumSteps, int yNumSteps, 
+                                int skipFrames, bool boxcarFilter) {
+  KstDataSourcePtr src;
+  /* generate or find the kstfile */
+  KST::dataSourceList.lock().writeLock();
+  KstDataSourceList::Iterator it = KST::dataSourceList.findReusableFileName(file);
 
-  //suggest a name if not supplied
-  QString artag;
-  if (name.isEmpty()) {
-    artag = KST::suggestMatrixName(vector);
+  if (it == KST::dataSourceList.end()) {
+    src = KstDataSource::loadSource(file);
+    if (!src || !src->isValid()) {
+      KST::dataSourceList.lock().writeUnlock();
+      return QString::null;
+    }
+    if (src->isEmpty()) {
+      KST::dataSourceList.lock().writeUnlock();
+      return QString::null;
+    }
+    KST::dataSourceList.append(src);
   } else {
-    QString artag_end = QString(name);
-    //count number of data objects and make a unique name
-    int i = KST::dataObjectList.count() + 1;
-    QString artag = "A" + QString::number(i) + "-" + artag_end;
+    src = *it;
   }
-  while (KST::dataTagNameNotUnique(artag, false)) {
-    artag += "\'";
+  src->writeLock();
+  KST::dataSourceList.lock().writeUnlock();
+
+  // make sure field is valid
+  if (!src->isValidMatrix(field)) {
+    src->writeUnlock();
+    return QString::null;  
+  }
+  
+  // make sure name is unique, else generate a unique one
+  KST::matrixList.lock().readLock();
+ 
+  QString matrixName;
+  if (name.isEmpty()) {
+    matrixName = "M" + QString::number(KST::matrixList.count() + 1); 
+  } else {
+    matrixName = name;  
   }
 
-  //basic check to see if parameters are consistent with vector
-  (*iter)->readLock();
-  if (!useMaxX && (*iter)->sampleCount() < (int)(xNumSteps * yNumSteps)) {
-    (*iter)->readUnlock();
-    return QString();
+  while (KST::matrixTagNameNotUnique(matrixName, false)) {
+    matrixName = "M" + QString::number(KST::matrixList.count() + 1);
+  }
+  KST::matrixList.lock().readUnlock();
+
+  KstMatrixPtr p = new KstRMatrix(src, field, matrixName, xStart, yStart, xNumSteps, yNumSteps, 
+                                  boxcarFilter, skipFrames > 0, skipFrames);
+  KST::addMatrixToList(p);
+
+  src->writeUnlock();
+
+  if (p) {
+    _doc->forceUpdate();
+    _doc->setModified();
+    return p->tagName();
   }
 
-  //can't let yNumSteps be less than 1, or will divide by 0
-  if (yNumSteps < 1) {
-    (*iter)->readUnlock();
-    return QString();
-  }
-
-  if (xStepSize <= 0 || yStepSize <= 0) {
-    (*iter)->readUnlock();
-    return QString();
-  }
-  (*iter)->readUnlock();
-
-  //create the matrix
-  KstMatrixPtr matrix = new KstMatrix(artag, *iter, xNumSteps, yNumSteps, xMin, yMin, xStepSize, yStepSize, useMaxX);
-
-  KST::dataObjectList.lock().writeLock();
-  KST::dataObjectList.append(KstDataObjectPtr(matrix));
-  KST::dataObjectList.lock().writeUnlock();
-
-  _doc->forceUpdate();
-  _doc->setModified();
-
-  return artag;
+  return QString::null;
 }
 
+  
+QString KstIfaceImpl::createGradient(const QString& name, bool xDirection, double zAtMin, double zAtMax, 
+                                     int xNumSteps, int yNumSteps, double xMin, double yMin, 
+                                     double xStepSize, double yStepSize) {
+  // make sure tag name is unique
+  // make sure name is unique, else generate a unique one
+  KST::matrixList.lock().readLock();
+
+  QString matrixName;
+  if (name.isEmpty()) {
+    matrixName = "M" + QString::number(KST::matrixList.count() + 1); 
+  } else {
+    matrixName = name;  
+  }
+
+  while (KST::matrixTagNameNotUnique(matrixName, false)) {
+    matrixName = "M" + QString::number(KST::matrixList.count() + 1);
+  }
+  KST::matrixList.lock().readUnlock();
+  
+  // create the gradient matrix
+  KstMatrixPtr p = new KstSMatrix(matrixName, xNumSteps, yNumSteps, xMin, yMin, xStepSize, yStepSize, 
+                                  zAtMin, zAtMax, xDirection);
+  
+  KST::addMatrixToList(p);
+
+  if (p) {
+    _doc->forceUpdate();
+    _doc->setModified();
+    return p->tagName();
+  }
+
+  return QString::null;
+}
+                
+
 QString KstIfaceImpl::createImage(const QString &name,
                                   const QString &in_matrix,
                                   double lowerZ,
@@ -1559,7 +1704,7 @@ QString KstIfaceImpl::createImage(const QString &name,
     QString imgtag_end = QString(name);
     //count number of data objects and make a unique name
     int i = KST::dataObjectList.count() + 1;
-    QString imgtag = "Img" + QString::number(i) + "-" + imgtag_end;
+    imgtag = QString::number(i) + "-" + imgtag_end;
   }
   while (KST::dataTagNameNotUnique(imgtag, false)) {
     imgtag += "\'";
@@ -1589,13 +1734,10 @@ QString KstIfaceImpl::createImage(const QString &name,
     if (lowerZ > upperZ) {
       return QString::null;
     }
-    KPalette* pal = new KPalette(paletteName);
-    if (!pal) {
-      return QString::null;
-    }
     if (numContours < 1) {
       return QString::null;
     }
+    KPalette* pal = new KPalette(paletteName);
     matrix->readLock();
     image = new KstImage(imgtag, matrix, lowerZ, upperZ, false, pal,
                          numContours, contourColor.isValid() ? contourColor : QColor("darkBlue"), 0);
@@ -1627,6 +1769,9 @@ bool KstIfaceImpl::addPlotMarker(const QString &plotName, double markerValue) {
       Kst2DPlotList::Iterator plot_iter=plotlist.findTag(plotName);
       if (plot_iter != plotlist.end() && (*plot_iter)->setPlotMarker(markerValue)) {
         app->deleteIterator(iter);
+        // repaint the plot 
+        (*plot_iter)->setDirty();
+        viewwindow->view()->paint(P_PLOT);
         return true;
       }
     }
@@ -1635,4 +1780,85 @@ bool KstIfaceImpl::addPlotMarker(const QString &plotName, double markerValue) {
   app->deleteIterator(iter);
   return false;
 }
+
+
+bool KstIfaceImpl::changeDataFile(const QString& fileName) {
+  QStringList vl;
+  KstRVectorList rvl = kstObjectSubList<KstVector,KstRVector>(KST::vectorList);
+  for (KstRVectorList::ConstIterator i = rvl.begin(); i != rvl.end(); ++i) {
+    vl += (*i)->tagName();
+  }
+  return changeDataFiles(vl, fileName);
+}
+
+
+bool KstIfaceImpl::changeDataFile(const QString& vector, const QString& fileName) {
+  return changeDataFile(vector, fileName, true);
+}
+
+
+bool KstIfaceImpl::changeDataFile(const QString& vector, const QString& fileName, bool update) {
+  KST::vectorList.lock().readLock();
+  KstRVectorPtr rvp = kst_cast<KstRVector>(*KST::vectorList.findTag(vector));
+  KST::vectorList.lock().readUnlock();
+  if (!rvp) {
+    return false;
+  }
+
+  KST::dataSourceList.lock().writeLock();
+  KstDataSourceList::Iterator it = KST::dataSourceList.findReusableFileName(fileName);
+  KstDataSourcePtr file;
+  QString invalidSources;
+
+  if (it == KST::dataSourceList.end()) {
+    file = KstDataSource::loadSource(fileName);
+    if (!file || !file->isValid() || file->isEmpty()) {
+      KST::dataSourceList.lock().writeUnlock();
+      return false;
+    }
+    KST::dataSourceList.append(file);
+  } else {
+    file = *it;
+  }
+
+  KST::dataSourceList.lock().writeUnlock();
+
+  rvp->writeLock();
+  file->writeLock();
+
+  if (!file->isValidField(vector)) {
+    file->writeUnlock();
+    rvp->writeUnlock();
+    return false;
+  }
+
+  rvp->changeFile(file);
+  
+  file->writeUnlock();
+  bool rc = rvp->isValid();
+  rvp->writeUnlock();
+
+  if (update) {
+    KstApp::inst()->forceUpdate();
+  }
+
+  return rc;
+}
+
+
+bool KstIfaceImpl::changeDataFiles(const QStringList& vectors, const QString& fileName) {
+  bool rc = true;
+  bool any = false;
+  for (QStringList::ConstIterator i = vectors.begin(); i != vectors.end(); ++i) {
+    bool thisrc = changeDataFile(*i, fileName, false);
+    any = thisrc || any;
+    rc = thisrc && rc;
+  }
+
+  KstApp::inst()->forceUpdate();
+
+  return rc;
+}
+
+
 // vim: ts=2 sw=2 et
