@@ -22,7 +22,7 @@
 #include <qstylesheet.h>
 
 // include files for KDE
-#include <kdebug.h>
+#include "ksdebug.h"
 #include <klocale.h>
 
 // application specific includes
@@ -33,14 +33,16 @@
 #include "plugincollection.h"
 
 
-KstPlugin::KstPlugin() : KstDataObject() {
+KstPlugin::KstPlugin()
+: KstDataObject() {
   _inStringCnt = 0;
   _outStringCnt = 0;
   commonConstructor();
 }
 
 
-KstPlugin::KstPlugin(QDomElement &e) : KstDataObject(e) {
+KstPlugin::KstPlugin(const QDomElement& e)
+: KstDataObject(e) {
   QString pluginName;
 
   _inStringCnt = 0;
@@ -60,10 +62,10 @@ KstPlugin::KstPlugin(QDomElement &e) : KstDataObject(e) {
         _inputVectorLoadQueue.append(qMakePair(e.attribute("name"), e.text()));
       } else if (e.tagName() == "iscalar") {
         _inputScalarLoadQueue.append(qMakePair(e.attribute("name"), e.text()));
+      } else if (e.tagName() == "istring") {
+        _inputStringLoadQueue.append(qMakePair(e.attribute("name"), e.text()));
       } else if (e.tagName() == "ovector") {
         KstVectorPtr v;
-
-        // FIXME: these might already exist!!
         if (e.attribute("scalarList", "0").toInt()) {
           v = new KstVector(e.text(), 0, true);
         } else {
@@ -73,8 +75,17 @@ KstPlugin::KstPlugin(QDomElement &e) : KstDataObject(e) {
         _outputVectors.insert(e.attribute("name"), v);
         KST::addVectorToList(v);
       } else if (e.tagName() == "oscalar") {
-        // FIXME: these might already exist!!
-        _outputScalars.insert(e.attribute("name"), new KstScalar(e.text()));
+        KstScalarPtr sp = new KstScalar(e.text());
+        sp->writeLock();
+        sp->setProvider(this);
+        sp->writeUnlock();
+        _outputScalars.insert(e.attribute("name"), sp);
+      } else if (e.tagName() == "ostring") {
+        KstStringPtr sp = new KstString(e.text());
+        sp->writeLock();
+        sp->setProvider(this);
+        sp->writeUnlock();
+        _outputStrings.insert(e.attribute("name"), sp);
       }
     }
     n = n.nextSibling();
@@ -106,16 +117,39 @@ KstPlugin::KstPlugin(QDomElement &e) : KstDataObject(e) {
           _outputVectors.insert((*it)._name, v);
           KST::addVectorToList(v);
         }
+      } else if ((*it)._type == Plugin::Data::IOValue::MatrixType) {
+          abort(); // FIXME:
+#if 0
+        _outArrayCnt += 2;
+        if (!_outputMatrices.contains((*it)._name)) {
+          KstMatrixPtr m;
+
+          if ((*it)._subType == Plugin::Data::IOValue::FloatNonVectorSubType) {
+            m = new KstMatrix(tagName() + " matrix - " + (*it)._name, 0, true);
+          } else {
+            m = new KstMatrix(tagName() + " matrix - " + (*it)._name, 0, false);
+          }
+          m->setProvider(this);
+          _outputMatrices.insert((*it)._name, m);
+          KST::addVectorToList(v);
+        }
+#endif
       } else if ((*it)._type == Plugin::Data::IOValue::FloatType) {
         _outScalarCnt++;
         if (!_outputScalars.contains((*it)._name)) {
           KstScalarPtr s = new KstScalar(tagName() + " scalar - " + (*it)._name);
+          s->writeLock();
+          s->setProvider(this);
+          s->writeUnlock();
           _outputScalars.insert((*it)._name, s);
         }
       } else if ((*it)._type == Plugin::Data::IOValue::StringType) {
         _outStringCnt++;
         if (!_outputStrings.contains((*it)._name)) {
           KstStringPtr s = new KstString(tagName() + " string - " + (*it)._name);
+          s->writeLock();
+          s->setProvider(this);
+          s->writeUnlock();
           _outputStrings.insert((*it)._name, s);
         }
       }
@@ -140,9 +174,10 @@ void KstPlugin::commonConstructor() {
   _inArrayCnt = 0;
   _outArrayCnt = 0;
   _typeString = i18n("Plugin");
+  _type = "Plugin";
   _plugin = 0L;
   _localData = 0L;
-  //kdDebug() << "Creating KSTPlugin: " << long(this) << endl;
+  //kstdDebug() << "Creating KSTPlugin: " << long(this) << endl;
 }
 
 
@@ -154,7 +189,7 @@ KstPlugin::~KstPlugin() {
     }
     _localData = 0L;
   }
-  //kdDebug() << "Destroying KSTPlugin: " << long(this) << endl;
+  //kstdDebug() << "Destroying KSTPlugin: " << long(this) << endl;
 }
 
 
@@ -228,16 +263,14 @@ void KstPlugin::freeParameters() {
 
 KstObject::UpdateType KstPlugin::update(int update_counter) {
   if (!isValid()) {
-    return NO_CHANGE;
+    return setLastUpdateResult(NO_CHANGE);
   }
 
-  if (KstObject::checkUpdateCounter(update_counter)) {
-    return NO_CHANGE;
-  }
+  bool force = dirty();
+  setDirty(false);
 
-  if (update_counter != -1) {
-    //bool force = false;
-    // FIXME: we should only update if one of our dependencies was updated
+  if (KstObject::checkUpdateCounter(update_counter) && !force) {
+    return lastUpdateResult();
   }
 
 #define CLEANUP() do {\
@@ -258,9 +291,8 @@ KstObject::UpdateType KstPlugin::update(int update_counter) {
 
   const QValueList<Plugin::Data::IOValue>& itable = _plugin->data()._inputs;
   const QValueList<Plugin::Data::IOValue>& otable = _plugin->data()._outputs;
-  bool bNew = false; // what does this variable name mean?  That's why we don't
-                    // use HN in Kst!!
   int itcnt = 0, vitcnt = 0, sitcnt = 0;
+  bool doUpdate = force;
 
   // Populate the input scalars and vectors
   for (QValueList<Plugin::Data::IOValue>::ConstIterator it = itable.begin();
@@ -270,22 +302,31 @@ KstObject::UpdateType KstPlugin::update(int update_counter) {
       if (!_inputVectors.contains((*it)._name)) {
         KstDebug::self()->log(i18n("Input vector [%1] for plugin %2 not found.  Unable to continue.").arg((*it)._name).arg(tagName()), KstDebug::Error);
         CLEANUP();
-        return NO_CHANGE;
+        return setLastUpdateResult(NO_CHANGE);
       }
-      if (_inputVectors[(*it)._name]->sampleCount() == _inputVectors[(*it)._name]->numNew()) {
-        bNew = true;
-      }
-      _inVectors[vitcnt] = _inputVectors[(*it)._name]->value();
-      _inArrayLens[vitcnt++] = _inputVectors[(*it)._name]->sampleCount();
+      KstVectorPtr iv = _inputVectors[(*it)._name];
+      doUpdate = (UPDATE == iv->update(update_counter)) || doUpdate;
+      _inVectors[vitcnt] = iv->value();
+      _inArrayLens[vitcnt++] = iv->length();
     } else if ((*it)._type == Plugin::Data::IOValue::FloatType) {
-      _inScalars[itcnt++] = _inputScalars[(*it)._name]->value();
+      KstScalarPtr is = _inputScalars[(*it)._name];
+      doUpdate = (UPDATE == is->update(update_counter)) || doUpdate;
+      _inScalars[itcnt++] = is->value();
     } else if ((*it)._type == Plugin::Data::IOValue::StringType) {
-      _inStrings[sitcnt++] = strdup(_inputStrings[(*it)._name]->value().latin1());
+      KstStringPtr is = _inputStrings[(*it)._name];
+      doUpdate = (UPDATE == is->update(update_counter)) || doUpdate;
+      // Maybe we should use UTF-8 instead?
+      _inStrings[sitcnt++] = strdup(is->value().latin1());
     } else if ((*it)._type == Plugin::Data::IOValue::PidType) {
       _inScalars[itcnt++] = getpid();
     }
   }
 
+  if (!doUpdate) {
+    CLEANUP();
+    return setLastUpdateResult(NO_CHANGE);
+  }
+
   vitcnt = 0;
   // Populate the output vectors
   for (QValueList<Plugin::Data::IOValue>::ConstIterator it = otable.begin();
@@ -295,7 +336,7 @@ KstObject::UpdateType KstPlugin::update(int update_counter) {
       if (!_outputVectors.contains((*it)._name)) {
         KstDebug::self()->log(i18n("Output vector [%1] for plugin %2 not found.  Unable to continue.").arg((*it)._name).arg(tagName()), KstDebug::Error);
         CLEANUP();
-        return NO_CHANGE;
+        return setLastUpdateResult(NO_CHANGE);
       }
       _outVectors[vitcnt] = _outputVectors[(*it)._name]->value();
       _outArrayLens[vitcnt++] = _outputVectors[(*it)._name]->length();
@@ -331,38 +372,33 @@ KstObject::UpdateType KstPlugin::update(int update_counter) {
     itcnt = 0;
     vitcnt = 0;
     sitcnt = 0;
+    setLastUpdateResult(UPDATE); // make sure that provider callbacks work
     // Read back the output vectors and scalars
     for (QValueList<Plugin::Data::IOValue>::ConstIterator it = otable.begin();
         it != otable.end();
         ++it) {
       if ((*it)._type == Plugin::Data::IOValue::TableType) {
-        vectorRealloced(_outputVectors[(*it)._name], _outVectors[vitcnt], _outArrayLens[vitcnt], bNew);
+        KstVectorPtr vp = _outputVectors[(*it)._name];
+        vectorRealloced(vp, _outVectors[vitcnt], _outArrayLens[vitcnt]);
+        vp->setDirty();
+        // Inefficient, but do we have any other choice?  We don't really know
+        // from the plugin how much of this vector is "new" or "shifted"
+        vp->setNewAndShift(vp->length(), vp->numShift());
+        vp->update(update_counter);
         vitcnt++;
       } else if ((*it)._type == Plugin::Data::IOValue::FloatType) {
-        _outputScalars[(*it)._name]->setValue(_outScalars[itcnt++]);
+        KstScalarPtr sp = _outputScalars[(*it)._name];
+        sp->setValue(_outScalars[itcnt++]);
+        sp->update(update_counter);
       } else if ((*it)._type == Plugin::Data::IOValue::StringType) {
-        _outputStrings[(*it)._name]->setValue(_outStrings[sitcnt++]);
+        KstStringPtr sp = _outputStrings[(*it)._name];
+        sp->setValue(_outStrings[sitcnt++]);
+        sp->update(update_counter);
       }
     }
 
-    // if we have a fit plugin then create the necessary scalars from the parameter vector...
-    if (_plugin->data()._isFit) {
-      KstVectorPtr vectorParam = _outputVectors["Parameters"];
-      if (vectorParam) {
-        for (int i = 0; i < vectorParam->length(); i++) {
-          const QString& paramName = _plugin->parameterName(i);
-          if (!paramName.isEmpty()) {
-            QString scalarName = i18n("%1-%2").arg(tagName()).arg(paramName);
-            double scalarValue = vectorParam->value(i);
-            if (!_outputScalars.contains(paramName)) {
-              KstScalarPtr s = new KstScalar(scalarName);
-              _outputScalars.insert(paramName, s);
-            }
-            _outputScalars[paramName]->setValue(scalarValue);
-          }
-        }
-      }
-    }
+    // if we have a fit plugin then create the necessary scalars from the parameter vector
+    createFitScalars();
     _lastError = QString::null;
   } else if (rc > 0) {
     if (_lastError.isEmpty()) {
@@ -396,10 +432,10 @@ KstObject::UpdateType KstPlugin::update(int update_counter) {
       KstDebug::self()->log(i18n("Plugin %2 produced error: %1.").arg(_lastError).arg(tagName()), KstDebug::Error);
     }
   }
-  
+
   CLEANUP();
 #undef CLEANUP
-  return UPDATE;
+  return setLastUpdateResult(UPDATE);
 }
 
 
@@ -532,11 +568,13 @@ bool KstPlugin::setPlugin(KstSharedPtr<Plugin> plugin) {
     } else if ((*it)._type == Plugin::Data::IOValue::FloatType) {
       KstScalarPtr s = new KstScalar;
       s->writeLock();
+      s->setProvider(this);
       _outputScalars.insert((*it)._name, s);
       _outScalarCnt++;
     } else if ((*it)._type == Plugin::Data::IOValue::StringType) {
       KstStringPtr s = new KstString;
       s->writeLock();
+      s->setProvider(this);
       _outputStrings.insert((*it)._name, s);
       _outStringCnt++;
     }
@@ -562,9 +600,14 @@ const KstCurveHintList* KstPlugin::curveHints() const {
   _curveHints->clear();
   if (_plugin) {
     for (QValueList<Plugin::Data::CurveHint>::ConstIterator i = _plugin->data()._hints.begin(); i != _plugin->data()._hints.end(); ++i) {
-      _curveHints->append(new KstCurveHint((*i).name, _outputVectors[(*i).x]->tagName(), _outputVectors[(*i).y]->tagName()));
+      KstVectorPtr xv = _outputVectors[(*i).x];
+      KstVectorPtr yv = _outputVectors[(*i).y];
+      if (xv && yv) {
+        _curveHints->append(new KstCurveHint((*i).name, xv->tagName(), yv->tagName()));
+      }
     }
   }
+
   return KstDataObject::curveHints();
 }
 
@@ -573,4 +616,82 @@ QString KstPlugin::lastError() const {
   return _lastError;
 }
 
+
+// FIXME: KstPlugin should not know about fit scalars!!
+void KstPlugin::createFitScalars() {
+  if (_plugin->data()._isFit && _outputVectors.contains("Parameters")) {
+    KstVectorPtr vectorParam = _outputVectors["Parameters"];
+    if (vectorParam) {
+      QString paramName;
+      int i = 0;
+      int length = vectorParam->length();
+
+      for (paramName = _plugin->parameterName(i); 
+          !paramName.isEmpty() && i < length; 
+           paramName = _plugin->parameterName(++i)) {
+        double scalarValue = vectorParam->value(i);
+        if (!_outputScalars.contains(paramName)) {
+          QString scalarName = i18n("%1-%2").arg(tagName()).arg(paramName);
+          KstScalarPtr s = new KstScalar(scalarName, scalarValue);
+          s->writeLock();
+          s->setProvider(this);
+          s->writeUnlock();
+          _outputScalars.insert(paramName, s);
+        } else {
+          _outputScalars[paramName]->setValue(scalarValue);
+        }
+      }
+    }
+  }
+}
+
+
+KstDataObjectPtr KstPlugin::makeDuplicate(KstDataObjectDataObjectMap& duplicatedMap) {
+  KstPluginPtr plugin = new KstPlugin;
+
+  // use same inputs
+  for (KstVectorMap::ConstIterator iter = _inputVectors.begin(); iter != _inputVectors.end(); ++iter) {
+    plugin->inputVectors().insert(iter.key(), iter.data());
+  }
+  for (KstScalarMap::ConstIterator iter = _inputScalars.begin(); iter != _inputScalars.end(); ++iter) {
+    plugin->inputScalars().insert(iter.key(), iter.data());  
+  }
+  for (KstStringMap::ConstIterator iter = _inputStrings.begin(); iter != _inputStrings.end(); ++iter) {
+    plugin->inputStrings().insert(iter.key(), iter.data());  
+  }
+  
+  // create new outputs
+  for (KstVectorMap::ConstIterator iter = outputVectors().begin(); iter != outputVectors().end(); ++iter) {
+    KstVectorPtr v = new KstVector;
+    v->writeLock();
+    plugin->outputVectors().insert(iter.key(), v);
+    v->setTagName(iter.data()->tagName() + "'");
+    v->setProvider(plugin.data());
+    KST::addVectorToList(v);
+    v->writeUnlock();
+  }
+  for (KstScalarMap::ConstIterator iter = outputScalars().begin(); iter != outputScalars().end(); ++iter) {
+    KstScalarPtr s = new KstScalar;
+    s->writeLock();
+    plugin->outputScalars().insert(iter.key(), s);
+    s->setTagName(iter.data()->tagName() + "'");
+    s->setProvider(plugin.data());
+    s->writeUnlock();
+  }
+  for (KstStringMap::ConstIterator iter = outputStrings().begin(); iter != outputStrings().end(); ++iter) {
+    KstStringPtr s = new KstString;
+    s->writeLock();
+    plugin->outputStrings().insert(iter.key(), s);
+    s->setTagName(iter.data()->tagName() + "'");
+    s->setProvider(plugin.data());
+    s->writeUnlock();  
+  }
+  
+  // set the same plugin
+  plugin->setPlugin(_plugin);
+  plugin->setTagName(tagName() + "'");
+  duplicatedMap.insert(this, KstDataObjectPtr(plugin));  
+  return KstDataObjectPtr(plugin);
+}
+
 // vim: ts=2 sw=2 et
