@@ -19,122 +19,124 @@
 
 // include files for Qt
 #include <qcombobox.h>
+#include <qframe.h>
+#include <qgroupbox.h>
 #include <qlabel.h>
 #include <qlayout.h>
 #include <qlineedit.h>
+#include <qobjectlist.h>
+#include <qtextedit.h>
 #include <qtimer.h>
 #include <qtooltip.h>
+#include <qvbox.h>
 #include <qwhatsthis.h>
 
 // include files for KDE
 #include <kcolorbutton.h>
-#include <kdebug.h>
 #include <klocale.h>
 #include <kmessagebox.h>
 
 // application specific includes
+#include "kst.h"
 #include "kstplugindialog_i.h"
+#include "kstdefaultnames.h"
 #include "plugincollection.h"
+#include "plugindialogwidget.h"
 #include "pluginmanager.h"
 #include "scalarselector.h"
 #include "stringselector.h"
 #include "vectorselector.h"
 
-#define DIALOGTYPE KstPluginDialogI
-#define DTYPE "Plugin"
-#include "dataobjectdialog.h"
+const QString& KstPluginDialogI::plugin_defaultTag = KGlobal::staticQString("<Auto Name>");
 
-
-KstPluginDialogI *KstPluginDialogI::_inst = 0L;
-static KStaticDeleter<KstPluginDialogI> _plPluginInst;
+QGuardedPtr<KstPluginDialogI> KstPluginDialogI::_inst;
 
 KstPluginDialogI *KstPluginDialogI::globalInstance() {
   if (!_inst) {
-    _inst = _plPluginInst.setObject(new KstPluginDialogI);
+    _inst = new KstPluginDialogI(KstApp::inst());
   }
   return _inst;
 }
 
 
-static const QString& newPluginString = KGlobal::staticQString("<New_Plugin>");
+KstPluginDialogI::KstPluginDialogI(QWidget* parent, const char* name, bool modal, WFlags fl)
+: KstDataDialog(parent, name, modal, fl) {
+  _w = new PluginDialogWidget(_contents);
+  setMultiple(false);
+  connect(_w->PluginCombo, SIGNAL(activated(int)), this, SLOT(pluginChanged(int)));
+  connect(_w->_pluginManager, SIGNAL(clicked()), this, SLOT(showPluginManager()));
 
-KstPluginDialogI::KstPluginDialogI(QWidget* parent, const char* name,
-                                   bool modal, WFlags fl)
-: KstPluginDialog(parent, name, modal, fl) {
-  Init();
-  connect(PluginCombo, SIGNAL(activated(int)), this, SLOT(pluginChanged(int)));
-  connect(_pluginManager, SIGNAL(clicked()), this, SLOT(showPluginManager()));
-
-  QGridLayout *grid = new QGridLayout(_pluginFrame, 1, 1);
-  grid->addWidget(_frameWidget = new QWidget(_pluginFrame, "Frame Widget"), 0, 0);
+  _pluginInfoGrid = 0L;
+  _pluginInputOutputGrid = 0L;
 }
 
 
 KstPluginDialogI::~KstPluginDialogI() {
-  DP = 0L;
-}
-
-
-KstPluginPtr KstPluginDialogI::_getPtr(const QString &tagin) {
-  KstPluginList c = kstObjectSubList<KstDataObject, KstPlugin>(KST::dataObjectList);
-
-  return *c.findTag(tagin);
 }
 
 
 void KstPluginDialogI::updatePluginList() {
   PluginCollection *pc = PluginCollection::self();
-  QString previous = _pluginList[PluginCombo->currentItem()];
+  QString previous = _pluginList[_w->PluginCombo->currentItem()];
   int newFocus = -1;
 
-  const QMap<QString,Plugin::Data>& _pluginMap = pc->pluginList();
+  const QMap<QString,Plugin::Data>& pluginMap = pc->pluginList();
 
-  PluginCombo->clear();
+  QMap<QString,QString> oldIEntries, oldOEntries;
+
+  _w->PluginCombo->clear();
   _pluginList.clear();
   int cnt = 0;
-  for (QMap<QString,Plugin::Data>::ConstIterator it = _pluginMap.begin();
-                                                  it != _pluginMap.end();
-                                                                     ++it) {
+  Plugin::Data restoreEntry;
+  for (QMap<QString,Plugin::Data>::ConstIterator it = pluginMap.begin();
+                                                  it != pluginMap.end();
+                                                                   ++it) {
     _pluginList += it.data()._name;
-    PluginCombo->insertItem(i18n("%1 (v%2) - %3").arg(it.data()._readableName).arg(it.data()._version).arg(it.data()._description));
+    _w->PluginCombo->insertItem(i18n("%1 (v%2)").arg(it.data()._readableName).arg(it.data()._version));
     if (it.data()._name == previous) {
       newFocus = cnt;
+      oldIEntries = cacheInputs(it.data()._inputs);
+      oldOEntries = cacheInputs(it.data()._outputs);
+      restoreEntry = it.data();
     }
-    cnt++;
+    ++cnt;
   }
 
   if (newFocus != -1) {
-    PluginCombo->setCurrentItem(newFocus);
+    _w->PluginCombo->setCurrentItem(newFocus);
+    pluginChanged(_w->PluginCombo->currentItem());
+    restoreInputs(restoreEntry._inputs, oldIEntries);
+    restoreInputs(restoreEntry._outputs, oldOEntries);
   } else {
-    PluginCombo->setCurrentItem(0);
+    _w->PluginCombo->setCurrentItem(0);
     pluginChanged(0);
   }
 }
 
 
 void KstPluginDialogI::updateForm() {
-  KstSharedPtr<Plugin> plugin = PluginCollection::self()->plugin(_pluginList[PluginCombo->currentItem()]);
+  KstSharedPtr<Plugin> plugin = PluginCollection::self()->plugin(_pluginList[_w->PluginCombo->currentItem()]);
   if (plugin) {
     const QValueList<Plugin::Data::IOValue>& itable = plugin->data()._inputs;
     for (QValueList<Plugin::Data::IOValue>::ConstIterator it = itable.begin(); it != itable.end(); ++it) {
       if ((*it)._type == Plugin::Data::IOValue::TableType) { // vector
-        QObject *field = _frameWidget->child((*it)._name.latin1(), "VectorSelector");
+        QObject *field = _w->_pluginInputOutputFrame->child((*it)._name.latin1(), "VectorSelector");
         assert(field);
         if (field) {
           VectorSelector *vs = static_cast<VectorSelector*>(field);
           vs->update();
         }
       } else if ((*it)._type == Plugin::Data::IOValue::StringType) { // string
-        QObject *field = _frameWidget->child((*it)._name.latin1(), "StringSelector");
+        QObject *field = _w->_pluginInputOutputFrame->child((*it)._name.latin1(), "StringSelector");
         assert(field);
         if (field) {
           StringSelector *ss = static_cast<StringSelector*>(field);
           ss->update();
         }
-      } else if ((*it)._type == Plugin::Data::IOValue::PidType) { 
+      } else if ((*it)._type == Plugin::Data::IOValue::PidType) {
         // Nothing
       } else {
-        QObject *field = _frameWidget->child((*it)._name.latin1(), "ScalarSelector");
+        QObject *field = _w->_pluginInputOutputFrame->child((*it)._name.latin1(), "ScalarSelector");
         assert(field);
         if (field) {
           ScalarSelector *ss = static_cast<ScalarSelector*>(field);
@@ -146,103 +148,96 @@ void KstPluginDialogI::updateForm() {
 }
 
 
-void KstPluginDialogI::_fillFieldsForEdit() {
-  if (DP == 0L) {
-    return; // shouldn't be needed
+void KstPluginDialogI::fillFieldsForEdit() {
+  KstPluginPtr pp = kst_cast<KstPlugin>(_dp);
+  if (!pp) {
+    return;
   }
-  DP->readLock();
-  _tagName->setText(DP->tagName());
+  pp->readLock();
+  if (!pp->plugin()) { // plugin() can be null if the kst file is invalid
+    pp->readUnlock();
+    return;
+  }
+  _tagName->setText(pp->tagName());
 
   updatePluginList();
 
-  int i = _pluginList.findIndex(DP->plugin()->data()._name);
-  PluginCombo->setCurrentItem(i);
-  pluginChanged(PluginCombo->currentItem());
+  int i = _pluginList.findIndex(pp->plugin()->data()._name);
+  _w->PluginCombo->setCurrentItem(i);
+  pluginChanged(_w->PluginCombo->currentItem());
 
-  fillVectorScalarCombos(DP->plugin());
-  PluginCombo->setEnabled(DP->getUsage() < 2);
-  DP->readUnlock();
+  fillVectorScalarCombos(pp->plugin());
+  _w->PluginCombo->setEnabled(pp->getUsage() < 3);
+  pp->readUnlock();
 
   show();
   fixupLayout();
 }
 
 
-void KstPluginDialogI::_fillFieldsForNew() {
+void KstPluginDialogI::fillFieldsForNew() {
   updatePluginList();
-
-  KstPluginList plugins = kstObjectSubList<KstDataObject, KstPlugin>(KST::dataObjectList);
-
-  /* set tag name */
-  QString new_label;
-  new_label.sprintf("PL%d-", signed(plugins.count())+1);
-  new_label += newPluginString;
-  _tagName->setText(new_label);
-
-  updatePluginList();
-
-  PluginCombo->setCurrentItem(0);
-  pluginChanged(PluginCombo->currentItem());
-  PluginCombo->setEnabled(true);
-
-  KstSharedPtr<Plugin> plugin = PluginCollection::self()->plugin(_pluginList[PluginCombo->currentItem()]);
-
-  fillVectorScalarCombos(plugin);
+  _w->PluginCombo->setCurrentItem(0);
+  pluginChanged(_w->PluginCombo->currentItem());
+  _tagName->setText(plugin_defaultTag);
 }
 
 
 void KstPluginDialogI::fillVectorScalarCombos(KstSharedPtr<Plugin> plugin) {
   bool DPvalid = false;
+  KstPluginPtr pp = kst_cast<KstPlugin>(_dp);
 
-  if (DP && DP->isValid()) {
-    DPvalid = true;
+  if (pp) {
+    pp->readLock();
+    DPvalid = pp->isValid();
+    pp->readUnlock();
   }
 
   if (plugin) {
     if (DPvalid) {
-      DP->readLock();
+      pp->readLock();
     }
     // Update input vector and scalar combos
     const QValueList<Plugin::Data::IOValue>& itable = plugin->data()._inputs;
     for (QValueList<Plugin::Data::IOValue>::ConstIterator it = itable.begin();
          it != itable.end(); ++it) {
       if ((*it)._type == Plugin::Data::IOValue::TableType) { // vector
-        QObject *field = _frameWidget->child((*it)._name.latin1(), "VectorSelector");
+        QObject *field = _w->_pluginInputOutputFrame->child((*it)._name.latin1(), "VectorSelector");
         assert(field);
         if (field) {
           VectorSelector *vs = static_cast<VectorSelector*>(field);
           QString selectedVector = vs->selectedVector();
           vs->update();
           if (DPvalid) {
-            vs->setSelection(DP->inputVectors()[(*it)._name]->tagName());
+            vs->setSelection(pp->inputVectors()[(*it)._name]->tagName());
           } else {
             vs->setSelection(selectedVector);
           }
         }
       } else if ((*it)._type == Plugin::Data::IOValue::StringType) {
-        QObject *field = _frameWidget->child((*it)._name.latin1(), "StringSelector");
+        QObject *field = _w->_pluginInputOutputFrame->child((*it)._name.latin1(), "StringSelector");
         assert(field);
         if (field) {
           StringSelector *ss = static_cast<StringSelector*>(field);
           QString selectedString = ss->selectedString();
           ss->update();
           if (DPvalid) {
-            ss->setSelection(DP->inputStrings()[(*it)._name]->tagName());
+            ss->setSelection(pp->inputStrings()[(*it)._name]->tagName());
           } else {
             ss->setSelection(selectedString);
           }
         }
-      } else if ((*it)._type == Plugin::Data::IOValue::PidType) { 
+      } else if ((*it)._type == Plugin::Data::IOValue::PidType) {
         // Nothing
       } else {
-        QObject *field = _frameWidget->child((*it)._name.latin1(), "ScalarSelector");
+        QObject *field = _w->_pluginInputOutputFrame->child((*it)._name.latin1(), "ScalarSelector");
         assert(field);
         if (field) {
           ScalarSelector *ss = static_cast<ScalarSelector*>(field);
           QString selectedScalar = ss->selectedScalar();
           ss->update();
           if (DPvalid) {
-            ss->setSelection(DP->inputScalars()[(*it)._name]->tagName());
+            ss->setSelection(pp->inputScalars()[(*it)._name]->tagName());
           } else {
             ss->setSelection(selectedScalar);
           }
@@ -254,30 +249,30 @@ void KstPluginDialogI::fillVectorScalarCombos(KstSharedPtr<Plugin> plugin) {
     if (DPvalid) {
       const QValueList<Plugin::Data::IOValue>& otable = plugin->data()._outputs;
       for (QValueList<Plugin::Data::IOValue>::ConstIterator it = otable.begin(); it != otable.end(); ++it) {
-        QObject *field = _frameWidget->child((*it)._name.latin1(), "QLineEdit");
+        QObject *field = _w->_pluginInputOutputFrame->child((*it)._name.latin1(), "QLineEdit");
         assert(field);
         if (field) {
           QLineEdit *li = static_cast<QLineEdit*>(field);
           QString ts;
           if ((*it)._type == Plugin::Data::IOValue::TableType) { // vector
-            ts = DP->outputVectors()[(*it)._name]->tagName();
+            ts = pp->outputVectors()[(*it)._name]->tagName();
           } else if ((*it)._type == Plugin::Data::IOValue::PidType) {
           } else if ((*it)._type == Plugin::Data::IOValue::StringType) {
-            ts = DP->outputStrings()[(*it)._name]->tagName();
+            ts = pp->outputStrings()[(*it)._name]->tagName();
           } else { // scalar
-            ts = DP->outputScalars()[(*it)._name]->tagName();
+            ts = pp->outputScalars()[(*it)._name]->tagName();
           }
           li->setText(ts);
         }
       }
-      DP->readUnlock();
+      pp->readUnlock();
     }
   } else { // invalid plugin
     PluginCollection *pc = PluginCollection::self();
-    QString cur = _pluginList[PluginCombo->currentItem()];
+    QString cur = _pluginList[_w->PluginCombo->currentItem()];
     Plugin::Data pdata = pc->pluginList()[pc->pluginNameList()[cur]];
     for (QValueList<Plugin::Data::IOValue>::ConstIterator it = pdata._outputs.begin(); it != pdata._outputs.end(); ++it) {
-      QObject *field = _frameWidget->child((*it)._name.latin1(), "QLineEdit");
+      QObject *field = _w->_pluginInputOutputFrame->child((*it)._name.latin1(), "QLineEdit");
       if (field) {
         static_cast<QLineEdit*>(field)->setText(QString::null);
       }
@@ -292,34 +287,121 @@ void KstPluginDialogI::update() {
 
 
 void KstPluginDialogI::fixupLayout() {
-  _frameWidget->resize(_frameWidget->sizeHint());
-  _pluginFrame->resize(_pluginFrame->sizeHint());
-  resize(sizeHint());
-  setMinimumSize(size());
-  updateGeometry();
+  adjustSize();
+  resize(650, minimumSizeHint().height());
+  setFixedHeight(height());
+}
+
+
+QMap<QString,QString> KstPluginDialogI::cacheInputs(const QValueList<Plugin::Data::IOValue>& table) {
+  QMap<QString,QString> rc;
+  for (QValueList<Plugin::Data::IOValue>::ConstIterator it = table.begin(); it != table.end(); ++it) {
+    if ((*it)._type == Plugin::Data::IOValue::TableType) {
+      QObject *field = _w->_pluginInputOutputFrame->child((*it)._name.latin1(), "VectorSelector");
+      if (field) {
+        rc[(*it)._name] = static_cast<VectorSelector*>(field)->selectedVector();
+      } else {
+        field = _w->_pluginInputOutputFrame->child((*it)._name.latin1(), "QLineEdit");
+        if (field) {
+          rc[(*it)._name] = static_cast<QLineEdit*>(field)->text();
+        }
+      }
+
+    } else if ((*it)._type == Plugin::Data::IOValue::StringType) {
+      QObject *field = _w->_pluginInputOutputFrame->child((*it)._name.latin1(), "StringSelector");
+      if (field) {
+        rc[(*it)._name] = static_cast<StringSelector*>(field)->selectedString();
+      } else {
+        field = _w->_pluginInputOutputFrame->child((*it)._name.latin1(), "QLineEdit");
+        if (field) {
+          rc[(*it)._name] = static_cast<QLineEdit*>(field)->text();
+        }
+      }
+    } else if ((*it)._type == Plugin::Data::IOValue::PidType) {
+      // Nothing
+    } else if ((*it)._type == Plugin::Data::IOValue::FloatType) {
+      QObject *field = _w->_pluginInputOutputFrame->child((*it)._name.latin1(), "ScalarSelector");
+      if (field) {
+        rc[(*it)._name] = static_cast<ScalarSelector*>(field)->selectedScalar();
+      } else {
+        field = _w->_pluginInputOutputFrame->child((*it)._name.latin1(), "QLineEdit");
+        if (field) {
+          rc[(*it)._name] = static_cast<QLineEdit*>(field)->text();
+        }
+      }
+    }
+  }
+  return rc;
+}
+
+
+void KstPluginDialogI::restoreInputs(const QValueList<Plugin::Data::IOValue>& table, const QMap<QString,QString>& v) {
+  for (QValueList<Plugin::Data::IOValue>::ConstIterator it = table.begin(); it != table.end(); ++it) {
+    if (!v.contains((*it)._name)) {
+      continue;
+    }
+    if ((*it)._type == Plugin::Data::IOValue::TableType) {
+      QObject *field = _w->_pluginInputOutputFrame->child((*it)._name.latin1(), "VectorSelector");
+      if (field) {
+        static_cast<VectorSelector*>(field)->setSelection(v[(*it)._name]);
+      } else {
+        field = _w->_pluginInputOutputFrame->child((*it)._name.latin1(), "QLineEdit");
+        if (field) {
+          static_cast<QLineEdit*>(field)->setText(v[(*it)._name]);
+        }
+      }
+    } else if ((*it)._type == Plugin::Data::IOValue::StringType) {
+      QObject *field = _w->_pluginInputOutputFrame->child((*it)._name.latin1(), "StringSelector");
+      if (field) {
+        static_cast<StringSelector*>(field)->setSelection(v[(*it)._name]);
+      } else {
+        field = _w->_pluginInputOutputFrame->child((*it)._name.latin1(), "QLineEdit");
+        if (field) {
+          static_cast<QLineEdit*>(field)->setText(v[(*it)._name]);
+        }
+      }
+    } else if ((*it)._type == Plugin::Data::IOValue::PidType) {
+      // Nothing
+    } else if ((*it)._type == Plugin::Data::IOValue::FloatType) {
+      QObject *field = _w->_pluginInputOutputFrame->child((*it)._name.latin1(), "ScalarSelector");
+      if (field) {
+        static_cast<ScalarSelector*>(field)->setSelection(v[(*it)._name]);
+      } else {
+        field = _w->_pluginInputOutputFrame->child((*it)._name.latin1(), "QLineEdit");
+        if (field) {
+          static_cast<QLineEdit*>(field)->setText(v[(*it)._name]);
+        }
+      }
+    }
+  }
 }
 
 
 bool KstPluginDialogI::saveInputs(KstPluginPtr plugin, KstSharedPtr<Plugin> p) {
+  bool rc = true;
+
   KST::vectorList.lock().readLock();
   KST::scalarList.lock().readLock();
   KST::stringList.lock().readLock();
   const QValueList<Plugin::Data::IOValue>& itable = p->data()._inputs;
   for (QValueList<Plugin::Data::IOValue>::ConstIterator it = itable.begin(); it != itable.end(); ++it) {
     if ((*it)._type == Plugin::Data::IOValue::TableType) {
-      QObject *field = _frameWidget->child((*it)._name.latin1(), "VectorSelector");
+      QObject *field = _w->_pluginInputOutputFrame->child((*it)._name.latin1(), "VectorSelector");
       assert(field);
       VectorSelector *vs = static_cast<VectorSelector*>(field);
       KstVectorPtr v = *KST::vectorList.findTag(vs->selectedVector());
       if (v) {
         v->writeLock(); // to match with plugin->writeLock()
+        if (plugin->inputVectors().contains((*it)._name) && plugin->inputVectors()[(*it)._name] != v) {
+          plugin->inputVectors()[(*it)._name]->writeUnlock();
+        }
+        plugin->inputVectors().insert((*it)._name, v);
+      } else if (plugin->inputVectors().contains((*it)._name)) {
+        plugin->inputVectors().erase((*it)._name);
+        rc = false;
       }
-      if (plugin->inputVectors().contains((*it)._name) && plugin->inputVectors()[(*it)._name] != v) {
-        plugin->inputVectors()[(*it)._name]->writeUnlock();
-      }
-      plugin->inputVectors().insert((*it)._name, v);
     } else if ((*it)._type == Plugin::Data::IOValue::StringType) {
-      QObject *field = _frameWidget->child((*it)._name.latin1(), "StringSelector");
+      QObject *field = _w->_pluginInputOutputFrame->child((*it)._name.latin1(), "StringSelector");
       assert(field);
       StringSelector *ss = static_cast<StringSelector*>(field);
       KstStringPtr s = *KST::stringList.findTag(ss->selectedString());
@@ -341,7 +423,7 @@ bool KstPluginDialogI::saveInputs(KstPluginPtr plugin, KstSharedPtr<Plugin> p) {
     } else if ((*it)._type == Plugin::Data::IOValue::PidType) {
       // Nothing
     } else if ((*it)._type == Plugin::Data::IOValue::FloatType) {
-      QObject *field = _frameWidget->child((*it)._name.latin1(), "ScalarSelector");
+      QObject *field = _w->_pluginInputOutputFrame->child((*it)._name.latin1(), "ScalarSelector");
       assert(field);
       ScalarSelector *ss = static_cast<ScalarSelector*>(field);
       KstScalarPtr s = *KST::scalarList.findTag(ss->selectedScalar());
@@ -376,7 +458,8 @@ bool KstPluginDialogI::saveInputs(KstPluginPtr plugin, KstSharedPtr<Plugin> p) {
   KST::stringList.lock().readUnlock();
   KST::scalarList.lock().readUnlock();
   KST::vectorList.lock().readUnlock();
-  return true;
+
+  return rc;
 }
 
 
@@ -384,7 +467,7 @@ bool KstPluginDialogI::saveOutputs(KstPluginPtr plugin, KstSharedPtr<Plugin> p)
   const QValueList<Plugin::Data::IOValue>& otable = p->data()._outputs;
 
   for (QValueList<Plugin::Data::IOValue>::ConstIterator it = otable.begin(); it != otable.end(); ++it) {
-    QObject *field = _frameWidget->child((*it)._name.latin1(), "QLineEdit");
+    QObject *field = _w->_pluginInputOutputFrame->child((*it)._name.latin1(), "QLineEdit");
     if (!field) {
       continue; // Some are unsupported
     }
@@ -393,7 +476,7 @@ bool KstPluginDialogI::saveOutputs(KstPluginPtr plugin, KstSharedPtr<Plugin> p)
 
     if (li->text().isEmpty()) {
       QString tagName = _tagName->text();
-      if (tagName.endsWith(newPluginString)) {
+      if (tagName==plugin_defaultTag) {
         tagName = plugin->tagName();
       }
       li->setText(tagName + "-" + (*it)._name);
@@ -439,6 +522,7 @@ bool KstPluginDialogI::saveOutputs(KstPluginPtr plugin, KstSharedPtr<Plugin> p)
           plugin->outputStrings().insert((*it)._name, s);
         }
         s->setTagName(nt);
+        s->setProvider(plugin.data());
       } else if (plugin->outputStrings()[(*it)._name]->tagName() != nt) {
         while (KST::vectorTagNameNotUnique(nt, false)) {
           nt += "'";
@@ -452,6 +536,7 @@ bool KstPluginDialogI::saveOutputs(KstPluginPtr plugin, KstSharedPtr<Plugin> p)
           plugin->outputStrings().insert((*it)._name, s);
         }
         s->setTagName(nt);
+        s->setProvider(plugin.data());
       }
     } else if ((*it)._type == Plugin::Data::IOValue::PidType) {
       // Nothing
@@ -466,6 +551,7 @@ bool KstPluginDialogI::saveOutputs(KstPluginPtr plugin, KstSharedPtr<Plugin> p)
           plugin->outputScalars().insert((*it)._name, s);
         }
         s->setTagName(nt);
+        s->setProvider(plugin.data());
       } else if (plugin->outputScalars()[(*it)._name]->tagName() != nt) {
         while (KST::vectorTagNameNotUnique(nt, false)) {
           nt += "'";
@@ -479,6 +565,7 @@ bool KstPluginDialogI::saveOutputs(KstPluginPtr plugin, KstSharedPtr<Plugin> p)
           plugin->outputScalars().insert((*it)._name, s);
         }
         s->setTagName(nt);
+        s->setProvider(plugin.data());
       }
     }
   }
@@ -487,22 +574,22 @@ bool KstPluginDialogI::saveOutputs(KstPluginPtr plugin, KstSharedPtr<Plugin> p)
 }
 
 
-bool KstPluginDialogI::new_I() {
-  KstPluginPtr plugin;
+bool KstPluginDialogI::newObject() {
   QString tagName = _tagName->text();
 
-  if (!tagName.endsWith(newPluginString) && KST::dataTagNameNotUnique(tagName, true, this)) {
+  if (tagName != plugin_defaultTag && KST::dataTagNameNotUnique(tagName, true, this)) {
     _tagName->setFocus();
     return false;
   }
-
-  int pitem = PluginCombo->currentItem();
-  if (pitem >= 0 && PluginCombo->count() > 0) {
+  KstPluginPtr plugin;
+  int pitem = _w->PluginCombo->currentItem();
+  if (pitem >= 0 && _w->PluginCombo->count() > 0) {
     KstSharedPtr<Plugin> pPtr = PluginCollection::self()->plugin(_pluginList[pitem]);
     if (pPtr) {
       plugin = new KstPlugin;
       plugin->writeLock();
       if (!saveInputs(plugin, pPtr)) {
+        KMessageBox::sorry(this, i18n("One or more of the inputs was undefined."));
         plugin->writeUnlock();
         plugin = 0L;
         return false;
@@ -510,16 +597,12 @@ bool KstPluginDialogI::new_I() {
 
       plugin->setPlugin(pPtr);
 
-      if (tagName.endsWith(newPluginString)) {
-        QString tmpTagName;
-        int i = 0;
-        do {
-          tmpTagName = i18n("P%1-%2").arg(++i).arg(_pluginList[pitem]);
-        } while (KST::dataTagNameNotUnique(tmpTagName, false));
-        tagName = tmpTagName;
+      if (tagName == plugin_defaultTag) {
+        tagName = KST::suggestPluginName(_pluginList[pitem]);
       }
       plugin->setTagName(tagName);
       if (!saveOutputs(plugin, pPtr)) {
+        KMessageBox::sorry(this, i18n("One or more of the outputs was undefined."));
         plugin->writeUnlock();
         plugin = 0L;
         return false;
@@ -529,11 +612,11 @@ bool KstPluginDialogI::new_I() {
   }
 
   if (!plugin || !plugin->isValid()) {
-    KMessageBox::sorry(this, i18n("There is an error in the plugin you entered.  Please fix the plugin."));
+    KMessageBox::sorry(this, i18n("There is an error in the plugin you entered."));
     return false;
   }
 
-  plugin->update(-1);
+  plugin->setDirty();
   KST::dataObjectList.lock().writeLock();
   KST::dataObjectList.append(plugin.data());
   KST::dataObjectList.lock().writeUnlock();
@@ -544,68 +627,68 @@ bool KstPluginDialogI::new_I() {
 }
 
 
-bool KstPluginDialogI::edit_I() {
-  if (!DP) { // something is dreadfully wrong - this should never happen
+bool KstPluginDialogI::editObject() {
+  KstPluginPtr pp = kst_cast<KstPlugin>(_dp);
+  if (!pp) { // something is dreadfully wrong - this should never happen
     return false;
   }
 
-  DP->writeLock();
-  if (_tagName->text() != DP->tagName() && KST::dataTagNameNotUnique(_tagName->text())) {
+  pp->writeLock();
+  if (_tagName->text() != pp->tagName() && KST::dataTagNameNotUnique(_tagName->text())) {
     _tagName->setFocus();
-    DP->writeUnlock();
+    pp->writeUnlock();
     return false;
   }
 
-  DP->setTagName(_tagName->text());
+  pp->setTagName(_tagName->text());
 
-  int pitem = PluginCombo->currentItem();
+  int pitem = _w->PluginCombo->currentItem();
   KstSharedPtr<Plugin> pPtr = PluginCollection::self()->plugin(_pluginList[pitem]);
 
   // Must unlock before clear()
-  for (KstVectorMap::Iterator i = DP->inputVectors().begin(); i != DP->inputVectors().end(); ++i) {
+  for (KstVectorMap::Iterator i = pp->inputVectors().begin(); i != pp->inputVectors().end(); ++i) {
     (*i)->writeUnlock();
   }
-  for (KstScalarMap::Iterator i = DP->inputScalars().begin(); i != DP->inputScalars().end(); ++i) {
+  for (KstScalarMap::Iterator i = pp->inputScalars().begin(); i != pp->inputScalars().end(); ++i) {
     (*i)->writeUnlock();
   }
-  for (KstStringMap::Iterator i = DP->inputStrings().begin(); i != DP->inputStrings().end(); ++i) {
+  for (KstStringMap::Iterator i = pp->inputStrings().begin(); i != pp->inputStrings().end(); ++i) {
     (*i)->writeUnlock();
   }
-  DP->inputVectors().clear();
-  DP->inputScalars().clear();
-  DP->inputStrings().clear();
+  pp->inputVectors().clear();
+  pp->inputScalars().clear();
+  pp->inputStrings().clear();
 
   // Save the vectors and scalars
-  if (!saveInputs(DP, pPtr)) {
-    KMessageBox::sorry(this, i18n("There is an error in the plugin you "
-                                  "entered. Please fix it."));
-    DP->writeUnlock();
+  if (!saveInputs(pp, pPtr)) {
+    KMessageBox::sorry(this, i18n("There is an error in the plugin you entered."));
+    pp->writeUnlock();
     return false;
   }
 
-  if (pitem >= 0 && PluginCombo->count() > 0) {
-    DP->setPlugin(pPtr);
+  if (pitem >= 0 && _w->PluginCombo->count() > 0) {
+    pp->setPlugin(pPtr);
   }
 
-  if (!saveOutputs(DP, pPtr)) {
-    KMessageBox::sorry(this, i18n("There is an error in the plugin you "
-                                  "entered. Please fix it."));
-    DP->writeUnlock();
+  if (!saveOutputs(pp, pPtr)) {
+    KMessageBox::sorry(this, i18n("There is an error in the plugin you entered."));
+    pp->writeUnlock();
     return false;
   }
 
-  if (!DP->isValid()) {
-    KMessageBox::sorry(this, i18n("There is an error in the plugin you entered. Please fix it."));
-    DP->writeUnlock();
+  if (!pp->isValid()) {
+    KMessageBox::sorry(this, i18n("There is an error in the plugin you entered."));
+    pp->writeUnlock();
     return false;
   }
-  DP->update(-1);
-  DP->writeUnlock();
+  pp->setDirty();
+  pp->writeUnlock();
 
   emit modified();
   return true;
 }
 
+
 void KstPluginDialogI::generateEntries(bool input, int& cnt, QWidget *parent, QGridLayout *grid, const QValueList<Plugin::Data::IOValue>& table) {
   QString scalarLabelTemplate, vectorLabelTemplate, stringLabelTemplate;
 
@@ -666,6 +749,7 @@ void KstPluginDialogI::generateEntries(bool input, int& cnt, QWidget *parent, QG
         w->allowDirectEntry(true);
         if (p) {
           p->readLock();
+          QToolTip::remove(w->_scalar);
           QToolTip::add(w->_scalar, QString::number(p->value()));
           p->readUnlock();
         }
@@ -682,6 +766,7 @@ void KstPluginDialogI::generateEntries(bool input, int& cnt, QWidget *parent, QG
         w->allowDirectEntry(true);
         if (p) {
           p->readLock();
+          QToolTip::remove(w->_string);
           QToolTip::add(w->_string, p->value());
           p->readUnlock();
         }
@@ -694,79 +779,99 @@ void KstPluginDialogI::generateEntries(bool input, int& cnt, QWidget *parent, QG
     }
 
     grid->addWidget(label, cnt, 0);
+    _pluginWidgets.push_back(label);
+    label->show();
+
     grid->addWidget(widget, cnt, 1);
+    _pluginWidgets.push_back(widget);
+    widget->show();
 
     if (!(*it)._description.isEmpty()) {
+      QWhatsThis::remove(label);
+      QWhatsThis::remove(widget);
       QWhatsThis::add(label, (*it)._description);
       QWhatsThis::add(widget, (*it)._description);
     }
 
-    cnt++;
+    ++cnt;
   }
 }
 
 
 void KstPluginDialogI::pluginChanged(int idx) {
-  // Get rid of the old widget
-  delete _frameWidget;
-  _frameWidget = 0L;
 
-  // Create a new one
-  QGridLayout *topGrid = dynamic_cast<QGridLayout*>(_pluginFrame->layout());
-  if (topGrid) {
-    topGrid->invalidate();
-    topGrid->addWidget(_frameWidget = new QWidget(_pluginFrame, "Frame Widget"), 0, 0);
-  } else {
-    kdError() << "Somehow we lost the grid!" << endl;
-    return;
+  // find all children and delete them and the grids
+  while (!_pluginWidgets.isEmpty())
+  {
+    QWidget* tempWidget = _pluginWidgets.back();
+    _pluginWidgets.pop_back();
+    delete tempWidget;
   }
+  delete _pluginInfoGrid;
+  delete _pluginInputOutputGrid;
 
-  // Refill it
-  if (idx >= 0 && PluginCombo->count() > 0) {
+  // create new info grid
+  _pluginInfoGrid = new QGridLayout(_w->_pluginInfoFrame, 2, 2, 0, 8);
+  _pluginInfoGrid->setColStretch(1,1); // stretch the right column
+  _pluginInfoGrid->setColStretch(0,0); // don't stretch the left column
+
+  if (idx >= 0 && _w->PluginCombo->count() > 0) {
+
+    // get the plugin data
     const QString& pluginName = _pluginList[idx];
     const Plugin::Data& pluginData = PluginCollection::self()->pluginList()[PluginCollection::self()->pluginNameList()[pluginName]];
-    int cnt = 3;
 
     // Setup the grid and the "static" entries
-    int variables = pluginData._inputs.count() + pluginData._outputs.count();
-    QGridLayout *grid = new QGridLayout(_frameWidget, 4+variables, 2);
-    grid->setMargin(6);
-    grid->setSpacing(5);
-    grid->addWidget(new QLabel(i18n("Plugin name:"), _frameWidget), 0, 0);
-    grid->addWidget(new QLabel(pluginData._readableName, _frameWidget), 0, 1);
-    grid->addWidget(new QLabel(i18n("Description:"), _frameWidget), 1, 0);
-    QLabel *lab = new QLabel(pluginData._description, _frameWidget);
-    lab->setAlignment(lab->alignment() | Qt::WordBreak);
-    grid->addWidget(lab, 1, 1);
+    QLabel* infoLabel;
 
-    // Add a separator
-    QFrame* line = new QFrame(_frameWidget);
+    infoLabel = new QLabel(i18n("Plugin name:"), _w->_pluginInfoFrame);
+    _pluginInfoGrid->addWidget(infoLabel, 0, 0);
+    _pluginWidgets.append(infoLabel);
+    infoLabel->show();
+
+    infoLabel = new QLabel(pluginData._readableName, _w->_pluginInfoFrame);
+    _pluginInfoGrid->addWidget(infoLabel, 0, 1);
+    _pluginWidgets.append(infoLabel);
+    infoLabel->show();
+
+    infoLabel = new QLabel(i18n("Description:"), _w->_pluginInfoFrame);
+    infoLabel->setAlignment(Qt::AlignTop);
+    _pluginInfoGrid->addWidget(infoLabel, 1, 0);
+    _pluginWidgets.append(infoLabel);
+    infoLabel->show();
+
+    infoLabel = new QLabel(pluginData._description,  _w->_pluginInfoFrame);
+    _pluginInfoGrid->addWidget(infoLabel, 1, 1);
+    _pluginWidgets.append(infoLabel);
+    infoLabel->show();
+
+    // create a new inputoutput grid
+    int cnt = 0;
+    int numInputOutputs = pluginData._inputs.count() + pluginData._outputs.count();
+
+    // generate inputs
+    _pluginInputOutputGrid = new QGridLayout(_w->_pluginInputOutputFrame, numInputOutputs + 1, 2, 0, 8);
+    _pluginInputOutputGrid->setColStretch(1,1);
+    _pluginInputOutputGrid->setColStretch(0,0);
+    generateEntries(true, cnt, _w->_pluginInputOutputFrame, _pluginInputOutputGrid, pluginData._inputs);
+
+    // insert separator
+    cnt++;
+    QFrame* line = new QFrame(_w->_pluginInputOutputFrame);
     line->setFrameShadow(QFrame::Sunken);
     line->setFrameShape(QFrame::HLine);
-    grid->addMultiCellWidget(line, 2, 2, 0, 1);
+    _pluginInputOutputGrid->addMultiCellWidget(line, cnt, cnt, 0, 1);
+    _pluginWidgets.push_back(line);
+    line->show();
+    cnt++;
 
-    // Generate the input values
-    generateEntries(true, cnt, _frameWidget, grid, pluginData._inputs);
-
-    if (!pluginData._inputs.isEmpty() && !pluginData._outputs.isEmpty()) {
-      // Add a separator
-      line = new QFrame(_frameWidget);
-      line->setFrameShape(QFrame::HLine);
-      line->setFrameShadow(QFrame::Sunken);
-      line->setFrameShape(QFrame::HLine);
-      grid->addMultiCellWidget(line, cnt, cnt, 0, 1);
-      cnt++;
-    }
-
-    // Generate the output values
-    generateEntries(false, cnt, _frameWidget, grid, pluginData._outputs);
+    // generate outputs
+    _pluginInputOutputGrid->setColStretch(1,1);
+    _pluginInputOutputGrid->setColStretch(0,0);
+    generateEntries(false, cnt, _w->_pluginInputOutputFrame, _pluginInputOutputGrid, pluginData._outputs);
   }
-
-  // show it
-  _frameWidget->show();
-
   // resize everything
-  QTimer::singleShot(0, this, SLOT(fixupLayout()));
+  fixupLayout();
 }
 
 
