@@ -20,25 +20,38 @@
 #include <stdlib.h>
 
 // include files for Qt
+#include <qbitmap.h>
 #include <qdeepcopy.h>
+#include <qmetaobject.h>
 #include <qstylesheet.h>
 
 // include files for KDE
 #include <kdatastream.h>
-#include <kdebug.h>
+#include "ksdebug.h"
 
 // application specific includes
 #include "kst.h"
+#include "kst2dplot.h" // Yuck, fix this
+#include "kstaccessibility.h"
 #include "kstdoc.h"
+#include "ksteditviewobjectdialog_i.h"
 #include "kstobject.h"
 #include "kstplotgroup.h"
 #include "kstsettings.h"
 #include "ksttimers.h"
+#include "ksttoplevelview.h"
+#include "kstviewarrow.h"
+#include "kstviewbox.h"
+#include "kstviewellipse.h"
+#include "kstviewlabel.h"
+#include "kstviewlegend.h"
 #include "kstviewobject.h"
 #include "kstviewobjectfactory.h"
+#include "kstviewpicture.h"
 #include "kstviewwindow.h"
 
-KstViewObject::KstViewObject(const QString& type) : KstObject(), _geom(0, 0, 1, 1), _type(type) {
+KstViewObject::KstViewObject(const QString& type)
+: KstObject(), _geom(0, 0, 1, 1), _type(type) {
   _parent = 0L;
   _standardActions = 0;
   _layoutActions = 0;
@@ -50,18 +63,51 @@ KstViewObject::KstViewObject(const QString& type) : KstObject(), _geom(0, 0, 1,
   _selected = false;
   _foregroundColor = KstSettings::globalSettings()->foregroundColor;
   _backgroundColor = KstSettings::globalSettings()->backgroundColor;
+  _transparent = false;
+  _followsFlow = false;
+  setMinimumSize(QSize(3, 3)); // should be the default
 }
 
 
-KstViewObject::KstViewObject(QDomElement& e) : KstObject() {
+KstViewObject::KstViewObject(const QDomElement& e)
+: KstObject() {
   _foregroundColor = KstSettings::globalSettings()->foregroundColor;
   _backgroundColor = KstSettings::globalSettings()->backgroundColor;
   _parent = 0L;
+  _transparent = false;
+  _followsFlow = false;
+  setMinimumSize(QSize(1, 1));
   load(e);
 }
 
 
-void KstViewObject::load(QDomElement& e) {
+KstViewObject::KstViewObject(const KstViewObject& viewObject)
+: KstObject() {
+  _parent = 0L;
+  _foregroundColor = viewObject._foregroundColor;
+  _backgroundColor = viewObject._backgroundColor;
+  _aspect = viewObject._aspect;
+  _standardActions = viewObject._standardActions;
+  _layoutActions = viewObject._layoutActions;
+  _maximized = false;
+  _onGrid = viewObject._onGrid;
+  _columns = viewObject._columns;
+  _focus = false;
+  _selected = false;
+  _geom = viewObject._geom;
+  _transparent = viewObject._transparent;
+  _followsFlow = viewObject._followsFlow;
+  setMinimumSize(QSize(1, 1));
+
+  setContentsRect(viewObject.contentsRect());
+
+  for (KstViewObjectList::ConstIterator i = viewObject._children.begin(); i != viewObject._children.end(); ++i) {
+    (*i)->copyObjectQuietly(*this);
+  }
+}
+
+
+void KstViewObject::load(const QDomElement& e) {
   _children.clear();
   _standardActions = 0;
   _layoutActions = 0;
@@ -79,6 +125,8 @@ void KstViewObject::load(QDomElement& e) {
     if (!el.isNull()) { // the node was really an element.
       if (el.tagName() == "tag") {
         setTagName(el.text());
+      } else if (el.tagName() == "transparent") {
+        _transparent = true;
       } else if (el.tagName() == "columns") {
         _onGrid = true;
         _columns = el.text().toInt();
@@ -87,7 +135,7 @@ void KstViewObject::load(QDomElement& e) {
         _aspect.y = el.attribute("y", "0.0").toDouble();
         _aspect.w = el.attribute("w", "1.0").toDouble();
         _aspect.h = el.attribute("h", "1.0").toDouble();
-      }
+      } 
     }
     n = n.nextSibling();
   }
@@ -99,26 +147,63 @@ KstViewObject::~KstViewObject() {
 }
 
 
-void KstViewObject::loadChildren(QDomElement& e) {
+void KstViewObject::loadChildren(const QDomElement& e) {
   QDomNode n = e.firstChild();
 
   while (!n.isNull()) {
     QDomElement el = n.toElement();
     if (!el.isNull()) {
       if (el.tagName() == "plot") {
-        QString in_tag = el.text();
-        if (KstApp::inst()->plotHolderWhileOpeningDocument().count(in_tag) > 0) {
-          Kst2DPlotPtr plot = KstApp::inst()->plotHolderWhileOpeningDocument()[in_tag];
+        // get the <tag> value properly
+        QString in_tag;
+        QDomNode plotNode = el.firstChild();
+        while (!plotNode.isNull()) {
+          QDomElement plotElem = plotNode.toElement(); 
+          if (!plotElem.isNull()) {
+            if (plotElem.tagName() == "tag") {
+              in_tag = plotElem.text();
+            }  
+          }
+          plotNode = plotNode.nextSibling();      
+        }
+        // FIXME: View Object should not know about this!!!!
+        Kst2DPlotMap *pmap = KstApp::inst()->plotHolderWhileOpeningDocument();
+        if (pmap->count(in_tag) > 0) {
+          Kst2DPlotPtr plot = (*pmap)[in_tag];
           if (plot) {
             appendChild(plot.data(), true);
             plot->loadChildren(el);
-            KstApp::inst()->plotHolderWhileOpeningDocument().erase(in_tag);
+            pmap->erase(in_tag);
           }
         }
       } else if (el.tagName() == "plotgroup") {
         KstPlotGroupPtr plotGroup = new KstPlotGroup(el);
         appendChild(plotGroup.data(), true);
         plotGroup->loadChildren(el);
+      } else if (el.tagName() == "Box") {
+        KstViewBoxPtr box = new KstViewBox(el);
+        appendChild(box.data(), true);
+        box->loadChildren(el);
+      } else if (el.tagName() == "Arrow") {
+        KstViewArrowPtr arrow = new KstViewArrow(el);
+        appendChild(arrow.data(), true);
+        arrow->loadChildren(el);  
+      } else if (el.tagName() == "Ellipse") {
+        KstViewEllipsePtr ellipse = new KstViewEllipse(el);
+        appendChild(ellipse.data(), true);
+        ellipse->loadChildren(el);  
+      } else if (el.tagName() == "Label") {
+        KstViewLabelPtr label = new KstViewLabel(el);
+        appendChild(label.data(), true);
+        label->loadChildren(el);  
+      } else if (el.tagName() == "Legend") {
+        KstViewLegendPtr legend = new KstViewLegend(el);
+        appendChild(legend.data(), true);
+        legend->loadChildren(el);  
+      } else if (el.tagName() == "Picture") {
+        KstViewPicturePtr picture = new KstViewPicture(el);
+        appendChild(picture.data(), true);
+        picture->loadChildren(el);  
       }
     }
     n = n.nextSibling();
@@ -126,10 +211,27 @@ void KstViewObject::loadChildren(QDomElement& e) {
 }
 
 
-KstObject::UpdateType KstViewObject::update(int) {
-  return NO_CHANGE;
+KstObject::UpdateType KstViewObject::update(int counter) {
+  if (checkUpdateCounter(counter)) {
+    return lastUpdateResult();
+  }
+  
+  KstObject::UpdateType rc = updateChildren(counter);
+
+  return setLastUpdateResult(rc);
 }
 
+KstObject::UpdateType KstViewObject::updateChildren(int counter) {
+  KstObject::UpdateType rc = NO_CHANGE;
+  for (KstViewObjectList::Iterator i = _children.begin(); i != _children.end(); ++i) {
+    if (rc == NO_CHANGE) {
+      rc = (*i)->update(counter);
+    } else {
+      (*i)->update(counter);
+    }
+  }
+  return rc;
+}  
 
 void KstViewObject::save(QTextStream& ts, const QString& indent) {
   KstAspectRatio aspect;
@@ -140,146 +242,164 @@ void KstViewObject::save(QTextStream& ts, const QString& indent) {
     aspect = _aspect;
   }
 
+  if (_transparent) {
+    ts << indent << "<transparent/>" << endl;
+  }
   ts << indent << "<tag>" << QStyleSheet::escape(tagName()) << "</tag>" << endl;
   ts << indent << "<aspect x=\"" << aspect.x <<
     "\" y=\"" << aspect.y <<
     "\" w=\"" << aspect.w <<
     "\" h=\"" << aspect.h << "\" />" << endl;
+  
+  // save all properties
+  for (int i = 0; i < metaObject()->numProperties(true); i++) {
+    ts << indent << "<" << metaObject()->property(i, true)->name() << ">";
+    ts << property(metaObject()->property(i, true)->name()).toString().latin1();
+    ts << "</" << metaObject()->property(i, true)->name() << ">" << endl;
+  }
 
   for (KstViewObjectList::Iterator i = _children.begin(); i != _children.end(); ++i) {
-    (*i)->save(ts, indent + "  ");
+    if ((*i)->type() == "plot") {
+      (*i)->saveTagOnce(ts, indent); // special case: only save tags for plots when they are children  
+    } else {
+      (*i)->save(ts, indent + "  "); // all other view objects are saved normally
+    }
   }
 }
 
 
 void KstViewObject::saveTag(QTextStream& ts, const QString& indent) {
-  KstAspectRatio aspect;
-
-  if (_maximized) {
-    aspect = _aspectOldZoomedObject;
-  } else {
-    aspect = _aspect;
-  }
-
-  ts << indent << "<tag>" << QStyleSheet::escape(tagName()) << "</tag>" << endl;
-  ts << indent << "<aspect x=\"" << aspect.x <<
-    "\" y=\"" << aspect.y <<
-    "\" w=\"" << aspect.w <<
-    "\" h=\"" << aspect.h << "\" />" << endl;
+  QString l2 = indent + "  ";
+  ts << indent << "<" << type() << ">" << endl;
+  ts << l2 << "<tag>" << QStyleSheet::escape(tagName()) << "</tag>" << endl;
   for (KstViewObjectList::Iterator i = _children.begin(); i != _children.end(); ++i) {
-    (*i)->saveTag(ts, indent + "  ");
+    (*i)->saveTag(ts, l2);
   }
-}
-
-void KstViewObject::readLock() const {
-  abort();
+  ts << indent << "</" << type() << ">" << endl;
 }
 
 
-void KstViewObject::readUnlock() const {
-  abort();
+void KstViewObject::saveTagOnce(QTextStream& ts, const QString& indent) {
+  QString l2 = indent + "  ";
+  ts << indent << "<" << type() << ">" << endl;
+  ts << l2 << "<tag>" << QStyleSheet::escape(tagName()) << "</tag>" << endl;
+  for (KstViewObjectList::Iterator i = _children.begin(); i != _children.end(); ++i) {
+    if ((*i)->type() == "plot") {
+      (*i)->saveTagOnce(ts, indent); // special case: only save tags for plots when they are children  
+    } else {
+      (*i)->save(ts, indent + "  "); // all other view objects are saved normally
+    }
+  }
+  ts << indent << "</" << type() << ">" << endl;
 }
 
-
-void KstViewObject::writeLock() const {
-  abort();
-}
-
-
-void KstViewObject::writeUnlock() const {
-  abort();
-}
-
-
-void KstViewObject::paint(KstPaintType type, QPainter& p) {
+void KstViewObject::paint(KstPaintType type, QPainter& p, const QRegion& bounds) {
+  p.save();
+  p.setViewport(geometry());
+  p.setWindow(geometry());
   bool hadClipping = p.hasClipping();
   QRegion oldRegion = p.clipRegion();
   bool maximized = false;
+  bool nullBounds = bounds.isNull();
 
+  //update();
   // handle the case where we have maximized plots
   for (KstViewObjectList::Iterator i = _children.begin(); i != _children.end(); ++i) {
     if ((*i)->_maximized) {
       (*i)->_lastClipRegion = (*i)->geometry();
-      (*i)->paint(type, p);
+      (*i)->paint(type, p, bounds);
       maximized = true;
       break;
     }
   }
 
-  if (!maximized) {
-    for (KstViewObjectList::Iterator i = _children.begin(); i != _children.end(); ++i) {
+  if (!maximized && !_children.isEmpty()) {
+    QRegion clipRegion = oldRegion;
+    KstViewObjectList::Iterator begin = _children.begin();
+    for (KstViewObjectList::Iterator i = _children.fromLast();; --i) {
+      if (nullBounds || !bounds.intersect(QRegion((*i)->geometry())).isEmpty()) {
 #ifdef BENCHMARK
-      QTime t;
-      t.start();
+        QTime t;
+        t.start();
+#endif
+        (*i)->_lastClipRegion = clipRegion;
+        p.setClipRegion(clipRegion);
+        (*i)->paint(type, p, bounds);
+        clipRegion -= (*i)->clipRegion();
+#ifdef BENCHMARK
+        int x = t.elapsed();
+        kstdDebug() << "   -> object " << (*i)->tagName() << " took " << x << "ms" << endl;
 #endif
-      QRegion clipRegion = oldRegion;
-      KstViewObjectList::Iterator j = i;
-      for (++j; j != _children.end(); ++j) {
-        clipRegion -= QRegion((*j)->geometry());
       }
-      (*i)->_lastClipRegion = clipRegion;
-      p.setClipRegion(clipRegion);
-      (*i)->paint(type, p);
-#ifdef BENCHMARK
-      int x = t.elapsed();
-      kdDebug() << "   -> object " << (*i)->tagName() << " took " << x << "ms" << endl;
-#endif
-      if ((*i)->_maximized) {
+      if (i == begin) {
         break;
       }
     }
   }
-  
+
   p.setClipRegion(oldRegion);
 
   if (_focus) {
-    //kdDebug() << "Object " << tagName() << " has focus" << endl;
-    drawFocusRect(p);
+    //kstdDebug() << "Object " << tagName() << " has focus" << endl;
+    //drawFocusRect(p);
   }
 
   if (isSelected()) {
-    //kdDebug() << "Object " << tagName() << " is selected" << endl;
+    //kstdDebug() << "Object " << tagName() << " is selected" << endl;
     drawSelectRect(p);
   }
 
   p.setClipping(hadClipping);
+  p.restore();
+  p.flush();
+  setDirty(false);
 }
 
 
 void KstViewObject::drawFocusRect(QPainter& p) {
-  QPen pen(Qt::black, 2);
-  p.setPen(pen);
-  QRect r;
-  r.setX(_geom.left());
-  r.setY(_geom.top());
-  r.setWidth(_geom.width());
-  r.setHeight(_geom.height());
-  Qt::RasterOp op = p.rasterOp();
-  // FIXME: Qt bug?  Appears that clipping is ignored here
-  p.setRasterOp(Qt::NotROP);
-  p.drawWinFocusRect(r);
-  p.setRasterOp(op);
+      
+  // to draw a focus rect, put something here
+  // draw the 8 hotpoints
+  QPoint topLeft = geometry().topLeft();
+  QPoint topRight = geometry().topRight();
+  QPoint bottomLeft = geometry().bottomLeft();
+  QPoint bottomRight = geometry().bottomRight();
+  QPoint topMiddle = QPoint((topLeft.x() + topRight.x())/2, topLeft.y());
+  QPoint bottomMiddle = QPoint((bottomLeft.x() + bottomRight.x())/2, bottomLeft.y());
+  QPoint middleLeft = QPoint(topLeft.x(), (topLeft.y() + bottomLeft.y())/2);
+  QPoint middleRight = QPoint(topRight.x(), (topRight.y() + bottomRight.y())/2);
+  
+  int dx = KST_RESIZE_BORDER_W/2;
+  int width = 2*dx + 1;
+  
+  p.drawRect(topLeft.x()-dx, topLeft.y()-dx, width, width);
+  p.drawRect(topRight.x()-dx, topRight.y()-dx, width, width);
+  p.drawRect(bottomLeft.x()-dx, bottomLeft.y()-dx, width, width);
+  p.drawRect(bottomRight.x()-dx, bottomRight.y()-dx, width, width);
+  p.drawRect(topMiddle.x()-dx, topMiddle.y()-dx, width, width);
+  p.drawRect(bottomMiddle.x()-dx, bottomMiddle.y()-dx, width, width);
+  p.drawRect(middleLeft.x()-dx, middleLeft.y()-dx, width, width);
+  p.drawRect(middleRight.x()-dx, middleRight.y()-dx, width, width);
 }
 
 
 void KstViewObject::drawSelectRect(QPainter& p) {
-  QPen pen(Qt::black, 2);
-  p.setPen(pen);
-  QRect r;
-  r.setX(_geom.left() + 2);
-  r.setY(_geom.top() + 2);
-  r.setWidth(_geom.width() - 4);
-  r.setHeight(_geom.height() - 4);
-  Qt::RasterOp op = p.rasterOp();
-  p.setRasterOp(Qt::CopyROP);
-  p.drawRect(r);
-  p.setRasterOp(op);
+  p.setBrush(QBrush(Qt::green));
+  p.setPen(QPen(Qt::black, 0));
+  drawFocusRect(p);
 }
 
 
 void KstViewObject::appendChild(KstViewObjectPtr obj, bool keepAspect) {
   obj->_parent = this;
   _children.append(obj);
+
+  for (KstViewObjectList::Iterator i = children().begin(); i != children().end(); ++i) {
+    if ((*i)->maximized()) {
+      (*i)->setMaximized(false);
+    }
+  }
+
   if (keepAspect) {
     obj->updateFromAspect();
   } else {
@@ -291,6 +411,13 @@ void KstViewObject::appendChild(KstViewObjectPtr obj, bool keepAspect) {
 void KstViewObject::prependChild(KstViewObjectPtr obj, bool keepAspect) {
   obj->_parent = this;
   _children.prepend(obj);
+
+  for (KstViewObjectList::Iterator i = _children.begin(); i != _children.end(); ++i) {
+    if ((*i)->maximized()) {
+      (*i)->setMaximized(false);
+    }
+  }
+
   if (keepAspect) {
     obj->updateFromAspect();
   } else {
@@ -320,6 +447,13 @@ void KstViewObject::insertChildAfter(const KstViewObjectPtr after, KstViewObject
     _children.prepend(obj);
   }
   obj->_parent = this;
+
+  for (KstViewObjectList::Iterator i = _children.begin(); i != _children.end(); ++i) {
+    if ((*i)->maximized()) {
+      (*i)->setMaximized(false);
+    }
+  }
+
   if (keepAspect) {
     obj->updateFromAspect();
   } else {
@@ -347,10 +481,17 @@ KstViewObjectList& KstViewObject::children() {
 
 
 void KstViewObject::resize(const QSize& size) {
-  assert(size.width() > 0);
-  assert(size.height() > 0);
-  _geom.setSize(size);
+  double xc, yc;
+  
+  xc = _aspect.x;  // preserve position in double precision to avoid wander on resize...
+  yc = _aspect.y;
+  
+  setMinimumSize(minimumSize().expandedTo(QSize(_children.count(), _children.count())));
+  _geom.setSize(size.expandedTo(_minimumSize));
   updateAspect();
+  _aspect.x = xc; // restore position.
+  _aspect.y = yc;
+  updateFromAspect();
   for (KstViewObjectList::Iterator i = _children.begin(); i != _children.end(); ++i) {
     (*i)->parentResized();
   }
@@ -360,6 +501,7 @@ void KstViewObject::resize(const QSize& size) {
 void KstViewObject::resizeForPrint(const QSize& size) {
   _geomOld = _geom;
   _geom.setSize(size);
+  _clipMask = QRegion();
   for (KstViewObjectList::Iterator i = _children.begin(); i != _children.end(); ++i) {
     (*i)->parentResizedForPrint();
   }
@@ -368,6 +510,7 @@ void KstViewObject::resizeForPrint(const QSize& size) {
 
 void KstViewObject::revertForPrint() {
   _geom = _geomOld;
+  _clipMask = QRegion();
   for (KstViewObjectList::Iterator i = _children.begin(); i != _children.end(); ++i) {
     (*i)->parentRevertedForPrint();
   }
@@ -391,11 +534,8 @@ QSize KstViewObject::size() const {
 void KstViewObject::internalAlignment(KstPaintType type, QPainter& p, QRect& plotRegion) {
   Q_UNUSED(type)
   Q_UNUSED(p)
-
-  plotRegion.setLeft(0);
-  plotRegion.setRight(0);
-  plotRegion.setTop(0);
-  plotRegion.setLeft(0);
+  static const QRect x(0,0,0,0);
+  plotRegion = x;
 }
 
 
@@ -418,6 +558,7 @@ void KstViewObject::parentResizedForPrint() {
 
 void KstViewObject::parentRevertedForPrint() {
   _geom = _geomOld;
+  _clipMask = QRegion();
   for (KstViewObjectList::Iterator i = _children.begin(); i != _children.end(); ++i) {
     (*i)->parentRevertedForPrint();
   }
@@ -425,6 +566,7 @@ void KstViewObject::parentRevertedForPrint() {
 
 
 void KstViewObject::setBackgroundColor(const QColor& color) {
+  setDirty();
   _backgroundColor = color;
 }
 
@@ -435,6 +577,7 @@ QColor KstViewObject::backgroundColor() const {
 
 
 void KstViewObject::setForegroundColor(const QColor& color) {
+  setDirty();
   _foregroundColor = color;
 }
 
@@ -465,39 +608,46 @@ void KstViewObject::setOnGrid(bool on_grid) {
 
 
 void KstViewObject::cleanup(int cols) {
-  if (_children.count() < 1) {
+  KstViewObjectList childrenCopy;
+    
+  for (KstViewObjectList::ConstIterator i = _children.begin(); i != _children.end(); ++i) {
+    if ((*i)->followsFlow()) {
+      childrenCopy.append(*i);
+    }
+  }
+  
+  int cnt = childrenCopy.count();
+  if (cnt < 1) {
     return;
   }
 
+  // FIXME: don't allow regrid to a number of columns that will result in
+  //        >= height() plots in a column
   if (!_onGrid) {
     if (cols <= 0) {
-      cols = int(sqrt(_children.count()));
+      cols = int(sqrt(cnt));
     }
     _onGrid = true;
     _columns = QMAX(1, cols);
   } else {
     if (cols > 0) {
       _columns = cols;
-    } else if (_columns <= 0) {
-      _columns = QMAX(1, int(sqrt(_children.count())));
+    } else if (cols <= 0){
+      _columns = QMAX(1, int(sqrt(cnt)));
     }
-  }
+  }  
+  int rows = ( cnt + _columns - 1 ) / _columns;
 
-  KstViewObjectList childrenCopy;
-  double distance;
   double minDistance = 0.0;
   int pos = 0;
   int x = 0;
   int y = 0;
   int w = _geom.width() / _columns;
-  int lastRow = _children.count() / _columns;
-  int h = _geom.height() / (lastRow + (_children.count() % _columns > 0 ? 1 : 0));
-
-  childrenCopy = QDeepCopy<KstViewObjectList>(_children);
-
-  //kdDebug() << "cleanup with w=" << w << " and h=" << h << endl;
-  //kdDebug() << "columns=" << _columns  << endl;
-  for (unsigned i = 0; i < _children.count(); ++i) {
+  int h = _geom.height() / rows;
+    
+  //kstdDebug() << "cleanup with w=" << w << " and h=" << h << endl;
+  //kstdDebug() << "columns=" << _columns  << endl;
+  for (int i = 0; i < cnt; ++i) {
     KstViewObjectList::Iterator nearest = childrenCopy.end();
     QPoint pt(x, y);
     QSize sz(w, h);
@@ -508,23 +658,25 @@ void KstViewObject::cleanup(int cols) {
     }
 
     // adjust the last row to be sure that we don't spill over
-    if ((pos + 1) / _columns > lastRow) {
+    if (pos / _columns == rows - 1) {
       sz.setHeight(_geom.height() - y);
     }
 
     for (KstViewObjectList::Iterator it = childrenCopy.begin(); it != childrenCopy.end(); ++it) {
       // find plot closest to the desired position, based on top-left corner...
-      distance  = (double)(( x - (*it)->geometry().x() ) * ( x - (*it)->geometry().x() ));
-      distance += (double)(( y - (*it)->geometry().y() ) * ( y - (*it)->geometry().y() ));
-      if( it == childrenCopy.begin() || distance < minDistance ) {
+      double distance = double((x - (*it)->geometry().x()) * (x - (*it)->geometry().x())) + 
+                        double((y - (*it)->geometry().y()) * (y - (*it)->geometry().y()));
+      if (it == childrenCopy.begin() || distance < minDistance) {
         minDistance = distance;
         nearest = it;
       }
     }
+
     if (nearest != childrenCopy.end()) {
-      (*nearest)->move(pt);
-      (*nearest)->resize(sz);
-      childrenCopy.remove(*nearest);
+      KstViewObjectPtr vop = *nearest;
+      vop->move(pt);
+      vop->resize(sz);
+      childrenCopy.remove(vop);
     }
 
     if (++pos % _columns == 0) {
@@ -553,13 +705,15 @@ void KstViewObject::parentMoved(const QPoint& offset) {
 void KstViewObject::move(const QPoint& pos) {
   QPoint offset = pos - _geom.topLeft();
 
-  if( !offset.isNull() ) {
+  if (!offset.isNull()) {
     _geom.moveTopLeft(pos);
-    updateAspectPos( );
+    updateAspectPos();
     for (KstViewObjectList::Iterator i = _children.begin(); i != _children.end(); ++i) {
       (*i)->parentMoved(offset);
     }
   }
+
+  _clipMask = QRegion(); // not dirty, but need to rebuild
 }
 
 
@@ -609,7 +763,7 @@ KstViewObjectPtr KstViewObject::findChild(const QString& name, bool recursive) {
 }
 
 
-KstViewObjectPtr KstViewObject::findChild(const QPoint& pos) {
+KstViewObjectPtr KstViewObject::findChild(const QPoint& pos, bool borderForTransparent) {
   KstViewObjectPtr obj;
 
   if (!_geom.contains(pos) || _children.isEmpty()) {
@@ -618,14 +772,33 @@ KstViewObjectPtr KstViewObject::findChild(const QPoint& pos) {
 
   KstViewObjectList::Iterator i = _children.end();
   for (--i; ; --i) {
-    if (QRect((*i)->position(), (*i)->size()).contains(pos)) {
-      if (obj == 0L || (*i)->_maximized) {
+    if ((*i)->surroundingGeometry().contains(pos)) {
+      if ((*i)->_maximized) {
         obj = *i;
-        if ((*i)->_maximized) {
-          break;
+        break;
+      }
+      if (!obj) {
+        if ((*i)->transparent()) {
+          if ((*i)->clipRegion().contains(pos)) {
+            obj = *i;
+          } else if (borderForTransparent && (*i)->geometry().contains(pos)) {
+            QRect g = (*i)->geometry();
+            if ((pos.x() >= g.left() && pos.x() <= g.left() + KST_RESIZE_BORDER_W) ||
+                (pos.x() <= g.right() && pos.x() >= g.right() - KST_RESIZE_BORDER_W) ||
+                (pos.y() >= g.top() && pos.y() <= g.top() + KST_RESIZE_BORDER_W) ||
+                (pos.y() <= g.bottom() && pos.y() >= g.bottom() - KST_RESIZE_BORDER_W)) {
+              obj = *i;
+            }
+          }
+        } else {
+          obj = *i;
         }
       }
     }
+    // one last check - hotpoints
+    if (!obj && (*i)->isSelected() && (*i)->directionFor(pos) > 0) {
+      obj = *i;  
+    }
     if (i == _children.begin()) {
       break;
     }
@@ -645,7 +818,7 @@ QRect KstViewObject::contentsRect() const {
 }
 
 
-void KstViewObject::setContentsRect(QRect& rect) {
+void KstViewObject::setContentsRect(const QRect& rect) {
   _geom = rect;
 }
 
@@ -657,11 +830,12 @@ QString KstViewObject::menuTitle() const {
 
 bool KstViewObject::popupMenu(KPopupMenu *menu, const QPoint& pos, KstViewObjectPtr topParent) {
   Q_UNUSED(pos)
-  Q_UNUSED(topParent)
   bool rc = false;
   int id;
   QString menuTitle = this->menuTitle();
 
+  _topObjectForMenu = topParent;
+
   if (!menuTitle.isEmpty()) {
     menu->insertTitle(menuTitle);
   }
@@ -676,6 +850,11 @@ bool KstViewObject::popupMenu(KPopupMenu *menu, const QPoint& pos, KstViewObject
     rc = true;
   }
 
+  if (_standardActions & Copy) {
+    menu->insertItem(i18n("&Copy"), this, SLOT(copyObject()));
+    rc = true;
+  }
+
   if (_standardActions & Zoom) {
     id = menu->insertItem(i18n("Maximi&ze"), this, SLOT(zoomToggle()));
     if (_maximized) {
@@ -699,6 +878,8 @@ bool KstViewObject::popupMenu(KPopupMenu *menu, const QPoint& pos, KstViewObject
 bool KstViewObject::layoutPopupMenu(KPopupMenu *menu, const QPoint& pos, KstViewObjectPtr topParent) {
   Q_UNUSED(pos)
 
+  _topObjectForMenu = topParent;
+
   bool rc = false;
   int id;
 
@@ -713,6 +894,11 @@ bool KstViewObject::layoutPopupMenu(KPopupMenu *menu, const QPoint& pos, KstView
     rc = true;
   }
 
+  if (_layoutActions & Copy) {
+    menu->insertItem(i18n("&Copy"), this, SLOT(copyObject()));
+    rc = true;
+  }
+
   if (_layoutActions & Raise) {
     menu->insertItem(i18n("&Raise"), this, SLOT(raise()));
     rc = true;
@@ -772,16 +958,54 @@ bool KstViewObject::layoutPopupMenu(KPopupMenu *menu, const QPoint& pos, KstView
     rc = true;
   }
 
+  if (_layoutActions & CopyTo) {
+    int i = 0;
+    bool hasEntry = false;
+
+    KPopupMenu *submenu = new KPopupMenu(menu);
+
+    id = menu->insertItem(i18n("&Copy To"), submenu);
+
+    KMdiIterator<KMdiChildView*> *it = KstApp::inst()->createIterator();
+    while (it->currentItem()) {
+      KstViewWindow *c = dynamic_cast<KstViewWindow*>(it->currentItem());
+      KstTopLevelViewPtr tlv = kst_cast<KstTopLevelView>(topParent);
+
+      if (c) {
+        hasEntry = true;
+        if (tlv && tlv == c->view()) {
+          submenu->insertItem(i18n("%1 (here)").arg(it->currentItem()->caption()), i);
+        } else {
+          submenu->insertItem(it->currentItem()->caption(), i);
+        }
+        submenu->connectItem(i, this, SLOT(copyTo(int)));
+        _copyToMap[i] = it->currentItem()->caption();
+        i++;
+      }
+      it->next();
+    }
+    KstApp::inst()->deleteIterator(it);
+
+    menu->setItemEnabled(id, hasEntry);
+
+    rc = true;
+  }
+    
   return rc;
 }
 
 
+void KstViewObject::edit() {
+  KstTopLevelViewPtr tlv = kst_cast<KstTopLevelView>(KstViewObjectPtr(_topObjectForMenu));
+  showDialog(tlv);
+}
+
+
 void KstViewObject::deleteObject() {
-  // FIXME: eliminate _parent!!
   KstApp::inst()->document()->setModified();
   KstViewObjectPtr vop(this);
-  if (_parent) {
-    _parent->removeChild(this);
+  if (_topObjectForMenu) {
+    _topObjectForMenu->removeChild(this, true);
     _parent = 0L;
   }
   while (!_children.isEmpty()) {
@@ -792,6 +1016,16 @@ void KstViewObject::deleteObject() {
 }
 
 
+void KstViewObject::copyObject() {
+}
+
+
+void KstViewObject::copyObjectQuietly(KstViewObject &parent, const QString& name) const {
+  Q_UNUSED(parent)
+  Q_UNUSED(name)
+}
+
+
 void KstViewObject::raiseToTop() {
   // FIXME: eliminate _parent!!
   KstApp::inst()->document()->setModified();
@@ -804,6 +1038,7 @@ void KstViewObject::raiseToTop() {
       _parent->_children.append(t);
     }
   }
+  setDirty();
 }
 
 
@@ -819,6 +1054,7 @@ void KstViewObject::lowerToBottom() {
       _parent->_children.prepend(t);
     }
   }
+  setDirty();
 }
 
 
@@ -839,6 +1075,7 @@ void KstViewObject::raise() {
       }
     }
   }
+  setDirty();
 }
 
 
@@ -859,16 +1096,18 @@ void KstViewObject::lower() {
       }
     }
   }
+  setDirty();
 }
 
 
+// FIXME: delete
 void KstViewObject::moveTo(int id) {
-  // FIXME: eliminate _parent!!
   KstApp::inst()->document()->setModified();
   QString windowName = _moveToMap[id];
   KMdiChildView *mdiChild;
   KstViewWindow *window;
 
+  setDirty();
   if (!windowName.isEmpty()) {
     mdiChild = KstApp::inst()->findWindow(windowName);
     if (mdiChild) {
@@ -891,14 +1130,40 @@ void KstViewObject::moveTo(int id) {
 }
 
 
+// FIXME: delete
+void KstViewObject::copyTo(int id) {
+  KstApp::inst()->document()->setModified();
+  QString windowName = _copyToMap[id];
+  KMdiChildView *mdiChild;
+  KstViewWindow *window;
+
+  setDirty();
+  if (!windowName.isEmpty()) {
+    mdiChild = KstApp::inst()->findWindow(windowName);
+    if (mdiChild) {
+      window = dynamic_cast<KstViewWindow*>(mdiChild);
+      if (window) {
+        copyObjectQuietly(*(window->view().data()));
+      }
+    }
+  }
+}
+
+
 void KstViewObject::updateFromAspect() {
   // FIXME: eliminate _parent!!
+  setMinimumSize(minimumSize().expandedTo(QSize(_children.count(), _children.count())));
   if (_parent) {
     _geom.setLeft(_parent->geometry().left() + int(_aspect.x * _parent->geometry().width()));
     _geom.setTop(_parent->geometry().top() + int(_aspect.y * _parent->geometry().height()));
-    _geom.setRight(_parent->geometry().left() + int((_aspect.x + _aspect.w) * _parent->geometry().width()));
-    _geom.setBottom(_parent->geometry().top() + int((_aspect.y + _aspect.h) * _parent->geometry().height()));
+    _geom.setRight(_parent->geometry().left() + int((_aspect.x + _aspect.w) * _parent->geometry().width()) - 1);
+    _geom.setBottom(_parent->geometry().top() + int((_aspect.y + _aspect.h) * _parent->geometry().height()) - 1);
   }
+  if (_geom.width() >= _minimumSize.width() || _geom.height() >= _minimumSize.height()) {
+    _geom.setSize(_geom.size().expandedTo(_minimumSize));
+  }
+  assert(_geom.left() >= 0 && _geom.top() >= 0);
+  setDirty();
 }
 
 
@@ -929,6 +1194,7 @@ void KstViewObject::updateAspectSize() {
 void KstViewObject::updateAspect() {
   updateAspectSize();
   updateAspectPos();
+  setDirty();
 }
 
 
@@ -947,8 +1213,13 @@ void KstViewObject::setSelected(bool selected) {
 }
 
 
-void KstViewObject::zoom(bool zoom) {
-  if (_maximized != zoom) {
+bool KstViewObject::maximized() const {
+  return _maximized;
+}
+
+
+void KstViewObject::setMaximized(bool maximized) {
+  if (_maximized != maximized) {
     zoomToggle();
   }
 }
@@ -977,6 +1248,7 @@ void KstViewObject::zoomToggle() {
   for (KstViewObjectList::Iterator it = _children.begin(); it != _children.end(); ++it) {
     (*it)->parentResized();
   }
+  setDirty();
 }
 
 
@@ -1004,7 +1276,11 @@ void KstViewObject::detach() {
 
 void KstViewObject::rename() {
   bool ok = false;
+#if KDE_VERSION >= KDE_MAKE_VERSION(3,3,0)
+  QString newName = KInputDialog::getText(i18n("Kst"), i18n("Enter a new name for %1:").arg(tagName()), tagName(), &ok);
+#else
   QString newName = KLineEditDlg::getText(i18n("Enter a new name for %1:").arg(tagName()), tagName(), &ok, 0L);
+#endif
   if (ok) {
     setTagName(newName);
   }
@@ -1051,7 +1327,7 @@ void KstViewObject::readBinary(QDataStream& str) {
   QString tagName;
   str >> tagName;
   setTagName(tagName);
-  kdDebug() << "Decoding " << tagName << " from drag." << endl;
+  kstdDebug() << "Decoding " << tagName << " from drag." << endl;
   // FIXME: rename objects if they cause a namespace conflict
   str >> _geom >> _backgroundColor >> _foregroundColor;
   str >> _standardActions >> _layoutActions >> _aspect;
@@ -1106,6 +1382,12 @@ void KstViewObject::mousePressEvent(QWidget *view, QMouseEvent *e) {
 }
 
 
+void KstViewObject::mouseDoubleClickEvent(QWidget *view, QMouseEvent *e) {
+  Q_UNUSED(view)
+  Q_UNUSED(e)
+}
+
+
 void KstViewObject::mouseReleaseEvent(QWidget *view, QMouseEvent *e) {
   Q_UNUSED(view)
   Q_UNUSED(e)
@@ -1164,10 +1446,173 @@ void KstViewObject::unselectAll() {
 
 bool KstViewObject::contains(KstViewObjectPtr child) {
   for (KstViewObjectList::Iterator i = _children.begin(); i != _children.end(); ++i) {
-    if ((*i).data() == child.data()) {
+    if ((*i).data() == child.data() || (*i)->contains(child)) {
       return true;
     }
-    if ((*i)->contains(child)) {
+  }
+  return false;
+}
+
+
+const QString& KstViewObject::type() const {
+  return _type;
+}
+
+
+void KstViewObject::setMinimumSize(const QSize& sz) {
+  _minimumSize = sz.expandedTo(QSize(1, 1)); // 1,1 is the absolute minimum
+}
+
+
+const QSize& KstViewObject::minimumSize() const {
+  return _minimumSize;
+}
+
+
+void KstViewObject::setTransparent(bool transparent) {
+  if (_transparent != transparent) {
+    setDirty();
+  }
+  _transparent = transparent;
+}
+
+
+bool KstViewObject::transparent() const {
+  return _transparent;
+}
+
+
+QRegion KstViewObject::clipRegion() {
+  if (_clipMask.isNull()) {
+    if (_transparent) {
+      QBitmap bm(_geom.bottomRight().x(), _geom.bottomRight().y(), true);
+      if (!bm.isNull()) {
+        QPainter p(&bm);
+        p.setRasterOp(Qt::SetROP);
+        p.setViewXForm(true);
+        paint(P_PAINT, p, QRegion());
+        p.flush();
+        _clipMask = QRegion(bm);
+      } else {
+        _clipMask = QRegion();
+      }
+    } else {
+      _clipMask = QRegion(_geom);
+    }
+  }
+  
+  return _clipMask;
+}
+
+
+void KstViewObject::setFollowsFlow(bool follow) {
+  _followsFlow = follow;
+}
+
+
+bool KstViewObject::followsFlow() const {
+  return _followsFlow;
+}
+
+
+// id=0 v1.2.0 setDirty() was called.  data == 0 means false, else true
+void KstViewObject::virtual_hook(int id, void *data) {
+  switch (id) {
+    case 0:
+      if (data) {
+        _clipMask = QRegion();
+      }
+      break;
+    default:
+      break;
+  }
+  KstObject::virtual_hook(id, data);
+}
+
+
+QMap<QString, QVariant > KstViewObject::widgetHints(const QString& propertyName) const {
+  Q_UNUSED(propertyName)
+  return QMap<QString, QVariant>();
+}
+
+
+inline bool pointsCloseEnough(const QPoint& point1, const QPoint& point2) {
+  const int dx = KST_RESIZE_BORDER_W/2;
+  return point1.x() <= point2.x() + dx &&
+      point1.x() >= point2.x() - dx &&
+      point1.y() <= point2.y() + dx &&
+      point1.y() >= point2.y() - dx;
+}
+
+
+signed int KstViewObject::directionFor(const QPoint& pos) {
+  
+  // no hotpoints if object is not selected - can only move it
+  if (!isSelected()) {
+    return NONE;  
+  }
+  
+  signed int direction = NONE;
+  
+  QPoint topLeft = geometry().topLeft();
+  QPoint topRight = geometry().topRight();
+  QPoint bottomLeft = geometry().bottomLeft();
+  QPoint bottomRight = geometry().bottomRight();
+  QPoint topMiddle = QPoint((topLeft.x() + topRight.x())/2, topLeft.y());
+  QPoint bottomMiddle = QPoint((bottomLeft.x() + bottomRight.x())/2, bottomLeft.y());
+  QPoint middleLeft = QPoint(topLeft.x(), (topLeft.y() + bottomLeft.y())/2);
+  QPoint middleRight = QPoint(topRight.x(), (topRight.y() + bottomRight.y())/2);
+  
+  if (pointsCloseEnough(topLeft, pos)) {
+    direction |= UP | LEFT;  
+  } else if (pointsCloseEnough(topRight, pos)) {
+    direction |= UP | RIGHT;  
+  } else if (pointsCloseEnough(bottomLeft, pos)) {
+    direction |= DOWN | LEFT;  
+  } else if (pointsCloseEnough(bottomRight, pos)) {
+    direction |= DOWN | RIGHT;  
+  } else if (pointsCloseEnough(topMiddle, pos)) {
+    direction |= UP;  
+  } else if (pointsCloseEnough(bottomMiddle, pos)) {
+    direction |= DOWN;  
+  } else if (pointsCloseEnough(middleLeft, pos)) {
+    direction |= LEFT;  
+  } else if (pointsCloseEnough(middleRight, pos)) {
+    direction |= RIGHT;  
+  } 
+  
+  return direction;
+}
+
+
+void KstViewObject::showDialog(KstTopLevelViewPtr invoker) {
+  KstEditViewObjectDialogI *dlg = new KstEditViewObjectDialogI(KstApp::inst());
+  dlg->showEditViewObjectDialog(this, invoker);
+  dlg->exec();
+  delete dlg;
+}
+
+
+void KstViewObject::drawShadow(QPainter& p, const QPoint& pos) {
+  // default is a rectangle
+  QRect rect = geometry();
+  rect.moveTopLeft(pos);
+  p.drawRect(rect);  
+}
+
+
+QRect KstViewObject::surroundingGeometry() const {
+  return geometry();  
+}
+
+
+bool KstViewObject::objectDirty() const {
+  if (dirty()) {
+    return true;
+  }
+
+  for (KstViewObjectList::ConstIterator i = _children.begin(); i != _children.end(); ++i) {
+    if ((*i)->objectDirty()) {
       return true;
     }
   }
