@@ -1,5 +1,5 @@
 /***************************************************************************
-                                  main.cpp 
+                                  main.cpp
                              -------------------
     begin                : Tue Aug 22 13:46:13 CST 2000
     copyright            : (C) 2000 by Barth Netterfield
@@ -18,9 +18,12 @@
 #include <stdlib.h>
 #include <stdio.h>
 
+#include <qstringlist.h>
+#include <qfileinfo.h>
+
 #include <kaboutdata.h>
 #include <kcmdlineargs.h>
-#include <kdebug.h>
+#include "ksdebug.h"
 #include <kimageio.h>
 
 #include "kst.h"
@@ -30,24 +33,24 @@
 #include "kstdoc.h"
 #include "kstequation.h"
 #include "ksthistogram.h"
-#include "kstlegend.h"
+#include "kst2dplot.h"
+#include "kstviewlegend.h"
 #include "kstpsd.h"
 #include "kstrvector.h"
+#include "kstsvector.h"
+#include "kstavector.h"
 #include "kstvcurve.h"
-#include "kstversion.h"
-#include "plugincollection.h"
+#include "kstrmatrix.h"
+#include "kstimage.h"
 
 
-// George is going to freak out if main() gets more complicated. :-)
-// Let's try to come up with a KstCommandLineProcessor class that can
-// properly and cleanly deal with all of this.
-
 static const char description[] = I18N_NOOP("Kst: a data viewing program.");
-
+static QStringList startupErrors;
 
 static KCmdLineOptions options[] = {
   {"F <dataFile>", I18N_NOOP("Specify data file: used to override a kst file default"), "|"},
   { "y <Y>",  I18N_NOOP("Field for Y axis (multiple allowed)"), 0 },
+  { "z <Z>",  I18N_NOOP("Field for a Z image (multiple allowed)"), 0 },
   { "ye <equation>",  I18N_NOOP("Equation for Y axis (multiple allowed)"), 0 },
   { "e <E>",  I18N_NOOP("Field for Y errors (multiple allowed)"), 0 },
   { "x <X>",  I18N_NOOP("Field or range for X axis"),        "INDEX"},
@@ -65,6 +68,7 @@ static KCmdLineOptions options[] = {
   { "m <NC>", I18N_NOOP("Separate plots arranged in <NC> columns"),0},
   { "d",      I18N_NOOP("Display as points rather than curves"),0},
   { "g",      I18N_NOOP("Provide a legend box"),0},
+  { "w <file>",      I18N_NOOP("Display the data wizard"),"<none>"},
   { "print <file>",  I18N_NOOP("Print to file and exit"),"<none>"},
   { "png <file>",  I18N_NOOP("Save as a png file and exit"),"<none>"},
   { "nq",     I18N_NOOP("Bypass the quickstart dialog"), 0},
@@ -99,14 +103,13 @@ static void CheckForCMDErrors(KCmdLineArgs *args) {
   bool nOK;
 
   if (args->getOption("n").toInt(&nOK) == 0) {
-    kdError() << i18n("Exiting because '-n 0' requests vectors with no data in them.") << endl;
-    exit(0); // FIXME: can cause crashes
+    kstdError() << i18n("Exiting because '-n 0' requests vectors with no data in them.") << endl;
+    exit(0); // ugly but now safe
   }
   if (args->getOption("n").toInt(&nOK)>0) {
-    if (args->getOption("n").toInt(&nOK) <
-        2*args->getOption("s").toInt(&nOK)) {
-      kdError() << i18n("Requested data skipping would leave vector with fewer than two points.") << endl;
-      exit(0); // FIXME: can cause crashes
+    if (args->getOption("n").toInt(&nOK) < 2*args->getOption("s").toInt(&nOK)) {
+      kstdError() << i18n("Requested data skipping would leave vector with fewer than two points.") << endl;
+      exit(0); // ugly but now safe
     }
   }
 }
@@ -119,23 +122,37 @@ static void CheckForCMDErrors(KCmdLineArgs *args) {
 /****************************************************************/
 static void SetCMDOptions(KCmdLineArgs *args, InType &in, int n_y) {
   bool nOK;
+
   in.n = args->getOption("n").toInt(&nOK);
   in.f = args->getOption("f").toInt(&nOK);
   in.rate = args->getOption("r").toDouble(&nOK);
   in.len = args->getOption("l").toInt(&nOK);
-  if (in.n<-1) in.n = -1;
-  if (in.f<-1) in.f = -1;
-  if (in.rate<=0) in.rate = 1.0;
-  if (in.len<3) in.len = 3;
-  if (in.len>23) in.len = 23;
+  if (in.n < -1) {
+    in.n = -1;
+  }
+  if (in.f < -1) {
+    in.f = -1;
+  }
+  if (in.rate <= 0) {
+    in.rate = 1.0;
+  }
+  if (in.len < 3) {
+    in.len = 3;
+  }
+  if (in.len > 23) {
+    in.len = 23;
+  }
 
   in.has_points = args->isSet("d");
   in.n_cols = args->getOption("m").toInt(&nOK);
-  if (in.n_cols>0) {
-    if (args->count()>0) {
+  if (in.n_cols > 0) {
+    if (in.n_cols > 10) {
+      in.n_cols = 10;
+    }
+    if (args->count() > 0) {
       in.sep_plots = true;
       in.n_plots = args->count() * (n_y);
-    } else if (args->getOptionList("ye").count()>0) {
+    } else if (args->getOptionList("ye").count() > 0) {
       in.sep_plots = true;
       in.n_plots = args->getOptionList("ye").count();
     }
@@ -147,9 +164,9 @@ static void SetCMDOptions(KCmdLineArgs *args, InType &in, int n_y) {
 
   in.n_rows = (in.n_plots-1)/in.n_cols + 1;
 
-  /* set skip options */
+  // set skip options
   in.skip = args->getOption("s").toInt(&nOK);
-  if (in.skip<1) {
+  if (in.skip < 1) {
     in.skip = 0;
     in.doskip = false;
     in.doave = false;
@@ -158,7 +175,7 @@ static void SetCMDOptions(KCmdLineArgs *args, InType &in, int n_y) {
     in.doave = args->isSet("a");
   }
 
-  /* set Units for PSDs */
+  // set Units for PSDs
   in.VUnits = args->getOption("yu");
   in.RUnits = args->getOption("ru");
   in.dolegend = args->isSet("g");
@@ -172,28 +189,20 @@ static void SetCMDOptions(KCmdLineArgs *args, InType &in, int n_y) {
 static void CreatePlots(InType &in, KstTopLevelViewPtr tlv) {
   KstApp *kst = KstApp::inst();
   QString creatingPlots = i18n("Creating plots");
-  Kst2DPlotPtr plot;
-  int i_plot;
-  int size_boost;
-  int count;
-  int handled;
-  
-  kst->slotUpdateProgress( 0, 0, creatingPlots );
-  count = in.n_plots;
-  handled = 0;
 
-  for (i_plot=0; i_plot < in.n_plots; i_plot++) {
-    plot = new Kst2DPlot(KST::suggestPlotName());
-    size_boost = (in.n_rows + in.n_cols)/2-1;
-    size_boost = size_boost*12/2;
-    plot->initFonts(QFont::QFont(), size_boost);
+  kst->slotUpdateProgress( 0, 0, creatingPlots );
+  int count = in.n_plots;
+  int handled = 0;
+
+  for (int i_plot = 0; i_plot < in.n_plots; ++i_plot) {
+    Kst2DPlotPtr plot = new Kst2DPlot(KST::suggestPlotName());
     tlv->appendChild(plot.data());
 
     plot->resizeFromAspect(double(i_plot%in.n_cols)/double(in.n_cols),
                             double(i_plot/in.n_cols)/double(in.n_rows),
                             1.0/double(in.n_cols), 1.0/double(in.n_rows));
 
-    handled++;
+    ++handled;
     kst->slotUpdateProgress( count, handled, creatingPlots );
   }
 
@@ -212,14 +221,15 @@ static void CreatePlots(InType &in, KstTopLevelViewPtr tlv) {
 static void SetEqXRanges(QString xeqS, double *min, double *max, int *n, bool *ok) {
   *ok = true;
   QStringList fields = QStringList::split( QChar(':'), xeqS );
-  if (fields.count()!=3) {
-    *ok=false;
-    *n=2;
+  if (fields.count() != 3) {
+    *ok = false;
+    *n = 2;
     *min = -1;
     *max = 1;
     return;
   }
   QString f;
+
   f = *fields.at(0);
   *min = f.toDouble(ok);
   f = *fields.at(1);
@@ -227,8 +237,12 @@ static void SetEqXRanges(QString xeqS, double *min, double *max, int *n, bool *o
   f = *fields.at(2);
   *n = f.toInt(ok);
 
-  if (*n<2) *ok=false;
-  if (*min>=*max) *ok = false;
+  if (*n < 2) {
+    *ok=false;
+  }
+  if (*min >= *max) {
+    *ok = false;
+  }
 }
 
 /******************************************************************/
@@ -250,14 +264,17 @@ static KstRVector *GetOrCreateVector(const QString& field, KstDataSourcePtr file
       return V;
     }
   }
-  V = new KstRVector(file, field, QString("V") + QString::number(1 + i_v++)
-       + "-" + field, in.f, in.n, in.skip, in.doskip, in.doave);
-  if (!V->isValid()) {
-    kdError() << i18n("Invalid field: %1 in file %2").arg(V->field()).arg(V->filename()) << endl;
-    exit(0); // FIXME: this can cause crashes
+
+  V = new KstRVector(file, field, KST::suggestVectorName(field), in.f, in.n, in.skip, in.doskip, in.doave);
+  if (V->isValid()) {
+    KST::addVectorToList(V);
+  } else {
+    startupErrors.append(i18n("Failed to create vector '%1' from file '%2'.").arg(field).arg(file->fileName()));
+
+    delete V;
+    V = 0L;
   }
 
-  KST::addVectorToList(V);
   return V;
 }
 
@@ -282,7 +299,9 @@ static void ProcessEq(QString &eq, KstDataSourcePtr file, InType &in, bool *ok)
     if (i1>0) {
       field = eq.mid(i0+1, i1-i0-1);
       v = GetOrCreateVector(field, file, in);
-      eq.replace(i0,i1-i0+1, "["+v->tagName()+"]");
+      if (v) {
+        eq.replace(i0,i1-i0+1, "["+v->tagName()+"]");
+       }
     }
     i0++;
   }
@@ -301,12 +320,13 @@ static void exitHelper(void) {
 int main(int argc, char *argv[]) {
   int i_file, i_v, i_curve;
   int i_plot;
+  QString fullPath;
 
   KAboutData aboutData("kst", I18N_NOOP("Kst"),
                        KSTVERSION, description, KAboutData::License_GPL,
-                       I18N_NOOP("(c) 2000-2004 Barth Netterfield"),
+                       I18N_NOOP("(c) 2000-2005 Barth Netterfield"),
                        0,
-                       "http://extragear.kde.org/apps/kst.php");
+                       "http://kst.kde.org/");
   aboutData.addAuthor("Barth Netterfield",
                       I18N_NOOP("Original author and maintainer."),
                       "netterfield@astro.utoronto.ca",
@@ -327,6 +347,14 @@ int main(int argc, char *argv[]) {
                       "",
                       "",
                       "");
+  aboutData.addAuthor("Matthew Truch",
+                      "",
+                      "http://matt.truch.net/",
+                      "matt@truch.net");
+  aboutData.addAuthor("Theodore Kisner",
+                      "",
+                      "tskisner.public@gmail.com",
+                      "");
 
   KCmdLineArgs::init( argc, argv, &aboutData );
   KCmdLineArgs::addCmdLineOptions( options ); // Add our own options.
@@ -346,6 +374,7 @@ int main(int argc, char *argv[]) {
     QColor color;
     KstDataSourcePtr file;
     QCStringList ycolList;
+    QCStringList matrixList;
     QCStringList yEqList;
     QCStringList psdList;
     QCStringList hsList;
@@ -353,15 +382,16 @@ int main(int argc, char *argv[]) {
     unsigned int i_ycol;
     QCStringList::Iterator hs_string;
     QCStringList::Iterator eq_i;
+    QCStringList::Iterator mat_i;
     bool showQuickStart = false;
+    bool showDataWizard = false;
     bool nOK;
     int n_y = 0;
-    int n_eq = 0;
-
     KCmdLineArgs *args = KCmdLineArgs::parsedArgs();
 
     CheckForCMDErrors(args);
 
+    QString wizardfile = args->getOption("w");
     QString printfile = args->getOption("print");
     QString pngfile = args->getOption("png");
     bool print_and_exit = false;
@@ -373,34 +403,28 @@ int main(int argc, char *argv[]) {
       print_and_exit = true;
     }
 
-    // Initialise the plugin loader and collection.
-    PluginCollection::self();
-
     if (!print_and_exit) {
       app.setMainWidget(kst);
       kst->show();
     }
 
-    //
     // get Y axis columns
-    //
     ycolList = args->getOptionList("y");
+    matrixList = args->getOptionList("z");
     yEqList = args->getOptionList("ye");
     psdList = args->getOptionList("p");
     hsList = args->getOptionList("h");
     errorList = args->getOptionList("e");
 
-    //
     // y axis or PSD specified, so the files are data files, not kst files.
-    //
-    n_y = ycolList.count() + psdList.count() + hsList.count() + yEqList.count();
+    n_y = ycolList.count() + psdList.count() + hsList.count() + yEqList.count() + matrixList.count();
     if (n_y > 0) {
       QString creatingEquations = i18n("Creating equations");
       QString creatingCurves = i18n("Creating curves");
       QString creatingPlots = i18n("Creating plots");
       int count;
       int handled;
-      
+
       kst->slotUpdateProgress( 0, 0, QString::null );
 
       SetCMDOptions(args, in, n_y);
@@ -408,15 +432,13 @@ int main(int argc, char *argv[]) {
       KstTopLevelViewPtr tlv = kst->activeView();
 
       if (!tlv) {
-        //
         // if there was no active view then we create one...
-        //
         kst->newWindow(false);
         tlv = kst->activeView();
       }
 
       if (!tlv) {
-        kdError() << i18n("Can't create a view.") << endl;
+        kstdError() << i18n("Can't create a view.") << endl;
         return 0;
       }
 
@@ -426,16 +448,16 @@ int main(int argc, char *argv[]) {
       i_plot = 0;
       Kst2DPlotPtr plot = *plist.at(i_plot);
 
-      //
+      KstVCurveList vcurves = kstObjectSubList<KstBaseCurve,KstVCurve>(plot->Curves);
+
       // make stand alone equations if there are no files
-      //
       if (args->count() < 1) {
         if (!yEqList.isEmpty()) {
           QString eqS;
           double max, min;
           int n;
           bool xeq;
-          
+
           SetEqXRanges(args->getOption("xe"), &min, &max, &n, &xeq);
           if (xeq) {
             count = yEqList.size();
@@ -445,9 +467,10 @@ int main(int argc, char *argv[]) {
             for (eq_i = yEqList.begin(); eq_i != yEqList.end(); ++eq_i) {
               eqS = *eq_i;
               if (NoVectorEq(eqS)) {
-                KstEquationPtr eq = new KstEquation(QString("E") +QString::number(n_eq+1)+ "-" + eqS, eqS, min, max, n);
-                KstVCurvePtr vc = new KstVCurve(eq->tagName() + "-curve", eq->vX(), eq->vY(), 0L, 0L, KstColorSequence::next(plot->Curves,plot->backgroundColor()));
-
+                KstEquationPtr eq = new KstEquation(KST::suggestEQName(eqS), eqS, min, max, n);
+                KstVCurvePtr vc = new KstVCurve(KST::suggestCurveName(eq->tagName(), true),
+                                        eq->vX(), eq->vY(), 0L, 0L, 0L, 0L,
+                                        KstColorSequence::next(vcurves,plot->backgroundColor()));
                 KST::dataObjectList.lock().writeLock();
                 KST::dataObjectList.append(eq.data());
                 KST::dataObjectList.append(vc.data());
@@ -461,205 +484,248 @@ int main(int argc, char *argv[]) {
               }
 
               handled++;
-              kst->slotUpdateProgress( count, handled, creatingEquations );            
+              kst->slotUpdateProgress( count, handled, creatingEquations );
             }
           }
         }
       }
 
-      //
       // make the requested curves for each data file
-      //
       count = args->count();
       handled = 0;
       kst->slotUpdateProgress( count, handled, creatingCurves );
 
       for (i_curve = i_v = 0, i_file = 0; i_file < args->count(); i_file++) {
-        //
         // make the file
-        //
-        file = KstDataSource::loadSource(args->arg(i_file));
-
-        if (!file) {
-          kdError() << i18n("No data in file: %1").arg(args->arg(i_file)) << endl;
-          delete kst;
-          exit(0);
+        if (QFile::exists(args->arg(i_file))) {
+          fullPath = QFileInfo(args->arg(i_file)).absFilePath();
+        } else {
+          fullPath = args->arg(i_file);
         }
+        file = KstDataSource::loadSource(fullPath);
+        if (file) {
+          if (!file->isValid() || file->isEmpty()) {
+            kstdError() << i18n("No data in file %1.  Trying to continue...").arg(args->arg(i_file)) << endl;
+            // The file might get data later!
+          }
 
-        if (!file->isValid() || file->isEmpty()) {
-          kdError() << i18n("No data in file %1.  Trying to continue...").arg(args->arg(i_file)) << endl;
-          // The file might get data later!
-        }
+          KST::dataObjectList.lock().writeLock();
+          KST::dataSourceList.append(file);
+          KST::dataObjectList.lock().writeUnlock();
 
-        KST::dataObjectList.lock().writeLock();
-        KST::dataSourceList.append(file);
-        KST::dataObjectList.lock().writeUnlock();
+          KstRVectorPtr yvector;
+          KstRVectorPtr evector;
+          KstVCurvePtr curve;
+          KstPSDPtr psd;
+          KstHistogramPtr hs;
+          KstRVectorPtr xvector;
 
-        KstRVectorPtr yvector;
-        KstRVectorPtr evector;
-        KstVCurvePtr curve;
-        KstPSDPtr psd;
-        KstHistogramPtr hs;
-        KstRVectorPtr xvector;
+          if (!ycolList.isEmpty()) { // if there are some xy plots
+            // make the x axis vector
+            xvector = GetOrCreateVector(args->getOption("x"), file, in);
+            if (xvector) {
+              // make the y axis vectors
+              for (i_ycol = 0; i_ycol < ycolList.count(); ++i_ycol ) {
+                yvector = GetOrCreateVector(*(ycolList.at(i_ycol)), file, in);
+                if (yvector) {
+                  // make the curves
+                  color = KstColorSequence::next(vcurves,plot->backgroundColor());
+                  curve = new KstVCurve(KST::suggestCurveName(yvector->field(), false),
+                                      KstVectorPtr(xvector), KstVectorPtr(yvector),
+                                      0L, 0L, 0L, 0L, color);
+                  if (in.has_points) {
+                    curve->setHasPoints(true);
+                    curve->setHasLines(false);
+                  }
 
-        if (!ycolList.isEmpty()) { // if there are some xy plots
-          //
-          // make the x axis vector
-          //
-          xvector = GetOrCreateVector(args->getOption("x"), file, in);
+                  if (i_ycol<errorList.count()) {
+                    evector = GetOrCreateVector(*(errorList.at(i_ycol)), file, in);
+                    if (evector) {
+                      curve->setYError(KstVectorPtr(evector));
+                      curve->setYMinusError(KstVectorPtr(evector));
+                    }
+                  }
 
-          //
-          // make the y axis vectors
-          //
-          for (i_ycol = 0; i_ycol < ycolList.count(); ++i_ycol ) {
-            yvector = GetOrCreateVector(*(ycolList.at(i_ycol)), file, in);
+                  KST::dataObjectList.lock().writeLock();
+                  KST::dataObjectList.append(curve.data());
+                  KST::dataObjectList.lock().writeUnlock();
+                  plot->addCurve(curve.data());
 
-            //
-            // make the curves
-            //
-            color = KstColorSequence::next(plot->Curves,plot->backgroundColor());
-            curve = new KstVCurve(QString("C") + QString::number(1+i_curve++)
-                                  + "-" + yvector->field(),
-                                  KstVectorPtr(xvector), KstVectorPtr(yvector),
-                                  0L, 0L, color);
-            if (in.has_points) {
-              curve->setHasPoints(true);
-              curve->setHasLines(false);
+                  if (in.sep_plots) {
+                    plot->setTagName(curve->tagName());
+                    i_plot++;
+                    if (i_plot < in.n_plots) {
+                      plot = *plist.at(i_plot);
+                    }
+                  } // end (if they are separate plots)
+                }
+              } // next y col
             }
+          } // end (if there are some xy plots)
 
-            if (i_ycol<errorList.count()) {
-              evector = GetOrCreateVector(*(errorList.at(i_ycol)), file, in);
-              curve->setYError(KstVectorPtr(evector));
-            }
+          if (!yEqList.isEmpty()) {
+            QString eqS;
+            double max, min;
+            int n;
+            bool xeq, eq_ok;
 
-            KST::dataObjectList.lock().writeLock();
-            KST::dataObjectList.append(curve.data());
-            KST::dataObjectList.lock().writeUnlock();
-            plot->addCurve(curve.data());
+            SetEqXRanges(args->getOption("xe"), &min, &max, &n, &xeq);
+            for (eq_i = yEqList.begin(); eq_i != yEqList.end(); ++eq_i) {
+              KstEquationPtr eq;
 
-            if (in.sep_plots) {
-              i_plot++;
-              if (i_plot < in.n_plots) {
-                plot =  *plist.at(i_plot);
+              eqS = *eq_i;
+              ProcessEq(eqS, file, in, &eq_ok);
+
+              if (xeq) {
+                eq = new KstEquation(KST::suggestEQName(eqS), eqS, min,max,n);
+              } else {
+                if (!xvector) {
+                  xvector = GetOrCreateVector(args->getOption("x"), file, in);
+                }
+                if (xvector) {
+                  eq = new KstEquation(KST::suggestEQName(eqS), eqS, KstVectorPtr(xvector), true);
+                }
               }
-            } // end (if they are separate plots)
-          } // next y col
-        } // end (if there are some xy plots)
 
-        if (!yEqList.isEmpty()) {
-          QString eqS;
-          double max, min;
-          int n;
-          bool xeq, eq_ok;
+              if (eq) {
+                KstVCurvePtr vc = new KstVCurve(KST::suggestCurveName(eq->tagName(), true),
+                                      eq->vX(), eq->vY(), 0L, 0L, 0L, 0L,
+                                      KstColorSequence::next(vcurves,plot->backgroundColor()));
+                KST::dataObjectList.lock().writeLock();
+                KST::dataObjectList.append(eq.data());
+                KST::dataObjectList.append(vc.data());
+                KST::dataObjectList.lock().writeUnlock();
+                plot->addCurve(vc.data());
 
-          SetEqXRanges(args->getOption("xe"), &min, &max, &n, &xeq);
-          for (eq_i = yEqList.begin(); eq_i != yEqList.end(); ++eq_i) {
-            eqS = *eq_i;
-            ProcessEq(eqS, file, in, &eq_ok);
-            KstEquationPtr eq;
-            if (xeq) {
-              eq = new KstEquation(QString("E")+QString::number(n_eq+1)+
-                    "-" + eqS, eqS, min,max,n);
-            } else {
-              if (!xvector)
-                xvector = GetOrCreateVector(args->getOption("x"), file, in);
-
-              eq = new KstEquation(QString("E")+QString::number(n_eq+1)+eqS,
-                    eqS,
-                    KstVectorPtr(xvector),
-                    true);
-            }
-            KstVCurvePtr vc = new KstVCurve(eq->tagName() + "-curve", eq->vX(), eq->vY(), 0L, 0L, KstColorSequence::next(plot->Curves,plot->backgroundColor()));
-            KST::dataObjectList.lock().writeLock();
-            KST::dataObjectList.append(eq.data());
-            KST::dataObjectList.append(vc.data());
-            KST::dataObjectList.lock().writeUnlock();
-            plot->addCurve(vc.data());
-
-            if (in.sep_plots) {
-              i_plot++;
-              if (i_plot <in.n_plots) {
-                plot = *plist.at(i_plot);
+                if (in.sep_plots) {
+                  plot->setTagName(eq->tagName());
+                  i_plot++;
+                  if (i_plot <in.n_plots) {
+                    plot = *plist.at(i_plot);
+                  }
+                }
               }
             }
           }
+
+          if (psdList.count() > 0) { // if there are some psd plots
+            KstRVectorList rvl = kstObjectSubList<KstVector,KstRVector>(KST::vectorList);
+            for (QCStringList::ConstIterator it = psdList.begin(); it != psdList.end(); ++it) {
+
+              yvector = GetOrCreateVector(*it, file, in);
+              if (yvector) {
+                color = KstColorSequence::next(vcurves,plot->backgroundColor());
+
+                psd = new KstPSD( KST::suggestPSDName(yvector->field()),
+                    KstVectorPtr(yvector), in.rate, true, in.len,
+                    true, true, in.VUnits, in.RUnits, 0);
+                KstVCurvePtr vc = new KstVCurve(KST::suggestCurveName(psd->tagName(), true),
+                    psd->vX(), psd->vY(),
+                    0L, 0L, 0L, 0L, color);
+                if (in.has_points) {
+                  vc->setHasPoints(true);
+                  vc->setHasLines(false);
+                }
+                KST::dataObjectList.lock().writeLock();
+                KST::dataObjectList.append(psd.data());
+                KST::dataObjectList.append(vc.data());
+                KST::dataObjectList.lock().writeUnlock();
+                plot->addCurve(vc.data());
+
+                if (in.sep_plots) {
+                  plot->setTagName(psd->tagName());
+                  i_plot++;
+                  if (i_plot <in.n_plots) {
+                    plot = *plist.at(i_plot);
+                  }
+                }
+              }
+            } // next psd
+          } // end (if there are some psds)
+
+          if (hsList.count() > 0) { // if there are some histograms
+            double max, min;
+            int N;
+
+            KstRVectorList rvl = kstObjectSubList<KstVector,KstRVector>(KST::vectorList);
+            for (hs_string = hsList.begin(); hs_string != hsList.end(); ++hs_string) {
+              yvector = GetOrCreateVector(*hs_string, file, in);
+              if (yvector) {
+                color = KstColorSequence::next(vcurves,plot->backgroundColor());
+
+                KstHistogram::AutoBin(KstVectorPtr(yvector), &N, &max, &min);
+
+                hs = new KstHistogram(KST::suggestHistogramName(yvector->field()),
+                    KstVectorPtr(yvector), min, max, N, KST_HS_NUMBER);
+                KstVCurvePtr vc = new KstVCurve(KST::suggestCurveName(hs->tagName(), true),
+                    hs->vX(), hs->vY(),
+                    0L, 0L, 0L, 0L, color);
+
+                vc->setHasPoints(false);
+                vc->setHasLines(false);
+                vc->setHasBars(true);
+                vc->setBarStyle(1);
+
+                KST::dataObjectList.lock().writeLock();
+                KST::dataObjectList.append(hs.data());
+                KST::dataObjectList.append(vc.data());
+                KST::dataObjectList.lock().writeUnlock();
+                plot->addCurve(vc.data());
+
+                if (in.sep_plots) {
+                  plot->setTagName(hs->tagName());
+                  i_plot++;
+                  if (i_plot < in.n_plots) {
+                    plot = *plist.at(i_plot);
+                  }
+                }
+              }
+            } // next histogram
+          } // end (if there are some histograms)
+
+          if (matrixList.count() >0) { // if there are some matrixes
+            for (mat_i = matrixList.begin(); mat_i != matrixList.end(); ++mat_i) {
+              QString tag_name = KST::suggestMatrixName(*mat_i);
+              if (!file->isValidMatrix(*mat_i)) {
+                startupErrors.append(i18n("Failed to create matrix '%1' from file '%2'.").arg(*mat_i).arg(file->fileName()));
+              }
+              KstRMatrixPtr matrix = new KstRMatrix(file, *mat_i, tag_name, 
+                  0,0,-1,-1,false,false,0);
+                  // xStart, yStart, xNumSteps, yNumSteps, 
+                  //doAve, doSkip, skip);
+              KST::addMatrixToList(KstMatrixPtr(matrix));
+              
+              // Time to create the image from the matrix
+              tag_name = KST::suggestImageName(matrix->tagName());
+              QStringList palList = KPalette::getPaletteList();
+              QString pal;
+              if (palList.contains("IDL 13 RAINBOW")) {
+                pal = QString("IDL 13 RAINBOW");
+              } else {
+                pal = QString(*palList.at(0));
+              }
+              
+              KPalette* newPal = new KPalette(pal);
+              KstImagePtr image = new KstImage(tag_name, KstMatrixPtr(matrix), 0.0, 1.0,
+                                   true, newPal);
+              plot->addCurve(KstBaseCurvePtr(image));
+              KST::dataObjectList.lock().writeLock();
+              KST::dataObjectList.append(image.data());
+              KST::dataObjectList.lock().writeUnlock();
+              image = 0L; // drop the reference
+              
+              if (in.sep_plots) {
+                plot->setTagName(matrix->tagName());
+                i_plot++;
+                if (i_plot < in.n_plots) {
+                  plot = *plist.at(i_plot);
+                }
+              }
+            }
+          }
+        } else {
+          startupErrors.append(i18n("Failed to load file '%1'.").arg(args->arg(i_file)));
         }
-
-        if (psdList.count() > 0) { // if there are some psd plots
-          KstRVectorList rvl = kstObjectSubList<KstVector,KstRVector>(KST::vectorList);
-          for (QCStringList::ConstIterator it = psdList.begin(); it != psdList.end(); ++it) {
-
-            yvector = GetOrCreateVector(*it, file, in);
-
-            color = KstColorSequence::next(plot->Curves,plot->backgroundColor());
-
-            psd = new KstPSD( QString("P") + QString::number(1+i_curve++) + "-" + yvector->field(),
-                KstVectorPtr(yvector), in.rate, true, in.len,
-                true, true, in.VUnits, in.RUnits);
-            KstVCurvePtr vc = new KstVCurve(psd->tagName() + "-curve",
-                psd->vX(), psd->vY(),
-                0L, 0L, color);
-            if (in.has_points) {
-              vc->setHasPoints(true);
-              vc->setHasLines(false);
-            }
-            KST::dataObjectList.lock().writeLock();
-            KST::dataObjectList.append(psd.data());
-            KST::dataObjectList.append(vc.data());
-            KST::dataObjectList.lock().writeUnlock();
-            plot->addCurve(vc.data());
-
-            if (in.sep_plots) {
-              i_plot++;
-              if (i_plot <in.n_plots) {
-                plot = *plist.at(i_plot);
-              }
-            }
-          } // next psd
-        } // end (if there are some psds)
-
-        if (hsList.count() > 0) { // if there are some histograms
-          double max, min;
-          int N;
-
-          KstRVectorList rvl = kstObjectSubList<KstVector,KstRVector>(KST::vectorList);
-          for (hs_string = hsList.begin();
-              hs_string != hsList.end(); ++hs_string ) {
-            yvector = GetOrCreateVector(*hs_string, file, in);
-
-            color = KstColorSequence::next(plot->Curves,plot->backgroundColor());
-
-            KstHistogram::AutoBin(KstVectorPtr(yvector), &N, &max, &min);
-
-            hs = new KstHistogram(QString("H") +
-                QString::number(1+i_curve++)
-                + "-" + yvector->field(),
-                KstVectorPtr(yvector), min, max, N,
-                KST_HS_NUMBER);
-            KstVCurvePtr vc = new KstVCurve(hs->tagName() + "-curve",
-                hs->vX(), hs->vY(),
-                0L, 0L, color);
-
-            vc->setHasPoints(false);
-            vc->setHasLines(false);
-            vc->setHasBars(true);
-            vc->setBarStyle(1);
-
-            KST::dataObjectList.lock().writeLock();
-            KST::dataObjectList.append(hs.data());
-            KST::dataObjectList.append(vc.data());
-            KST::dataObjectList.lock().writeUnlock();
-            plot->addCurve(vc.data());
-
-            if (in.sep_plots) {
-              i_plot++;
-              if (i_plot < in.n_plots) {
-                plot = *plist.at(i_plot);
-              }
-            }
-          } // next histogram
-        } // end (if there are some histograms)
-
         handled++;
         kst->slotUpdateProgress( count, handled, creatingCurves );
       } // next data file
@@ -669,10 +735,11 @@ int main(int argc, char *argv[]) {
       kst->slotUpdateProgress( count, handled, creatingPlots );
       for (i_plot = 0; i_plot < in.n_plots; i_plot++) {
         plot = *plist.at(i_plot);
-        plot->GenerateDefaultLabels();
-        if (in.dolegend) {
-          plot->Legend->setShow(true);
-          plot->Legend->setFront(true);
+        plot->generateDefaultLabels();
+        if (plot->Curves.count() > 3 || in.dolegend) {
+          KstViewLegendPtr vl = plot->getOrCreateLegend();
+          vl->resizeFromAspect(0.1, 0.1, 0.2, 0.1);
+          vl->setBorderWidth(2);
         }
         handled++;
         kst->slotUpdateProgress( count, handled, creatingPlots );
@@ -696,6 +763,10 @@ int main(int argc, char *argv[]) {
     if (args->isSet("nq")) {
       showQuickStart = false;
     }
+    if (args->isSet("w")) {
+      showDataWizard = true;
+      showQuickStart = false;
+    }
 
     if (printfile != "<none>") {
       kst->forceUpdate();
@@ -718,15 +789,21 @@ int main(int argc, char *argv[]) {
       if (showQuickStart) {
         kst->showQuickStartDialog();
       }
+      if (showDataWizard) {
+        kst->showDataWizardWithFile(wizardfile);
+      }
+      for (size_t i = 0; i < startupErrors.size(); ++i) {
+        KstDebug::self()->log(startupErrors[i], KstDebug::Error);
+      }
+      startupErrors.clear();
     }
 
-    //
     // LEAVE THIS HERE - causes crashes otherwise!
-    //
     int rc = app.exec();
     delete kst;
     return rc;
   }
+
   return app.exec();
 }
 // vim: ts=2 sw=2 et
